#ifndef AUDIO_H
#define AUDIO_H

#include "Component.h"

namespace FMOD
{
	class System;
}

class SoundGroup;
class SoundInstance;
class Listener;

// Audiokomponente
class ENGINE_API Audio : public Component
{
	friend class Engine;
	friend class SoundGroup;
	friend class Sound;
	friend class SoundInstance;
	friend class Listener;

public:
	// liefert den Hrer
	Listener* getListener() const;

	// liefert die Master-Soundgruppe
	SoundGroup* getMasterSoundGroup() const;

	// erzeugt eine Soundgruppe
	SoundGroup* createSoundGroup( const std::string& name, SoundGroup* p_parent = 0 );

	// liefert die Soundgruppe mit dem angegebenen Namen
	SoundGroup* getSoundGroupByName( const std::string& name ) const;

	// zerstrt eine Soundgruppe
	int destroySoundGroup( const std::string& name );

	// spielt einen Sound ab
	SoundInstance* playSound( const std::string& filename, const std::string& soundGroup = "Master", const std::string& type = "Sample", const std::string& mode = "2D", float volume = 1.0f, float pitch = 1.0f );

	// spielt Hintergrundmusik ab (fadet die alte Musik aus, falls gewnscht)
	SoundInstance* playMusic( const std::string& filename, bool restartIfAlreadyPlaying = false, float fadeInSpeed = 1.0f, float fadeOutSpeed = 1.0f );

	// pausiert die Hintergrundmusik
	int pauseMusic();

	// setzt die Hintergrundmusik fort
	int resumeMusic();

	// stoppt die Hintergrundmusik (wenn gewnscht mit Fade-Out)
	int stopMusic( float fadeOutSpeed = 1.0f );

	// setzt die Soundgruppe der Hintergrundmusik
	int setMusicSoundGroup( SoundGroup* p_soundGroup );

	// liefert die Soundgruppe der Hintergrundmusik
	SoundGroup* getMusicSoundGroup() const;

	// liefert true, wenn die Audio-Komponente im "Kein Sound"-Modus luft
	bool getNoSoundMode() const;

	// liefert true, wenn die Audio-Komponente im Hardwaremodus luft
	bool getHardwareMode() const;

	// liefert die Qualittsstufe des 3D-Sounds im Softwaremodus
	unsigned int getSoftware3DQuality() const;

private:
	Audio( Engine& engine );
	~Audio();

	void onAppGotFocus();
	void onAppLostFocus();
	void onUpdate();
	void onFrameEnd();

	// das FMOD-System-Objekt
	FMOD::System* p_system;

	// kein Sound?
	bool noSound;

	// Hardware?
	bool hardware;

	// Qualittslevel des 3D-Sounds im Softwaremodus (0 bis 2)
	unsigned int software3DQuality;

	// der Hrer
	Listener* p_listener;

	// die Master-Soundgruppe
	SoundGroup* p_masterSoundGroup;

	// Musik-Soundgruppe
	SoundGroup* p_musicSoundGroup;

	// alle Soundgruppen
	std::map<std::string, SoundGroup*> soundGroups;

	// aktuelles Musikstck
	SoundInstance* p_musicInst;
};

#endif