#ifndef GAMESTATE_H
#define GAMESTATE_H

class ParameterMap;
class Engine;
class Logger;
class Video;
class Audio;
class Input;

// abstrakte Klasse fr Spielzustnde
class ENGINE_API GameState
{
public:
	GameState( Engine& engine, const std::string& stateName );
	virtual ~GameState() = 0;

	// wird beim Betreten des Spielzustands aufgerufen (Zustand wird auf den Stack gelegt)
	virtual void onEnter( const ParameterMap& params );

	// wird beim Verlassen des Spielzustands aufgerufen (Entfernen vom Stack)
	virtual void onLeave();

	// wird aufgerufen, wenn der Spielzustand wieder aktiv wird, weil der darberliegende entfernt wurde
	virtual void onActivate();

	// wird aufgerufen, wenn der Spielzustand deaktiviert wird, weil ein anderer darber gelegt wurde
	virtual void onDeactivate();

	// bewegt den Spielzustand fort
	virtual void update();

	// rendert den Spielzustand
	virtual void render();

	// liefert das Engine-Objekt, zu dem dieses Objekt gehrt
	Engine& getEngine() const;

protected:
	// Es folgen Referenzen auf die wichtigsten Objekte zum einfacheren Zugriff auf die Engine.

	Engine& engine;
	Logger& logger;
	Video& video;
	Audio& audio;
	Input& input;

private:
	const std::string name;
};

#endif