#ifndef LOGGER_H
#define LOGGER_H

// Log-Stufen
enum LogLevel
{
	LL_ERROR,
	LL_WARNING,
	LL_INFO,
	LL_FINE,
	LL_FINER,
	LL_FINEST,

	LL_MAX_VALUE
};

// Dieses Makro kann anstelle der Parameter "filename" und "line" angegeben werden und erzeugt sie automatisch.
#define LOGGER_AUTO __FILE__, __LINE__

class LogHandler;

class ENGINE_API Logger
{
	friend class LogHandler;

public:
	Logger();
	~Logger();
	
	// alle LogHandler darber informieren, dass sie ihre Ausgabe flushen sollen
	void flush();
	
	// schreibt einen Log-Eintrag
	void log( LogLevel level, const std::string& text, const std::string& filename, unsigned int line );

	// Hilfsmethoden fr mehr Bequemlichkeit
	void error( const std::string& text, const std::string& filename, unsigned int line );
	void warning( const std::string& text, const std::string& filename, unsigned int line );
	void info( const std::string& text, const std::string& filename, unsigned int line );
	void fine( const std::string& text, const std::string& filename, unsigned int line );
	void finer( const std::string& text, const std::string& filename, unsigned int line );
	void finest( const std::string& text, const std::string& filename, unsigned int line );

	// setzt den Log-Level
	void setLogLevel( LogLevel level );

	// liefert den Log-Level zurck
	LogLevel getLogLevel() const;

private:
	// fgt den bergebenen LogHandler zum Logger hinzu
	void addLogHandler( LogHandler* p_handler );

	// entfernt den LogHandler aus dem Logger
	void removeLogHandler( LogHandler* p_handler );

	std::list<LogHandler*> logHandlers;
	LogLevel logLevel;
};

#endif