#ifndef MAT44_H
#define MAT44_H

#include "Vec.h"

// 4x4-Matrixklasse
template<typename T> class Mat44
{
public:
	Mat44()
	{
	}

	Mat44( const Mat44& rhs )
	{
		for( int i = 0; i < 16; i++ )
		{
			value[i] = rhs.value[i];
		}
	}

	Mat44( const T* p_values )
	{
		for( int i = 0; i < 16; i++ )
		{
			value[i] = p_values[i];
		}
	}

	~Mat44()
	{
	}

	T determinant() const
	{
		return m11 * ( m22 * ( m33 * m44 - m34 * m43 ) + m23 * ( m34 * m42 - m32 * m44 ) + m24 * ( m32 * m43 - m33 * m42 ) ) -
			   m12 * ( m21 * ( m33 * m44 - m34 * m43 ) + m23 * ( m34 * m41 - m31 * m44 ) + m24 * ( m31 * m43 - m33 * m41 ) ) +
			   m13 * ( m21 * ( m32 * m44 - m34 * m42 ) + m22 * ( m34 * m41 - m31 * m44 ) + m24 * ( m31 * m42 - m32 * m41 ) ) -
			   m14 * ( m21 * ( m32 * m43 - m33 * m42 ) + m22 * ( m33 * m41 - m31 * m43 ) + m23 * ( m31 * m42 - m32 * m41 ) );
	}

	Mat44 inverse() const
	{
		Mat44 r;
		r.m11 =    m22 * ( m33 * m44 - m34 * m43 ) + m23 * ( m34 * m42 - m32 * m44 ) + m24 * ( m32 * m43 - m33 * m42 );
		r.m12 = -( m12 * ( m33 * m44 - m34 * m43 ) + m13 * ( m34 * m42 - m32 * m44 ) + m14 * ( m32 * m43 - m33 * m42 ) );
		r.m13 =    m12 * ( m23 * m44 - m24 * m43 ) + m13 * ( m24 * m42 - m22 * m44 ) + m14 * ( m22 * m43 - m23 * m42 );
		r.m14 = -( m12 * ( m23 * m34 - m24 * m33 ) + m13 * ( m24 * m32 - m22 * m34 ) + m14 * ( m22 * m33 - m23 * m32 ) );
		r.m21 = -( m21 * ( m33 * m44 - m34 * m43 ) + m23 * ( m34 * m41 - m31 * m44 ) + m24 * ( m31 * m43 - m33 * m41 ) );
		r.m22 =    m11 * ( m33 * m44 - m34 * m43 ) + m13 * ( m34 * m41 - m31 * m44 ) + m14 * ( m31 * m43 - m33 * m41 );
		r.m23 = -( m11 * ( m23 * m44 - m24 * m43 ) + m13 * ( m24 * m41 - m21 * m44 ) + m14 * ( m21 * m43 - m23 * m41 ) );
		r.m24 =    m11 * ( m23 * m34 - m24 * m33 ) + m13 * ( m24 * m31 - m21 * m34 ) + m14 * ( m21 * m33 - m23 * m31 );
		r.m31 =    m21 * ( m32 * m44 - m34 * m42 ) + m22 * ( m34 * m41 - m31 * m44 ) + m24 * ( m31 * m42 - m32 * m41 );
		r.m32 = -( m11 * ( m32 * m44 - m34 * m42 ) + m12 * ( m34 * m41 - m31 * m44 ) + m14 * ( m31 * m42 - m32 * m41 ) );
		r.m33 =    m11 * ( m22 * m44 - m24 * m42 ) + m12 * ( m24 * m41 - m21 * m44 ) + m14 * ( m21 * m42 - m22 * m41 );
		r.m34 = -( m11 * ( m22 * m34 - m24 * m32 ) + m12 * ( m24 * m31 - m21 * m34 ) + m14 * ( m21 * m32 - m22 * m31 ) );
		r.m41 = -( m21 * ( m32 * m43 - m33 * m42 ) + m22 * ( m33 * m41 - m31 * m43 ) + m23 * ( m31 * m42 - m32 * m41 ) );
		r.m42 =    m11 * ( m32 * m43 - m33 * m42 ) + m12 * ( m33 * m41 - m31 * m43 ) + m13 * ( m31 * m42 - m32 * m41 );
		r.m43 = -( m11 * ( m22 * m43 - m23 * m42 ) + m12 * ( m23 * m41 - m21 * m43 ) + m13 * ( m21 * m42 - m22 * m41 ) );
		r.m44 =    m11 * ( m22 * m33 - m23 * m32 ) + m12 * ( m23 * m31 - m21 * m33 ) + m13 * ( m21 * m32 - m22 * m31 );
		return r / determinant();
	}

	Mat44& invert()
	{
		return *this = inverse();
	}

	Mat44 transposed() const
	{
		Mat44 r;
		for( int i = 0; i < 4; i++ )
		{
			for( int j = 0; j < 4; j++ )
			{
				r.m[i][j] = m[j][i];
			}
		}

		return r;
	}

	Mat44& transpose()
	{
		return *this = transposed();
	}

	// transformiert einen 3D-Positionsvektor, ohne Projektion
	Vec<3, T> transformPosition( const Vec<3, T>& rhs ) const
	{
		return Vec<3, T>( m11 * rhs.x + m12 * rhs.y + m13 * rhs.z + m14,
						  m21 * rhs.x + m22 * rhs.y + m23 * rhs.z + m24,
						  m31 * rhs.x + m32 * rhs.y + m33 * rhs.z + m34 );
	}

	// transformiert einen 3D-Richtungsvektor
	Vec<3, T> transformDirection( const Vec<3, T>& rhs ) const
	{
		return Vec<3, T>( m11 * rhs.x + m12 * rhs.y + m13 * rhs.z,
						  m21 * rhs.x + m22 * rhs.y + m23 * rhs.z,
						  m31 * rhs.x + m32 * rhs.y + m33 * rhs.z );
	}

	// liefert die n-te Achse
	Vec<3, T> getAxis( unsigned int n ) const
	{
		return Vec<3, T>( m[n][0], m[n][1], m[n][2] );
	}

	// liefert die Identittsmatrix
	static Mat44 identity()
	{
		Mat44 r;
		r.m11 = r.m22 = r.m33 = r.m44 = static_cast<T>( 1 );
		r.m12 = r.m13 = r.m14 = r.m21 = r.m23 = r.m24 = r.m31 = r.m32 = r.m34 = r.m41 = r.m42 = r.m43 = static_cast<T>( 0 );
		return r;
	}

	const T& operator () ( unsigned int row,
						   unsigned int col ) const
	{
		return m[row][col];
	}

	T& operator () ( unsigned int row,
					 unsigned int col )
	{
		return m[row][col];
	}

	Mat44 operator + ( const Mat44& rhs ) const
	{
		Mat44 r;
		for(int i = 0; i < 16; i++)
		{
			r.value[i] = value[i] + rhs.value[i];
		}

		return r;
	}

	Mat44 operator - ( const Mat44& rhs ) const
	{
		Mat44 r;
		for( int i = 0; i < 16; i++ )
		{
			r.value[i] = value[i] - rhs.value[i];
		}

		return r;
	}

	Mat44 operator - () const
	{
		Mat44 r;
		for( int i = 0; i < 16; i++ )
		{
			r.value[i] = -value[i];
		}

		return r;
	}

	Mat44 operator * ( const Mat44& rhs ) const
	{
		Mat44 r;
		for( int i = 0; i < 4; i++ )
		{
			for( int j = 0; j < 4; j++ )
			{
				r.m[i][j] = m[i][0] * rhs.m[0][j] +
					        m[i][1] * rhs.m[1][j] +
							m[i][2] * rhs.m[2][j] +
							m[i][3] * rhs.m[3][j];
			}
		}

		return r;
	}

	Mat44 operator * ( T rhs ) const
	{
		Mat44 r;
		for( int i = 0; i < 16; i++ )
		{
			r.value[i] = value[i] * rhs;
		}

		return r;
	}

	// Transformation eines 4D-Vektors
	Vec<4, T> operator * ( const Vec<4, T>& rhs ) const
	{
		return Vec<4, T>( m11 * rhs.x + m12 * rhs.y + m13 * rhs.z + m14 * rhs.w,
						  m21 * rhs.x + m22 * rhs.y + m23 * rhs.z + m24 * rhs.w,
						  m31 * rhs.x + m32 * rhs.y + m33 * rhs.z + m34 * rhs.w,
						  m41 * rhs.x + m42 * rhs.y + m43 * rhs.z + m44 * rhs.w );
	}

	Mat44 operator / ( T rhs ) const
	{
		Mat44 r;
		for( int i = 0; i < 16; i++ )
		{
			r.value[i] = value[i] / rhs;
		}

		return r;
	}

	Mat44& operator = ( const Mat44& rhs )
	{
		for( int i = 0; i < 16; i++ )
		{
			value[i] = rhs.value[i];
		}

		return *this;
	}

	Mat44& operator += ( const Mat44& rhs )
	{
		return *this = *this + rhs;
	}

	Mat44& operator -= ( const Mat44& rhs )
	{
		return *this = *this - rhs;
	}

	Mat44& operator *= ( const Mat44& rhs )
	{
		return *this = *this * rhs;
	}

	Mat44& operator *= (T rhs)
	{
		return *this = *this * rhs;
	}

	Mat44& operator /= ( T rhs )
	{
		return *this = *this / rhs;
	}

	bool operator == ( const Mat44& rhs ) const
	{
		for( int i = 0; i < 16; i++ )
		{
			if( value[i] != rhs.value[i] ) return false;
		}

		return true;
	}

	bool operator != ( const Mat44& rhs ) const
	{
		return !( *this == rhs );
	}

	// Konvertierung in anderes Matrixformat
	template<typename U> operator Mat44<U> () const
	{
		Mat44<U> r;
		for( int i = 0; i < 16; i++ )
		{
			r.value[i] = static_cast<U>( value[i] );
		}

		return r;
	}

	operator const T* () const
	{
		return value;
	}

	operator T* ()
	{
		return value;
	}

	// die Elemente
	union
	{
		T m[4][4];
		T value[16];

		struct
		{
			T m11;
			T m12;
			T m13;
			T m14;
			T m21;
			T m22;
			T m23;
			T m24;
			T m31;
			T m32;
			T m33;
			T m34;
			T m41;
			T m42;
			T m43;
			T m44;
		};
	};
};

// Multipliktion mit Skalar von links
template<typename T> inline Mat44<T> operator * ( T lhs,
												  const Mat44<T>& rhs )
{
	return rhs * lhs;
}

// in Stream schreiben
template<typename T> inline std::ostream& operator << ( std::ostream& lhs,
														const Mat44<T>& rhs )
{
	for( int i = 0; i < 16; i++ )
	{
		lhs << rhs.value[i];
		if( i != 15 ) lhs << " ";
	}

	return lhs;
}

// aus Stream lesen
template<typename T> inline std::istream& operator >> ( std::istream& lhs,
														Mat44<T>& rhs )
{
	for( int i = 0; i < 16; i++ )
	{
		lhs >> rhs.value[i];
	}

	return lhs;
}

// Standard-Matrixtypen
typedef Mat44<int> Mat44i;
typedef Mat44<float> Mat44f;
typedef Mat44<double> Mat44d;

#endif