#ifndef PARAMETERMAP_H
#define PARAMETERMAP_H

#include "BasicVariant.h"
#include "Serializer.h"

// verwaltet Parameter beliebigen Typs
class ENGINE_API ParameterMap
{
public:
	ParameterMap();
	ParameterMap( const ParameterMap& copy );
	~ParameterMap();

	// leert die Map
	void clear();

	// liefert true, falls die Map den angegebenen Key enthlt
	bool has( const std::string& key ) const;

	// liefert den Parameter mit dem angegebenen Key zurck
	template<typename T> T get( const std::string& key ) const
	{
		assert( has( key ) );

		std::map<std::string, Parameter>::const_iterator it = parameters.find( key );
		BasicVariant<T>* p_value = dynamic_cast<BasicVariant<T>*>( it->second.p_value );
		assert( p_value );

		return p_value->getValue();
	}

	// liefert den Parameter mit dem angegenen Key zurck, falls er existiert, ansonsten einen Standardwert
	template<typename T> T get( const std::string& key,
								const T& defaultValue ) const
	{
		if( has( key ) ) return get<T>( key );
		else return defaultValue;
	}

	// setzt einen Parameter
	template<typename T> void set( const std::string& key,
								   const T& value )
	{
		// Wert eintragen
		parameters[key] = Parameter( key, new BasicVariant<T>( value ) );
	}

	// liefert true, wenn die Map leer ist
	bool isEmpty() const;

	void serialize( SerializationStream& target ) const;
	void deserialize( const SerializationStream& source );

	ParameterMap operator + ( const ParameterMap& rhs ) const;
	ParameterMap& operator = ( const ParameterMap& rhs );
	ParameterMap& operator += ( const ParameterMap& rhs );

	bool operator == ( const ParameterMap& rhs ) const;
	bool operator != ( const ParameterMap& rhs ) const;

private:
	struct Parameter
	{
		Parameter() : p_value( 0 )
		{
		}

		Parameter( const Parameter& copy )
		{
			*this = copy;
		}

		Parameter( const std::string& key,
				   Variant* p_value ) : key( key ),
										p_value( p_value )
		{
		}

		~Parameter()
		{
			delete p_value;
		}

		Parameter& operator = ( const Parameter& rhs )
		{
			key = rhs.key;
			p_value = rhs.p_value ? rhs.p_value->copy() : 0;
			return *this;
		}

		bool operator == ( const Parameter& rhs ) const
		{
			return key == rhs.key && p_value->isEqualTo( rhs.p_value );
		}

		bool operator != ( const Parameter& rhs ) const
		{
			return !( *this == rhs );
		}

		std::string key;
		Variant* p_value;
	};

	std::map<std::string, Parameter> parameters;
};

#endif