#ifndef PARTICLEEMITTER_H
#define PARTICLEEMITTER_H

#include "Base.h"
#include "Placeable.h"

class ParticleSystem;

class ENGINE_API ParticleEmitter : public Placeable
{
	friend class ParticleSystem;

public:
	// setzt die restliche Lebenszeit des Emitters (null: endlos)
	void setTimeToLive( float timeToLive );

	// liefert die restliche Lebenszeit des Emitters
	float getTimeToLive() const;

	// setzt die Startverzgerung des Emitters
	void setStartDelay( float startDelay );

	// liefert die Startverzgerung des Emitters
	float getStartDelay() const;

	// liefert das Alter des Emitters
	float getAge() const;

	// setzt die relative Partikelrate
	void setRate( float rate );

	// liefert die relative Partikelrate
	float getRate() const;

	// setzt die maximale Anzahl von Partikeln, die emittiert werden sollen
	void setMaxNumParticles( unsigned int maxNumParticles );

	// liefert die maximale Anzahl von Partikeln, die emittiert werden sollen
	unsigned int getMaxNumParticles() const;

	// legt das Verhalten des Emitters fest
	void setVisibilityBehavior( bool emitWhenInvisible, float visibilityRadius );

	// liefert das Partikelsystem, zu dem der Emitter gehrt
	ParticleSystem& getParticleSystem() const;

protected:
	ParticleEmitter( ParticleSystem& ps );
	virtual ~ParticleEmitter();

	// liefert true, wenn der Emitter gelscht werden soll
	virtual bool isToBeDeleted() const;

	// aktualisiert den Emitter
	void update();

	// lsst den Emitter seine Arbeit tun
	virtual unsigned int doWork( unsigned int maxNumParticles ) = 0;

private:
	// das Partikelsystem
	ParticleSystem& ps;

	// restliche Lebenszeit des Emitters
	float timeToLive;

	// Startverzgerung
	float startDelay;

	// Alter des Emitters
	float age;

	// Rate
	float rate;

	// maximale Anzahl von Partikeln
	unsigned int maxNumParticles;

	// schon emittierte Partikel
	unsigned int numParticlesEmitted;

	// Partikel emittieren, wenn Emitter unsichtbar?
	bool emitWhenInvisible;

	// Sichtbarkeitsradius
	float visibilityRadius;
};

#endif