#ifndef PLACEABLE_H
#define PLACEABLE_H

class ENGINE_API Placeable
{
public:
	Placeable();
	virtual ~Placeable();

	void attachTo( Placeable* p_parent, bool maintainAbsTransform = true );
	void detachFromParent( bool maintainAbsTransform = true );
	void attachChild( Placeable* p_child, bool maintainAbsTransform = true );
	void detachChild( Placeable* p_child, bool maintainAbsTransform = true );
	Placeable* getParent() const;
	const std::set<Placeable*>& getChildren() const;

	void setPosition( const Vec2f& position );
	Vec2f getPosition() const;
	void setAbsPosition( const Vec2f& position );
	Vec2f getAbsPosition() const;

	void setAngle( float angle );
	float getAngle() const;
	void setAbsAngle( float angle );
	float getAbsAngle() const;

	void setDirection( const Vec2f& direction );
	Vec2f getDirection() const;
	void setAbsDirection( const Vec2f& direction );
	Vec2f getAbsDirection() const;

	virtual void setTransform( const Mat33f& transform );
	virtual Mat33f getTransform() const;
	void setAbsTransform( const Mat33f& transform );
	Mat33f getAbsTransform() const;

	const Mat33f& getInvTransform() const;
	Mat33f getInvAbsTransform() const;

	Vec2f relPositionToAbsPosition( const Vec2f& position ) const;
	Vec2f absPositionToRelPosition( const Vec2f& position ) const;
	float relAngleToAbsAngle( float angle ) const;
	float absAngleToRelAngle( float angle ) const;
	Vec2f relDirectionToAbsDirection( const Vec2f& direction ) const;
	Vec2f absDirectionToRelDirection( const Vec2f& direction ) const;
	Mat33f relTransformToAbsTransform( const Mat33f& transform ) const;
	Mat33f absTransformToRelTransform( const Mat33f& transform ) const;

	// setzt die OpenGL-Matrix
	void setupGLMatrix();

private:
	void updateInvTransform() const;

	Placeable* p_parent;
	std::set<Placeable*> children;

	Mat33f transform;
	mutable Mat33f invTransform;
	mutable bool invTransformDirty;
};

#endif