#ifndef PROFILER_H
#define PROFILER_H

#ifndef NO_PROFILING
#define SETUP_PROFILE( NAME, DNAME, PARENT )	{ Profiler::inst().setupProfile( ( NAME ), ( DNAME ), ( PARENT ) ); }
#define PROFILE_BEGIN( NAME )					{ Profiler::inst().begin( ( NAME ) ); }
#define PROFILE_END()							{ Profiler::inst().end(); }
#else
#define SETUP_PROFILE( NAME, DNAME, PARENT )	{ }
#define PROFILE_BEGIN( NAME )					{ }
#define PROFILE_END()							{ }
#endif

struct ProfileInfo
{
	ProfileInfo() : profiled( false ),
					numSamples( 0 ),
					currentTicksConsumed( 0 ),
					totalTicksConsumed( 0 ),
					currentTimeConsumed( 0.0 ),
					totalTimeConsumed( 0.0 ),
					p_parent( 0 )
	{
	}

	std::string name;
	std::string displayedName;

	bool profiled;
	unsigned int numSamples;

	unsigned long long currentTicksConsumed;
	unsigned long long totalTicksConsumed;
	double currentTimeConsumed;
	double totalTimeConsumed;

	ProfileInfo* p_parent;
	std::vector<ProfileInfo*> children;
};

class ENGINE_API Profiler
{
public:
	static Profiler& inst();

	void setUpdateCycle( unsigned int updateCycle );
	void setupProfile( const std::string& name, const std::string& displayedName = "", const std::string& parentName = "" );
	void begin( const std::string& name );
	void end();
	void updateStatistics();
	std::string getStatistics();
	void reset();

	unsigned long long getTimer() const;
	unsigned long long getTimerResolution() const;

private:
	Profiler();
	Profiler( const Profiler& copy );
	~Profiler();

	std::string buildStatistics( ProfileInfo* p_profile, int indent );

	struct RunningProfile
	{
		ProfileInfo* p_profile;
		unsigned long long startTime;
	};

	unsigned int updateCycle;
	ProfileInfo* p_root;
	std::map<std::string, ProfileInfo*> profiles;
	std::stack<RunningProfile> running;
	unsigned long long timerResolution;
	long double oneOverTimerResolution;
	unsigned int lastUpdate;
	unsigned int numUpdates;
	std::string statistics;
};

#endif