#ifndef SIMPLEEMITTER_H
#define SIMPLEEMITTER_H

#include "ParticleEmitter.h"

struct SimpleEmitterSettings
{
	SimpleEmitterSettings() : partTimeToLive( 1.0f ), partTimeToLiveVar( 0.0f ),
							  partPositionRadius( 0.0f ),
							  partDirection( 0.0f ), partDirectionVar( 6.2832f ),
							  partSpeed( 100.0f ), partSpeedVar( 20.0f ),
							  partAngle( 0.0f ), partAngleVar( 6.2832f ),
							  partAngularVelocity( 0.0f ), partAngularVelocityVar( 6.2832f ),
							  partStartColor( 1.0f ), partStartColorVar( 0.0f ),
							  partEndColor( 0.0f ), partEndColorVar( 0.0f ),
							  partStartSize( 10.0f ), partStartSizeVar( 0.0f ),
							  partEndSize( 0.0f ), partEndSizeVar( 0.0f ),
							  partFriction( 1.0f ), partFrictionVar( 0.0f ),
							  partGravity( 0.0f ), partGravityVar( 0.0f ),
							  partAnimFrame( 0.0f ), partAnimFrameVar( 0.0f ),
							  useRelativeVelocity( false )
	{
	}

	std::vector<unsigned int> partStyles;
	float partTimeToLive, partTimeToLiveVar;
	float partPositionRadius;
	float partDirection, partDirectionVar;
	float partSpeed, partSpeedVar;
	float partAngle, partAngleVar;
	float partAngularVelocity, partAngularVelocityVar;
	Vec4f partStartColor, partStartColorVar;
	Vec4f partEndColor, partEndColorVar;
	float partStartSize, partStartSizeVar;
	float partEndSize, partEndSizeVar;
	float partFriction, partFrictionVar;
	float partGravity, partGravityVar;
	float partAnimFrame, partAnimFrameVar;
	bool useRelativeVelocity;
};

class ENGINE_API SimpleEmitter : public ParticleEmitter
{
	friend class ParticleSystem;

public:
	void setSettings( const SimpleEmitterSettings& settings );
	const SimpleEmitterSettings& getSettings() const;

	void addParticleStyle( unsigned int style );
	void setParticleTimeToLive( float timeToLive, float variation = 0.0f );
	void setParticlePositionRadius( float radius = 0.0f );
	void setParticleDirection( const Vec2f& direction, float variation = 0.0f );
	void setParticleSpeed( float speed, float variation = 0.0f );
	void setParticleAngle( float angle, float variation = 0.0f );
	void setParticleAngularVelocity( float angularVelocity, float variation = 0.0f );
	void setParticleStartColor( const Vec4f& startColor, const Vec4f& variation = Vec4f( 0.0f ) );
	void setParticleEndColor( const Vec4f& endColor, const Vec4f& variation = Vec4f( 0.0f ) );
	void setParticleStartSize( float startSize, float variation = 0.0f );
	void setParticleEndSize( float endSize, float variation = 0.0f );
	void setParticleFriction( float friction, float variation = 0.0f );
	void setParticleGravity( float gravity, float variation = 0.0f );
	void setParticleAnimFrame( float animFrame, float variation = 0.0f );
	void setUseRelativeVelocity( bool useRelativeVelocity );

	unsigned int doWork( unsigned int maxNumParticles );

private:
	SimpleEmitter( ParticleSystem& ps );
	~SimpleEmitter();

	float particlesToEmit;
	Vec2f lastPosition;
	float lastAngle;

	SimpleEmitterSettings settings;
};

#endif