#ifndef SOUND_H
#define SOUND_H

#include "Resource.h"
#include "BasicResourceHandle.h"
#include "BasicResourceManager.h"

class Audio;
class SoundInstance;
class SoundGroup;

namespace FMOD
{
	class Sound;
}

class ENGINE_API Sound : public Resource
{
	friend class SoundManager;
	friend class BasicResourceManager<Sound>;
	friend class SoundInstance;

public:
	// erzeugt eine neue Instanz dieses Sounds
	SoundInstance* createInstance( bool forceCreation = false );

	// prft, ob die Sound-Instanz gltig ist
	bool isInstanceValid( SoundInstance* p_soundInstance ) const;

	// schaut nach, ob Instanzen bereit zum Lschen sind
	void update();

	// zerstrt alle Instanzen
	void destroyAll();

	// setzt die Standardsoundgruppe fr neu erzeugte Instanzen
	void setDefaultSoundGroup( SoundGroup* p_soundGroup );

	// setzt die Standardsoundgruppe fr neu erzeuget Instanzen per Name
	void setDefaultSoundGroupName( const std::string& soundGroup );

	// liefert die Standardsoundgruppe
	SoundGroup* getDefaultSoundGroup() const;

	// setzt die Standard-3D-Entfernungen fr neu erzeugte Instanzen
	int setDefaultDistances( float minDist, float maxDist );

	// liefert die Standard-3D-Entfernungen
	int getDefaultDistances( float* p_minDistOut, float* p_maxDistOut ) const;

	// setzt die Mindestzeit zwischen dem Erzeugen zweier Instanzen
	void setInstanceCreationInterval( float instanceCreationInterval );

	// liefert die Mindestzeit zwischen dem Erzeugen zweier Instanzen
	float getInstanceCreationInterval() const;

	// setzt die maximale Anzahl gleichzeitig abspielender Instanzen
	void setMaxPlayingInstances( unsigned int maxPlayingInstances );

	// liefert die maximale Anzahl gleichzeitig abspielender Instanzen
	unsigned int getMaxPlayingInstances() const;

	// liefert die native Frequenz des Sounds
	float getNativeFrequency() const;

	// liefert die Anzahl der gerade existierenden Instanzen
	unsigned int getNumInstances() const;

private:
	Sound( ResourceManager& manager, const std::string& filename, bool unique, const ParameterMap& options );
	~Sound();

	Audio& audio;
	FMOD::Sound* p_sound;
	std::set<SoundInstance*> instances;
	SoundGroup* p_defaultSoundGroup;
	float instanceCreationInterval;
	float lastInstanceCreation;
	unsigned int maxPlayingInstances;
	float nativeFrequency;
};

typedef BasicResourceHandle<Sound> SoundHandle;

#endif