#ifndef SOUNDGROUP_H
#define SOUNDGROUP_H

#include "SoundControllerTarget.h"

class Audio;

namespace FMOD
{
	class ChannelGroup;
}

class ENGINE_API SoundGroup : public Base, public SoundControllerTarget
{
	friend class Audio;
	friend class SoundInstance;

public:
	int stop();

	int setPaused( bool paused );
	int setMuted( bool muted );
	int setVolume( float volume );
	int setPitch( float pitch );

	bool isPaused() const;
	bool isMuted() const;
	float getVolume() const;
	float getPitch() const;

	const Audio& getAudio() const;
	const std::string& getName() const;
	SoundGroup* getParent() const;

private:
	SoundGroup( Audio& audio, FMOD::ChannelGroup* p_masterGroup );
	SoundGroup( Audio& audio, const std::string& name, SoundGroup* p_parent );
	~SoundGroup();

	// berechnet die eigenen Sound-Instanzen und die der Kindergruppen neu
	void evaluateAll();

	// das Audio-Objekt
	Audio& audio;

	// die FMOD-Channel-Group
	FMOD::ChannelGroup* p_channelGroup;

	// Name
	std::string name;

	// Elterngruppe
	SoundGroup* p_parent;

	// Kinder
	std::set<SoundGroup*> children;

	// Sound-Instanzen, die zu dieser Gruppe gehren
	std::set<SoundInstance*> soundInstances;

	// Ist dies die Master-Gruppe?
	bool isMaster;

	// Flags (Bit 1: Pausiert, Bit 2: Stumm)
	unsigned int flags;
};

#endif