#ifndef SOUNDINSTANCE_H
#define SOUNDINSTANCE_H

#include "Sound.h"
#include "SoundControllerTarget.h"
#include "Placeable.h"

class Audio;
class Sound;
class SoundGroup;

namespace FMOD
{
	class Channel;
}

class ENGINE_API SoundInstance : public Base, public SoundControllerTarget, public Placeable
{
	friend class Sound;
	friend class SoundGroup;

public:
	void update();

	int play();
	int play( bool looping );
	int stop();

	int setLooping( bool looping );
	int setPaused( bool paused );
	int setMuted( bool muted );
	int setVolume( float volume );
	int setPitch( float pitch );
	int setPlayCursor( float seconds );

	int setSoundGroup( SoundGroup* p_newSoundGroup );
	int setDistances( float minDist, float maxDist );

	bool isLooping() const;
	bool isPaused() const;
	bool isMuted() const;
	bool isPlaying() const;

	float getVolume() const;
	float getPitch() const;
	float getPlayCursor() const;

	SoundGroup* getSoundGroup() const;
	int getDistances( float* p_minDistOut, float* p_maxDistOut ) const;

	void setZ( float z );
	float getZ() const;

	void setCalculateVelocity( bool calculateVelocity );
	bool getCalculateVelocity() const;

	SoundHandle getSound() const;

private:
	SoundInstance( Audio& audio, Sound& sound );
	~SoundInstance();

	// liefert true, wenn die Instanz gelscht werden soll
	bool isToBeDeleted() const;

	// berechnet neu, ob der Sound pausiert oder stumm ist
	void evaluate();

	Audio& audio;
	SoundHandle sound;
	FMOD::Channel* p_channel;
	SoundGroup* p_soundGroup;
	unsigned int flags;
	float z;
	bool calculateVelocity;
	Vec2f lastPosition;
};

#endif