#include <PCH.h>
#include <Listener.h>
#include <Engine.h>
#include <Audio.h>
#include <fmod.hpp>
#include <fmod_errors.h>

Listener::Listener( Audio& audio ) : audio( audio ),
									 z( 0.0f ),
									 lastPosition( FLT_MAX, FLT_MAX )
{
}

Listener::~Listener()
{
}

void Listener::setZ( float z )
{
	this->z = z;
}

float Listener::getZ() const
{
	return z;
}

void Listener::update()
{
	// aktuelle Position und Geschwindigkeit berechnen
	const Vec2f currentPosition( getAbsPosition() );
	if( lastPosition.x == FLT_MAX ) lastPosition = currentPosition;
	const Vec2f currentVelocity( ( currentPosition - lastPosition ) / audio.getEngine().getTickDurationInSeconds() );

	// an FMOD bermitteln
	FMOD_VECTOR pos = { currentPosition.x, currentPosition.y, z };
	FMOD_VECTOR vel = { currentVelocity.x, currentVelocity.y, 0.0f };
	audio.p_system->set3DListenerAttributes( 0, &pos, &vel, 0, 0 );

	lastPosition = currentPosition;
}