#include <PCH.h>
#include <Logger.h>
#include <LogHandler.h>

Logger::Logger() : logLevel( LL_MAX_VALUE )
{
}

Logger::~Logger()
{
	flush();
}

void Logger::flush()
{
	std::list<LogHandler*>::const_iterator it = logHandlers.begin();
	while( it != logHandlers.end() )
	{
		(*it)->flush();
		it++;
	}
}

void Logger::log( LogLevel level,
				  const std::string& text,
				  const std::string& filename,
				  unsigned int line )
{
	// Wenn dieser Level zu hoch ist, wird der Eintrag ignoriert.
	if( level > logLevel ) return;

	// alle Log-Handler durchgehen
	std::list<LogHandler*>::const_iterator it = logHandlers.begin();
	while( it != logHandlers.end() )
	{
		(*it)->log( level, text, filename, line );
		it++;
	}
}

void Logger::error( const std::string& text,
					const std::string& filename,
					unsigned int line )
{
	log( LL_ERROR, text, filename, line );
}

void Logger::warning( const std::string& text,
					  const std::string& filename,
					  unsigned int line )
{
	log( LL_WARNING, text, filename, line );
}

void Logger::info( const std::string& text,
				   const std::string& filename,
				   unsigned int line )
{
	log( LL_INFO, text, filename, line );
}

void Logger::fine( const std::string& text,
				   const std::string& filename,
				   unsigned int line )
{
	log( LL_FINE, text, filename, line );
}

void Logger::finer( const std::string& text,
					const std::string& filename,
					unsigned int line )
{
	log( LL_FINER, text, filename, line );
}

void Logger::finest( const std::string& text,
					 const std::string& filename,
					 unsigned int line )
{
	log( LL_FINEST, text, filename, line );
}

void Logger::setLogLevel( LogLevel level )
{
	assert( level >= 0 && level < LL_MAX_VALUE );
	logLevel = level;
}

LogLevel Logger::getLogLevel() const
{
	return logLevel;
}

void Logger::addLogHandler( LogHandler* p_handler )
{
	logHandlers.push_back( p_handler );
}

void Logger::removeLogHandler( LogHandler* p_handler )
{
	logHandlers.remove( p_handler );
}