#include <PCH.h>
#include <ParameterMap.h>
#include <tinyxml.h>

ParameterMap::ParameterMap()
{
}

ParameterMap::ParameterMap( const ParameterMap& copy )
{
	*this = copy;
}

ParameterMap::~ParameterMap()
{
	clear();
}

void ParameterMap::clear()
{
	// Map leeren
	parameters.clear();
}

bool ParameterMap::has( const std::string& key ) const
{
	// prfen, ob dieser Key in der Map enthalten ist
	std::map<std::string, Parameter>::const_iterator it = parameters.find( key );
	return it != parameters.end();
}

bool ParameterMap::isEmpty() const
{
	return parameters.empty();
}

void ParameterMap::serialize( SerializationStream& target ) const
{
	// alle Parameter serialisieren
	for( std::map<std::string, Parameter>::const_iterator it = parameters.begin();
		 it != parameters.end();
		 it++ )
	{
		Serializer::inst().serialize( it->second.key, target );
		it->second.p_value->serialize( target );
	}
}

void ParameterMap::deserialize( const SerializationStream& source )
{
	clear();

	while( true )
	{
		// Key holen
		std::string key;
		if( Serializer::inst().deserialize( key, source ) )
		{
			// Das war das Ende!
			break;
		}

		// Wert holen
		Variant* p_value = Serializer::inst().deserialize( source );

		parameters[key] = Parameter( key, p_value );
	}
}

ParameterMap ParameterMap::operator + ( const ParameterMap& rhs ) const
{
	// Kopie anfertigen
	ParameterMap result( *this );

	// alle Parameter der zweiten Map hinzufgen
	for( std::map<std::string, Parameter>::const_iterator it = rhs.parameters.begin();
		 it != rhs.parameters.end();
		 it++ )
	{
		result.parameters[it->first] = it->second;
	}

	return result;
}

ParameterMap& ParameterMap::operator = ( const ParameterMap& rhs )
{
	clear();

	// alle Parameter aus der anderen Map kopieren
	for( std::map<std::string, Parameter>::const_iterator it = rhs.parameters.begin();
		 it != rhs.parameters.end();
		 it++ )
	{
		parameters[it->first] = it->second;
	}

	return *this;
}

ParameterMap& ParameterMap::operator += ( const ParameterMap& rhs )
{
	return *this = *this + rhs;
}

bool ParameterMap::operator == ( const ParameterMap& rhs ) const
{
	if( parameters.size() != rhs.parameters.size() ) return false;

	for( std::map<std::string, Parameter>::const_iterator it = parameters.begin();
		 it != parameters.end();
		 it++ )
	{
		std::map<std::string, Parameter>::const_iterator other = rhs.parameters.find( it->first );
		if( other == rhs.parameters.end() || other->second != it->second ) return false;
	}

	return true;
}

bool ParameterMap::operator != ( const ParameterMap& rhs ) const
{
	return !( *this == rhs );
}