#include <PCH.h>
#include <ParticleEmitter.h>
#include <ParticleSystem.h>
#include <Video.h>

ParticleEmitter::ParticleEmitter( ParticleSystem& ps ) : ps( ps ),
														 timeToLive( 0.0f ),
														 startDelay( 0.0f ),
														 age( 0.0f ),
														 rate( 1.0f ),
														 maxNumParticles( ~0 ),
														 numParticlesEmitted( 0 ),
														 emitWhenInvisible( false ),
														 visibilityRadius( 0.0f )
{
}

ParticleEmitter::~ParticleEmitter()
{
}

void ParticleEmitter::setTimeToLive( float timeToLive )
{
	this->timeToLive = timeToLive;
}

float ParticleEmitter::getTimeToLive() const
{
	return timeToLive;
}

void ParticleEmitter::setStartDelay( float startDelay )
{
	this->startDelay = startDelay;
}

float ParticleEmitter::getStartDelay() const
{
	return startDelay;
}

float ParticleEmitter::getAge() const
{
	return age;
}

void ParticleEmitter::setRate( float rate )
{
	this->rate = rate;
}

float ParticleEmitter::getRate() const
{
	return rate;
}

void ParticleEmitter::setMaxNumParticles( unsigned int maxNumParticles )
{
	this->maxNumParticles = maxNumParticles;
}

unsigned int ParticleEmitter::getMaxNumParticles() const
{
	return maxNumParticles;
}

void ParticleEmitter::setVisibilityBehavior( bool emitWhenInvisible,
											 float visibilityRadius )
{
	this->emitWhenInvisible = emitWhenInvisible;
	this->visibilityRadius = visibilityRadius;
}

ParticleSystem& ParticleEmitter::getParticleSystem() const
{
	return ps;
}

bool ParticleEmitter::isToBeDeleted() const
{
	return timeToLive < 0.0f;
}

void ParticleEmitter::update()
{
	if( age >= startDelay )
	{
		bool emit = true;

		if( numParticlesEmitted > maxNumParticles )
		{
			// Dieser Emitter hat die maximale Anzahl an Partikeln emittiert!
			emit = false;
			timeToLive = -1.0f;
		}
		else if( !emitWhenInvisible )
		{
			// Partikel nur dann emittieren, wenn der Emitter sichtbar ist
			emit = ps.getVideo().isCircleVisible( getAbsPosition(), visibilityRadius );
		}

		if( emit )
		{
			unsigned int n = doWork( maxNumParticles - numParticlesEmitted );
			numParticlesEmitted += n;
		}
	}

	if( timeToLive > 0.0f )
	{
		timeToLive -= ps.getTickDurationInSeconds();
		if( timeToLive == 0.0f ) timeToLive = -1.0f;
	}

	age += ps.getTickDurationInSeconds();
}