#include <PCH.h>
#include <Resource.h>
#include <Engine.h>

Resource::Resource( ResourceManager& manager,
				    const std::string& filename,
					bool unique,
					const ParameterMap& options) : manager( manager ),
												   filename( filename ),
												   unique( unique ),
												   options( options ),
												   refCount( 0 )
{
	if( filename.empty() )
	{
		// automatischen Namen generieren
		static int no = 0;
		std::ostringstream temp;
		temp << "Resource#" << ++no;
		this->filename = temp.str();
	}
}

Resource::~Resource()
{
	if( refCount )
	{
		std::ostringstream msg;
		msg << "Ressource " << filename << " wird zerstrt mit einem Referenzzhler von " << refCount;
		manager.getEngine().getLogger().warning( msg.str(), LOGGER_AUTO );
	}
}

unsigned int Resource::getRefCount() const
{
	return refCount;
}

unsigned int Resource::addRef()
{
	return ++refCount;
}

unsigned int Resource::release()
{
	assert( refCount );

	refCount--;
	if( !refCount )
	{
		// Dies war die letzte Referenz.
		manager.hintGC( this );
	}

	return refCount;
}

const ResourceManager& Resource::getManager() const
{
	return manager;
}

const std::string& Resource::getFilename() const
{
	return filename;
}

bool Resource::isUnique() const
{
	return unique;
}

const ParameterMap& Resource::getOptions() const
{
	return options;
}