#include <PCH.h>
#include <ResourceManager.h>
#include <Engine.h>

ResourceManager::ResourceManager( Engine& engine,
								  const std::string& componentName) : Component( engine, componentName )
{
	engine.connectUpdate( this, &ResourceManager::onUpdate );
}

ResourceManager::~ResourceManager()
{
}

void ResourceManager::hintGC( Resource* p_resource )
{
	gc.insert( p_resource );
}

void ResourceManager::runGC()
{
	if( gc.empty() ) return;

	// zu lschende Ressourcen durchgehen
	for( std::set<Resource*>::iterator it = gc.begin();
		 it != gc.end();
		 it++ )
	{
		// Muss die Ressource wirklich gelscht werden?
		if( !(*it)->getRefCount() )
		{
			// ja
			std::ostringstream msg;
			if( (*it)->isUnique() ) msg << "Einzigartige ";
			msg << "Ressource " << (*it)->getFilename() << " wird freigegeben";
			getEngine().getLogger().fine( msg.str(), LOGGER_AUTO );

			destroyResource( *it );
		}
	}

	gc.clear();
}

void ResourceManager::onUpdate()
{
	runGC();
}