#include <PCH.h>
#include <Serializer.h>
#include <Logger.h>
#include <BasicObjectSerializer.h>
#include <MethodObjectSerializer.h>
#include <ParameterMap.h>

Serializer::Serializer()
{
	// Standardserialisierer initialisieren
	BasicObjectSerializer<bool>::inst();
	BasicObjectSerializer<unsigned char>::inst();
	BasicObjectSerializer<signed char>::inst();
	BasicObjectSerializer<unsigned short>::inst();
	BasicObjectSerializer<signed short>::inst();
	BasicObjectSerializer<unsigned int>::inst();
	BasicObjectSerializer<signed int>::inst();
	BasicObjectSerializer<unsigned long int>::inst();
	BasicObjectSerializer<signed long int>::inst();
	BasicObjectSerializer<unsigned long long int>::inst();
	BasicObjectSerializer<signed long long int>::inst();
	BasicObjectSerializer<float>::inst();
	BasicObjectSerializer<double>::inst();
	BasicObjectSerializer<long double>::inst();
	BasicObjectSerializer<Vec2i>::inst().setReadableName( "Vec2i" );
	BasicObjectSerializer<Vec2f>::inst().setReadableName( "Vec2f" );
	BasicObjectSerializer<Vec2d>::inst().setReadableName( "Vec2d" );
	BasicObjectSerializer<Vec3i>::inst().setReadableName( "Vec3i" );
	BasicObjectSerializer<Vec3f>::inst().setReadableName( "Vec3f" );
	BasicObjectSerializer<Vec3d>::inst().setReadableName( "Vec3d" );
	BasicObjectSerializer<Vec4i>::inst().setReadableName( "Vec4i" );
	BasicObjectSerializer<Vec4f>::inst().setReadableName( "Vec4f" );
	BasicObjectSerializer<Vec4d>::inst().setReadableName( "Vec4d" );
	BasicObjectSerializer<Mat33i>::inst().setReadableName( "Mat33i" );
	BasicObjectSerializer<Mat33f>::inst().setReadableName( "Mat33f" );
	BasicObjectSerializer<Mat33d>::inst().setReadableName( "Mat33d" );
	BasicObjectSerializer<Mat44i>::inst().setReadableName( "Mat44i" );
	BasicObjectSerializer<Mat44f>::inst().setReadableName( "Mat44f" );
	BasicObjectSerializer<Mat44d>::inst().setReadableName( "Mat44d" );
	BasicObjectSerializer<std::string>::inst().setReadableName( "string" );
	MethodObjectSerializer<ParameterMap>::inst().setReadableName( "ParameterMap" );
}

Serializer::Serializer( const Serializer& copy )
{
}

Serializer::~Serializer()
{
}

Serializer& Serializer::inst()
{
	static Serializer inst;
	return inst;
}

void Serializer::registerObjectSerializer( const std::string& type,
										   ObjectSerializer* p_serializer )
{
	assert( p_serializer );

	objectSerializerRegistry[type] = p_serializer;
}

void Serializer::unregisterObjectSerializer( const std::string& type )
{
	objectSerializerRegistry.erase( type );
}

ObjectSerializer* Serializer::getObjectSerializerByType( const std::string& type ) const
{
	std::map<std::string, ObjectSerializer*>::const_iterator it = objectSerializerRegistry.find( type );
	return it == objectSerializerRegistry.end() ? 0 : it->second;
}

void Serializer::setReadableName( const std::string& type,
								  const std::string& readableName )
{
	objectSerializerRegistry[readableName] = getObjectSerializerByType( type );
}