#include <PCH.h>
#include <SoundControllerTarget.h>
#include <Audio.h>
#include <Engine.h>

SoundControllerTarget::SoundControllerTarget( Audio& audio ) : audio( audio ),
															   dt( static_cast<float>( audio.getEngine().getTickDurationInSeconds() ) ),
															   stopCountDown( 0.0f ),
															   pauseCountDown( 0.0f ),
															   volumeTarget( 0.0f ),
															   volumeSpeed( 0.0f ),
															   pitchTarget( 0.0f ),
															   pitchSpeed( 0.0f )
{
}

SoundControllerTarget::~SoundControllerTarget()
{
}

void SoundControllerTarget::setStopCountDown( float time )
{
	stopCountDown = time;
}

void SoundControllerTarget::setPauseCountDown( float time )
{
	pauseCountDown = time;
}

void SoundControllerTarget::setVolumeTarget( float volume,
											 float speed )
{
	volumeTarget = volume;
	if( speed > 0.0f ) volumeSpeed = speed * dt;
	else volumeSpeed = -1.0f;
}

void SoundControllerTarget::setPitchTarget( float pitch,
										    float speed )
{
	pitchTarget = pitch;
	if( speed > 0.0f ) pitchSpeed = speed * dt;
	else pitchSpeed = -1.0f;
}

void SoundControllerTarget::update()
{
	if( stopCountDown > 0.0f )
	{
		stopCountDown -= dt;
		if( stopCountDown <= 0.0f ) stop();
	}

	if( pauseCountDown > 0.0f )
	{
		pauseCountDown -= dt;
		if( pauseCountDown <= 0.0f ) setPaused( true );
	}

	if( volumeSpeed > 0.0f )
	{
		float current = getVolume();
		float delta = volumeTarget - current;
		if( fabsf( delta ) < 0.001f || volumeSpeed >= 1.0f )
		{
			// fertig
			setVolume( volumeTarget );
			volumeSpeed = 0.0f;

			if( stopCountDown == -1.0f ) stop();
			if( pauseCountDown == -1.0f ) setPaused( true );
		}
		else
		{
			setVolume( current * ( 1.0f - volumeSpeed ) + volumeTarget * volumeSpeed );
		}
	}

	if( pitchSpeed > 0.0f )
	{
		float current = getPitch();
		float delta = pitchTarget - current;
		if( fabsf( delta ) < 0.001f || pitchSpeed >= 1.0f )
		{
			// fertig
			setPitch( pitchTarget );
			pitchSpeed = 0.0f;

			if( stopCountDown == -2.0f ) stop();
			if( pauseCountDown == -2.0f ) setPaused( true );
		}
		else
		{
			setPitch( current * ( 1.0f - pitchSpeed ) + pitchTarget * pitchSpeed );
		}
	}
}