#include <PCH.h>
#include <Utils.h>
#include <MTRand.h>
#include <Roots.h>

#ifdef WIN32
#include <Windows.h>
#endif

static MTRand mt;

std::string getPathFilename( const std::string& path )
{
	// letzten Backslash suchen
	size_t i = path.find_last_of( '\\' );
	if( i == std::string::npos ) return path;
	else return path.substr( i + 1 );
}

std::string getPathDirectory( const std::string& path )
{
	// letzten Backslash suchen
	size_t i = path.find_last_of( '\\' );
	if( i == std::string::npos ) return "";
	else return path.substr( 0, i );
}

std::string removeExtension( const std::string& filename )
{
	// letzten Punkt suchen
	size_t i = filename.find_last_of( '.' );
	if( i == std::string::npos ) return filename;
	else return filename.substr( 0, i );
}

std::list<std::string> listFiles( const std::string& pattern )
{
	std::list<std::string> files;

#ifdef WIN32
	WIN32_FIND_DATAA findData;
	HANDLE find = FindFirstFileA( pattern.c_str(), &findData );
	if( find != INVALID_HANDLE_VALUE )
	{
		do
		{
			std::string filename = findData.cFileName;
			if( filename != "." && filename != ".." ) files.push_back( filename );
		}
		while( FindNextFileA( find, &findData ) );
	}
#endif

	return files;
}

unsigned int nextPow2( unsigned int value )
{
	unsigned int result = 1;

	__asm
	{
		xor ecx, ecx
		bsr ecx, value
		inc ecx
		shl result, cl
	}

	return result;
}

int randInt( int min,
			 int max )
{
	assert( min <= max );
	return min + mt.randInt( max - min );
}

unsigned int randInt()
{
	return mt.randInt();
}

float randFloat( float min,
				 float max )
{
	return static_cast<float>( randDouble( min, max ) );
}

double randDouble( double min,
				   double max )
{
	assert( min <= max );
	return min + mt.rand( max - min );
}

Vec2f randVec2fDir()
{
	const float angle = randFloat( 0.0f, 6.2832f );
	return Vec2f( sinf( angle ), cosf( angle ) );
}

int solveQuartic( double a,
				  double b,
				  double c,
				  double d,
				  double e,
				  double* p_outRealRoots )
{
	double coeff[] = { e, d, c, b, a };
	return SolveQuartic( coeff, p_outRealRoots );
}