#include <PCH.h>
#include <XMLSerializationStream.h>
#include <tinyxml.h>

XMLSerializationStream::XMLSerializationStream()
{
	p_doc = new TiXmlDocument;
	p_elem = new TiXmlElement( "Data" );
	p_doc->LinkEndChild( p_elem );
}

XMLSerializationStream::XMLSerializationStream( const std::string& filename ) : p_doc( 0 )
{
	setError( load( filename ) );
}

XMLSerializationStream::~XMLSerializationStream()
{
	delete p_doc;
}

int XMLSerializationStream::load( const std::string& filename )
{
	delete p_doc;

	// XML-Dokument laden
	p_doc = new TiXmlDocument( filename.c_str() );
	if( !p_doc->LoadFile() )
	{
		// Fehler!
		return 1;
	}

	// Wurzelelement holen
	p_elem = p_doc->FirstChildElement( "Data" );
	if( !p_elem )
	{
		// Fehler!
		return 2;
	}

	return 0;
}

int XMLSerializationStream::save( const std::string& filename ) const
{
	assert( p_doc );

	// Dokument speichern
	if( !p_doc->SaveFile( filename.c_str() ) )
	{
		// Fehler!
		return 1;
	}

	return 0;
}

void XMLSerializationStream::beginObject( const std::string& type )
{
	assert( p_elem );

	TiXmlElement* p_object = new TiXmlElement( "Object" );
	p_object->SetAttribute( "type", type.c_str() );
	p_elem->LinkEndChild( p_object );

	p_elem = p_object;
}

void XMLSerializationStream::writeString( const std::string& text )
{
	assert( p_elem );

	TiXmlText* p_text = new TiXmlText( text.c_str() );
	p_elem->LinkEndChild( p_text );
}

void XMLSerializationStream::endObject()
{
	assert( p_elem );

	p_elem = static_cast<TiXmlElement*>( p_elem->Parent() );
	assert( p_elem );
}

std::string XMLSerializationStream::openObject() const
{
	assert( p_elem );

	TiXmlElement* p_object = p_elem->FirstChildElement( "Object" );
	if( !p_object )
	{
		// Es gibt kein nchstes Objekt!
		return "";
	}

	const char* p_type = p_object->Attribute( "type" );
	assert( p_type );

	p_elem = p_object;

	return p_type;
}

std::string XMLSerializationStream::readString() const
{
	assert( p_elem );

	const char* p_text = p_elem->GetText();
	return p_text ? p_text : "";
}

void XMLSerializationStream::closeObject() const
{
	assert( p_elem );

	// Element entfernen
	TiXmlElement* p_parent = static_cast<TiXmlElement*>( p_elem->Parent() );
	p_parent->RemoveChild( p_elem );
	p_elem = p_parent;
}