#ifndef CARTOONRENDERER_H
#define CARTOONRENDERER_H

class CartoonRenderer : public Base
{
public:
	CartoonRenderer( GameEnvironment& env, const std::string& stylesFilename );
	~CartoonRenderer();

	// setzt den aktiven Zeichenstil
	int setStyle( const std::string& name );

	// initialisiert den Zufallsgenerator
	void setContext( const std::string& name );
	void setContext( unsigned int seed );

	// zeichnet einen Linienzug
	int drawPolyLine( const std::vector<Vec2f>& vertices, bool close = false, float widthScale = 1.0f, const Vec4f& color = Vec4f( 1.0f, 1.0f, 1.0f, 1.0f ), const std::vector<Vec4f>* p_colors = 0 );

	// fllt ein Polygon
	int fillPolygon( const std::vector<Vec2f>& vertices, bool overrideTexture = false, TextureHandle texture = 0 );

private:
	struct Style
	{
		std::string name;
		float fillTiling;
		TextureHandle fill;
		int numLines;
		int offset;
		float lineTiling;
		float lineWidth;
		TextureHandle lines;
	};

	// zeichnet ein Liniensegment
	void drawSegment( const Vec2f& from, const Vec2f& to, int lineSubStyle = 0, int nextLineSubStyle = 0, float widthScale = 1.0f, const Vec4f& color1 = Vec4f( 1.0f, 1.0f, 1.0f, 1.0f ), const Vec4f& color2 = Vec4f( 1.0f, 1.0f, 1.0f, 1.0f ) );

	// zieht eine Zufallszahl
	unsigned int randInt() const;

	GameEnvironment& env;
	std::map<std::string, Style> styles;
	Style* p_currentStyle;
	GLUtesselator* p_tess;
	Vec2f lastSegmentEnd;
	Vec2f lastSegmentDir;
	Vec2f lastCorner[2];
	mutable unsigned int rng;
};

#endif