#ifndef CONSOLE_H
#define CONSOLE_H

class Console : public Component
{
public:
	Console( GameEnvironment& env );
	~Console();

	void print( const std::string& text, const Vec4f& color = Vec4f( 1.0f, 1.0f, 1.0f, 1.0f ) );
	void clear();

	void setVisible( bool visible );
	bool isVisible() const;

private:
	void onPostRenderFinal();
	void onSDLEvent( const SDL_Event& e );

	GameEnvironment& env;
	bool visible;
	std::string buffer;
	std::string input;
	size_t cursor;
	std::vector<std::string> history;
	size_t historyCursor;
};

class GameConsoleLogHandler : public LogHandler
{
public:
	GameConsoleLogHandler( Logger& logger,
						   Console* p_console,
						   const Vec4f& color ) : LogHandler( logger ),
												  p_console( p_console ),
												  color( color )
	{
	}

	~GameConsoleLogHandler()
	{
	}
 
	void setConsole( Console* p_console )
	{
		this->p_console = p_console;
	}

	void flush()
	{
		if( p_console )
		{
			p_console->print( buffer, color );
			buffer.clear();
		}
	}

	void log( LogLevel level,
			  const std::string& text,
			  const std::string& filename,
			  unsigned int line )
	{
		static const char* p_caption[LL_MAX_VALUE] = { "ERROR", "WARNING", "INFO", "FINE", "FINER", "FINEST" };

		std::ostringstream msg;
		msg << p_caption[level] << ": " << text << " (" << getPathFilename( filename ) << ":" << line << ")" << std::endl;
		buffer += msg.str();
		flush();
	}

private:
	Console* p_console;
	Vec4f color;
	std::string buffer;
};

#endif