#ifndef GAMEENVIRONMENT_H
#define GAMEENVIRONMENT_H

#include "NeuralNet.h"
#include "GUI.h"
#include "Options.h"
#include "Scripting.h"

struct GameEnvironment
{
	GameEnvironment( Engine& engine,
					 Logger& logger,
					 NeuralNetManager& neuralNetMgr,
					 GUI& gui,
					 Options& options ) : engine( engine ),
										  logger( logger ),
										  neuralNetMgr( neuralNetMgr ),
										  gui( gui ),
										  options( options ),
										  dt( engine.getTickDurationInSeconds() )
	{
		// Schriftarten laden
		FontManager& fontMgr = engine.getFontManager();
		normalFont = fontMgr.get( "data\\david.fnt" );
		scoreFont = fontMgr.get( "data\\score.fnt" );
		countDownFont = fontMgr.get( "data\\big.fnt" );
		lcdFont = fontMgr.get( "data\\lcd.fnt" );
		fixedFont = fontMgr.get( "data\\fixed.fnt" );
		mobileFont = fontMgr.get( "data\\mobile.fnt" );
	}

	// die Engine
	Engine& engine;

	// der Spiel-Logger
	Logger& logger;

	// Manager fr neuronale Netze
	NeuralNetManager& neuralNetMgr;

	// GUI
	GUI& gui;

	// Optionen
	Options& options;

	// Schrittdauer
	const float dt;

	// Schriftarten
	FontHandle normalFont;
	FontHandle scoreFont;
	FontHandle countDownFont;
	FontHandle lcdFont;
	FontHandle fixedFont;
	FontHandle mobileFont;
};

#endif