#ifndef HIGHSCORESDB_H
#define HIGHSCORESDB_H

struct HighscoresEntry
{
	std::string name;
	unsigned int score;
};

typedef std::vector<HighscoresEntry> HighscoresList;

class HighscoresDB
{
public:
	static HighscoresDB& inst();

	// liefert den Platz, den die angegebene Punktzahl in der Bestenliste bekommen wrde
	// (die Anzahl der Eintrge, die besser sind)
	unsigned int getPlaceInHighscore( const std::string& level, unsigned int score ) const;

	// liefert die Highscore-Liste fr einen bestimmten Level
	HighscoresList getHighscores( const std::string& level ) const;

	// erzeugt zwei Strings fr die Highscores
	void buildHighscores( const std::string& level, std::string& outPlayers, std::string& outScores ) const;

	// fgt der Highscore-Liste eines Levels einen Eintrag hinzu
	void addHighscoreEntry( const std::string& level, const HighscoresEntry& entry );

	// setzt die Highscores fr einen Level zurck
	void resetHighscores( const std::string& level );

	// liefert true, wenn der angegebene Level schon geschafft wurde
	bool wasLevelCompleted( const std::string& level ) const;

	// fgt einen Level zur Geschafft-Menge hinzu oder entfernt ihn
	void setLevelCompleted( const std::string& level, bool completed = true );

	// berechnet den Hash-Code einer (Level-)Datei
	unsigned long long getHashCode( const std::string& filename, size_t ignoreLastNBytes = 0 ) const;

private:
	HighscoresDB();
	HighscoresDB( const HighscoresDB& copy );
	~HighscoresDB();

	void load();
	void save() const;

	size_t writeEncrypted( FILE* p_file, const void* p_source, size_t length ) const;
	size_t readEncrypted( FILE* p_file, void* p_dest, size_t length ) const;

	// Listen der Highscores fr jeden Level
	std::map<unsigned long long, HighscoresList> highscoresLists;

	// Menge der bereits geschafften Levels
	std::set<unsigned long long> levelsCompleted;
};

#endif