#ifndef OBJ_PLAYER_H
#define OBJ_PLAYER_H

#include "GameObject.h"
#include "Controllable.h"
#include "Damagable.h"
#include "Armed.h"

class Obj_Player : public GameObject, public Controllable, public Damagable, public Armed
{
public:
	Obj_Player( Level& level, const TiXmlElement* p_root );
	~Obj_Player();

	std::string getType() const { return "Player"; }

	void update();
	void render( RenderPass pass );
	void onCollision( GameObject* p_other, CollisionInfo& info );
	void addAvoidanceAreas();
	void onKill();
	void onRebirth();

	void addFuel( float value );
	void setFuel( float value );
	float getFuel() const;
	void addScore( int value );
	void setScore( int value );
	unsigned int getScore() const;
	void addBonusScore();
	void addShield( float value );
	void setShield( float value );
	float getShield() const;
	void setMouseAiming( bool enable );
	bool getMouseAiming() const;

private:
	float thrust;
	float fuel;
	float shield;
	bool mouseAiming;
	Vec2f crosshairPosition;
	bool canFireAtCrosshair;
	Vec2f firingDirectionAtCrosshair;
	unsigned int score;
	SimpleEmitter* p_smokeEmitter;
	SimpleEmitter* p_fireEmitter;
	SimpleEmitter* p_damageEmitter;
	SoundInstance* p_engineSoundInst;
	SoundHandle shieldSound;
	SoundInstance* p_shieldSoundInst;
	Placeable weaponPlatform;
};

#endif