#ifndef SCRIPTING_H
#define SCRIPTING_H

#include <lua.hpp>
#include <luabind/luabind.hpp>

class Console;
class Level;

class Scripting
{
	friend int runThread( lua_State* p_lua );

public:
	static Scripting& inst();
	void init( Console* p_console );
	void exit();

	// liefert einen neuen lua-Interpreter
	lua_State* createInterpreter();

	// fhrt ein Script aus
	void executeScript( lua_State* p_lua, const std::string& code );

	// ruft eine Funktion auf
	void call( luabind::object function );

	template<typename T1> void call( luabind::object function, const T1& arg1 )
	{
		if( !function || luabind::type( function ) != LUA_TFUNCTION ) return;

		try
		{
			luabind::call_function<void>( function, arg1 );
		}
		catch( luabind::error& err )
		{
			printException( err );
		}
	}

	template<typename T1, typename T2> void call( luabind::object function, const T1& arg1, const T2& arg2 )
	{
		if( !function || luabind::type( function ) != LUA_TFUNCTION ) return;

		try
		{
			luabind::call_function<void>( function, arg1, arg2 );
		}
		catch( luabind::error& err )
		{
			printException( err );
		}
	}

	// gibt einen Text aus
	void print( const std::string& text );

	// gibt lua-Informationen aus
	void luaInfo();

	Level* getLevel( lua_State* p_lua );
	void resumeThread( lua_State* p_lua );
	void finalizeThread( lua_State* p_lua );

	void pushCurrentLuaState( lua_State* p_currentLuaState );
	void popCurrentLuaState();

	lua_State* getLua() const;
	lua_State* getCurrentLuaState() const;
	Console* getConsole() const;

	void printException( const luabind::error& err );

private:
	Scripting();
	Scripting( const Scripting& copy );
	~Scripting();

	void bindToLua( lua_State* p_lua );

	lua_State* p_rootLua;
	std::stack<lua_State*> luaStates;
	std::map<lua_State*, int> reference;
	std::map<lua_State*, lua_State*> parent;
	Console* p_console;
};

#endif