#include <PCH.h>
#include <Armed.h>
#include <Obj_Projectile.h>

Armed::Armed( GameEnvironment& env ) : armedEnv( env ),
									   currentWeapon( WT_LAST ),
									   reload( 0.0f ),
									   p_defaultWeaponParent( 0 )
{
	for( int i = 0; i < WT_LAST; i++ )
	{
		weapon[i].initialized = false;
		weapon[i].ammo = 0;
		weapon[i].p_parent = 0;
	}
}

Armed::~Armed()
{
}

void Armed::updateArmed()
{
	reload += armedEnv.dt;
}

Obj_Projectile* Armed::fire( const Vec2f& direction,
							 bool absDir )
{
	if( currentWeapon == WT_LAST ) return 0;

	WeaponInfo& wpn = weapon[currentWeapon];
	GameObject* p_obj = dynamic_cast<GameObject*>( this );
	Placeable* p_placeable = wpn.p_parent ? wpn.p_parent : p_obj;

	// Ist die Waffe nachgeladen?
	if( wpn.ammo > 0 && reload >= wpn.reloadTime )
	{
		Obj_Projectile* p_projectile = new Obj_Projectile( p_obj->getLevel(), p_obj, wpn.projectileType );

		p_projectile->attachTo( p_placeable, false );
		p_projectile->setPosition( wpn.relPosition );

		// Richtung berechnen
		if( absDir ) p_projectile->setAbsDirection( -direction );
		else p_projectile->setDirection( -direction );
		p_projectile->setAngle( p_projectile->getAngle() + randFloat( -wpn.deviation, wpn.deviation ) );
		p_projectile->detachFromParent();

		// Geschwindigkeitsvektor setzen
		float speed = wpn.speed + randFloat( -wpn.speedVar, wpn.speedVar );
		p_projectile->setVelocity( speed * -p_projectile->getDirection() + p_obj->getVelocity() );

		// Sound abspielen
		SoundInstance* p_soundInst = wpn.fireSound->createInstance();
		if( p_soundInst )
		{
			p_soundInst->setPosition( p_projectile->getPosition() );
			p_soundInst->attachTo( p_placeable, true );
			p_soundInst->setVolume( wpn.fireSoundVolume + randFloat( -wpn.fireSoundVolumeVar, wpn.fireSoundVolumeVar ) );
			p_soundInst->setPitch( wpn.fireSoundPitch + randFloat( -wpn.fireSoundPitchVar, wpn.fireSoundPitchVar ) );
			p_soundInst->play();
		}

		if( wpn.ammo != ~0 ) wpn.ammo--;
		reload = 0.0f;

		return p_projectile;
	}

	return 0;
}

Obj_Projectile* Armed::fireAt( GameObject* p_target )
{
	if( !p_target ) return 0;
	if( currentWeapon == WT_LAST ) return 0;

	WeaponInfo& wpn = weapon[currentWeapon];
	GameObject* p_obj = dynamic_cast<GameObject*>( this );
	Placeable* p_placeable = wpn.p_parent ? wpn.p_parent : p_obj;

	// zielen
	Vec2f shootVector;
	if( computeHitWithGravity( p_placeable->relPositionToAbsPosition( wpn.relPosition ), p_obj->getVelocity(),
							   p_target->getPosition(), p_target->getVelocity(),
							   weapon[currentWeapon].calcSpeed,
							   p_obj->getLevel().getGravity(),
							   static_cast<Vec2f*>( 0 ), static_cast<float*>( 0 ), &shootVector ) )
	{
		// Feuer!
		return fire( shootVector, true );
	}
	else
	{
		// Treffer nicht mglich!
		return 0;
	}
}

void Armed::selectWeapon( WeaponType weapon )
{
	currentWeapon = weapon;
}

void Armed::setupWeapon( WeaponType weapon,
						 const Vec2f& relPosition,
						 Placeable* p_parent )
{
	GameObject* p_obj = dynamic_cast<GameObject*>( this );
	WeaponInfo& wpn = this->weapon[weapon];
	SoundManager& soundMgr = armedEnv.engine.getSoundManager();

	switch( weapon )
	{
	case WT_INK_BLASTER:
		wpn.projectileType = 1;
		wpn.speed = wpn.calcSpeed = 40.0f;
		wpn.speedVar = 2.5f;
		wpn.deviation = 0.05f;
		wpn.reloadTime = 0.2f;
		wpn.showTrajectory = false;
		wpn.fireSound = soundMgr.get( "data\\fire1.ogg" );
		wpn.fireSoundVolume = 0.75f;
		wpn.fireSoundVolumeVar = 0.05f;
		wpn.fireSoundPitch = 1.0f;
		wpn.fireSoundPitchVar = 0.25f;
		break;
	case WT_MISSILE:
		wpn.projectileType = 4;
		wpn.speed = 10.0f;
		wpn.calcSpeed = 30.0f;
		wpn.speedVar = 1.0f;
		wpn.deviation = 0.05f;
		wpn.reloadTime = 1.0f;
		wpn.showTrajectory = false;
		wpn.fireSound = soundMgr.get( "data\\fire3.ogg" );
		wpn.fireSoundVolume = 0.75f;
		wpn.fireSoundVolumeVar = 0.05f;
		wpn.fireSoundPitch = 1.0f;
		wpn.fireSoundPitchVar = 0.3f;
		break;
	case WT_CANNON:
		wpn.projectileType = 5;
		wpn.speed = wpn.calcSpeed = 75.0f;
		wpn.speedVar = 0.0f;
		wpn.deviation = 0.0f;
		wpn.reloadTime = 1.5f;
		wpn.showTrajectory = true;
		wpn.trajectoryFriction = 0.7f;
		wpn.fireSound = soundMgr.get( "data\\fire4.ogg" );
		wpn.fireSoundVolume = 0.9f;
		wpn.fireSoundVolumeVar = 0.05f;
		wpn.fireSoundPitch = 1.0f;
		wpn.fireSoundPitchVar = 0.2f;
		break;
	case WT_INK_BLASTER_2:
		wpn.projectileType = 2;
		wpn.speed = wpn.calcSpeed = 25.0f;
		wpn.speedVar = 5.0f;
		wpn.deviation = 0.25f;
		wpn.reloadTime = 0.6f;
		wpn.showTrajectory = false;
		wpn.fireSound = soundMgr.get( "data\\fire2.ogg" );
		wpn.fireSoundVolume = 0.75f;
		wpn.fireSoundVolumeVar = 0.05f;
		wpn.fireSoundPitch = 1.0f;
		wpn.fireSoundPitchVar = 0.25f;
		break;
	case WT_AA:
		wpn.projectileType = 3;
		wpn.speed = wpn.calcSpeed = 25.0f;
		wpn.speedVar = 5.0f;
		wpn.deviation = 0.15f;
		wpn.reloadTime = 0.75f;
		wpn.showTrajectory = false;
		wpn.fireSound = soundMgr.get( "data\\fire2.ogg" );
		wpn.fireSoundVolume = 0.95f;
		wpn.fireSoundVolumeVar = 0.05f;
		wpn.fireSoundPitch = 0.8f;
		wpn.fireSoundPitchVar = 0.1f;
		break;
	}

	wpn.initialized = true;
	wpn.relPosition = relPosition;
	wpn.p_parent = p_parent ? p_parent : p_defaultWeaponParent;
	if( wpn.fireSound ) wpn.fireSound->setDefaultSoundGroupName( "SFX" );
}

WeaponType Armed::getCurrentWeapon() const
{
	return currentWeapon;
}

const WeaponInfo* Armed::getWeapons() const
{
	return weapon;
}

void Armed::addAmmo( WeaponType weapon,
					 unsigned int value )
{
	if( value == ~0 ) this->weapon[weapon].ammo = value;
	else this->weapon[weapon].ammo += value;
}

void Armed::setAmmo( WeaponType weapon,
					 unsigned int value )
{
	this->weapon[weapon].ammo = value;
}

unsigned int Armed::getAmmo( WeaponType weapon ) const
{
	return this->weapon[weapon].ammo;
}