#include <PCH.h>
#include <GS_Loading.h>

GS_Loading::GS_Loading( GameEnvironment& env ) : GameState( env.engine, "Loading" ),
												 env( env )
{
	// Logo laden
	logo = engine.getTextureManager().get( "data\\logo.png" );
}

GS_Loading::~GS_Loading()
{
}

void GS_Loading::onEnter( const ParameterMap& params )
{
	env.logger.fine( "GS_Loading::onEnter", LOGGER_AUTO );

	this->params = params;
	time = 0;
}

void GS_Loading::onLeave()
{
	env.logger.fine( "GS_Loading::onLeave", LOGGER_AUTO );
}

void GS_Loading::onActivate()
{
	env.logger.fine( "GS_Loading::onActivate", LOGGER_AUTO );
}

void GS_Loading::onDeactivate()
{
	env.logger.fine( "GS_Loading::onDeactivate", LOGGER_AUTO );
}

void GS_Loading::render()
{
	video.clear();

	// Logo rendern
	video.resetCamera();
	video.setupProjection( Vec2f( -1.0f, -1.0f ), Vec2f( 1.0f, 1.0f ) );
	video.setTexture( 0 );
	video.setupAlphaBlending( true, GL_SRC_ALPHA, GL_ONE );
	video.setTexture( logo );
	glPushMatrix();
	glTranslatef( 0.75f, 0.75f, 0.0f );
	glColor4f( 1.0f, 1.0f, 1.0f, 0.75f );
	video.drawSprite( Vec2i( 0, 0 ), Vec2i( 1024, 512 ), Vec2f( 0.6f, 0.3f ) );
	glPopMatrix();
	glColor4f( 1.0f, 1.0f, 1.0f, 0.1f );
	video.drawSprite( Vec2i( 0, 0 ), Vec2i( 1024, 512 ), Vec2f( 4.0f, 2.0f ) );

	// Text ausgeben
	video.setupProjection( Vec2f( -24.0f, -18.0f ), Vec2f( 24.0f, 18.0f ) );
	glPushMatrix();
	glTranslatef( -10.0f, -0.75f, 0.0f );
	env.normalFont->print( "Daten werden geladen. Bitte warten ...", 1.5f );
	glPopMatrix();

	CEGUI::MouseCursor::getSingleton().hide();
}

void GS_Loading::update()
{
	if( time >= params.get<unsigned int>( "Redirect.Wait", 100 ) )
	{
		// an den neuen Spielzustand weiterleiten
		engine.popGameState( true, false );
		engine.pushGameState( params.get<std::string>( "Redirect.GameState" ), params );
	}

	time += engine.getTickDuration();
}