#include <PCH.h>
#include <GUI.h>
#include <SDL.h>
#include <CEGUI/CEGUIDefaultResourceProvider.h>
#include <CEGUI/renderers/OpenGLGUIRenderer/openglrenderer.h>

CEGUI::uint convertSDLKeyToCEGUIKey( SDLKey key,
									 Uint8 scancode )
{
	switch( key )
	{
	case SDLK_BACKSPACE:    return CEGUI::Key::Backspace;
	case SDLK_TAB:          return CEGUI::Key::Tab;
	case SDLK_RETURN:       return CEGUI::Key::Return;
	case SDLK_PAUSE:        return CEGUI::Key::Pause;
	case SDLK_ESCAPE:       return CEGUI::Key::Escape;
	case SDLK_SPACE:        return CEGUI::Key::Space;
	case SDLK_COMMA:        return CEGUI::Key::Comma;
	case SDLK_MINUS:        return CEGUI::Key::Minus;
	case SDLK_PERIOD:       return CEGUI::Key::Period;
	case SDLK_SLASH:        return CEGUI::Key::Slash;
	case SDLK_0:            return CEGUI::Key::Zero;
	case SDLK_1:            return CEGUI::Key::One;
	case SDLK_2:            return CEGUI::Key::Two;
	case SDLK_3:            return CEGUI::Key::Three;
	case SDLK_4:            return CEGUI::Key::Four;
	case SDLK_5:            return CEGUI::Key::Five;
	case SDLK_6:            return CEGUI::Key::Six;
	case SDLK_7:            return CEGUI::Key::Seven;
	case SDLK_8:            return CEGUI::Key::Eight;
	case SDLK_9:            return CEGUI::Key::Nine;
	case SDLK_COLON:        return CEGUI::Key::Colon;
	case SDLK_SEMICOLON:    return CEGUI::Key::Semicolon;
	case SDLK_EQUALS:       return CEGUI::Key::Equals;
	case SDLK_LEFTBRACKET:  return CEGUI::Key::LeftBracket;
	case SDLK_BACKSLASH:    return CEGUI::Key::Backslash;
	case SDLK_RIGHTBRACKET: return CEGUI::Key::RightBracket;
	case SDLK_a:            return CEGUI::Key::A;
	case SDLK_b:            return CEGUI::Key::B;
	case SDLK_c:            return CEGUI::Key::C;
	case SDLK_d:            return CEGUI::Key::D;
	case SDLK_e:            return CEGUI::Key::E;
	case SDLK_f:            return CEGUI::Key::F;
	case SDLK_g:            return CEGUI::Key::G;
	case SDLK_h:            return CEGUI::Key::H;
	case SDLK_i:            return CEGUI::Key::I;
	case SDLK_j:            return CEGUI::Key::J;
	case SDLK_k:            return CEGUI::Key::K;
	case SDLK_l:            return CEGUI::Key::L;
	case SDLK_m:            return CEGUI::Key::M;
	case SDLK_n:            return CEGUI::Key::N;
	case SDLK_o:            return CEGUI::Key::O;
	case SDLK_p:            return CEGUI::Key::P;
	case SDLK_q:            return CEGUI::Key::Q;
	case SDLK_r:            return CEGUI::Key::R;
	case SDLK_s:            return CEGUI::Key::S;
	case SDLK_t:            return CEGUI::Key::T;
	case SDLK_u:            return CEGUI::Key::U;
	case SDLK_v:            return CEGUI::Key::V;
	case SDLK_w:            return CEGUI::Key::W;
	case SDLK_x:            return CEGUI::Key::X;
	case SDLK_y:            return CEGUI::Key::Y;
	case SDLK_z:            return CEGUI::Key::Z;
	case SDLK_DELETE:       return CEGUI::Key::Delete;
	case SDLK_KP0:          return CEGUI::Key::Numpad0;
	case SDLK_KP1:          return CEGUI::Key::Numpad1;
	case SDLK_KP2:          return CEGUI::Key::Numpad2;
	case SDLK_KP3:          return CEGUI::Key::Numpad3;
	case SDLK_KP4:          return CEGUI::Key::Numpad4;
	case SDLK_KP5:          return CEGUI::Key::Numpad5;
	case SDLK_KP6:          return CEGUI::Key::Numpad6;
	case SDLK_KP7:          return CEGUI::Key::Numpad7;
	case SDLK_KP8:          return CEGUI::Key::Numpad8;
	case SDLK_KP9:          return CEGUI::Key::Numpad9;
	case SDLK_KP_PERIOD:    return CEGUI::Key::Decimal;
	case SDLK_KP_DIVIDE:    return CEGUI::Key::Divide;
	case SDLK_KP_MULTIPLY:  return CEGUI::Key::Multiply;
	case SDLK_KP_MINUS:     return CEGUI::Key::Subtract;
	case SDLK_KP_PLUS:      return CEGUI::Key::Add;
	case SDLK_KP_ENTER:     return CEGUI::Key::NumpadEnter;
	case SDLK_KP_EQUALS:    return CEGUI::Key::NumpadEquals;
	case SDLK_UP:           return CEGUI::Key::ArrowUp;
	case SDLK_DOWN:         return CEGUI::Key::ArrowDown;
	case SDLK_RIGHT:        return CEGUI::Key::ArrowRight;
	case SDLK_LEFT:         return CEGUI::Key::ArrowLeft;
	case SDLK_INSERT:       return CEGUI::Key::Insert;
	case SDLK_HOME:         return CEGUI::Key::Home;
	case SDLK_END:          return CEGUI::Key::End;
	case SDLK_PAGEUP:       return CEGUI::Key::PageUp;
	case SDLK_PAGEDOWN:     return CEGUI::Key::PageDown;
	case SDLK_F1:           return CEGUI::Key::F1;
	case SDLK_F2:           return CEGUI::Key::F2;
	case SDLK_F3:           return CEGUI::Key::F3;
	case SDLK_F4:           return CEGUI::Key::F4;
	case SDLK_F5:           return CEGUI::Key::F5;
	case SDLK_F6:           return CEGUI::Key::F6;
	case SDLK_F7:           return CEGUI::Key::F7;
	case SDLK_F8:           return CEGUI::Key::F8;
	case SDLK_F9:           return CEGUI::Key::F9;
	case SDLK_F10:          return CEGUI::Key::F10;
	case SDLK_F11:          return CEGUI::Key::F11;
	case SDLK_F12:          return CEGUI::Key::F12;
	case SDLK_F13:          return CEGUI::Key::F13;
	case SDLK_F14:          return CEGUI::Key::F14;
	case SDLK_F15:          return CEGUI::Key::F15;
	case SDLK_NUMLOCK:      return CEGUI::Key::NumLock;
	case SDLK_SCROLLOCK:    return CEGUI::Key::ScrollLock;
	case SDLK_RSHIFT:       return CEGUI::Key::RightShift;
	case SDLK_LSHIFT:       return CEGUI::Key::LeftShift;
	case SDLK_RCTRL:        return CEGUI::Key::RightControl;
	case SDLK_LCTRL:        return CEGUI::Key::LeftControl;
	case SDLK_RALT:         return CEGUI::Key::RightAlt;
	case SDLK_LALT:         return CEGUI::Key::LeftAlt;
	case SDLK_LSUPER:       return CEGUI::Key::LeftWindows;
	case SDLK_RSUPER:       return CEGUI::Key::RightWindows;
	case SDLK_SYSREQ:       return CEGUI::Key::SysRq;
	case SDLK_MENU:         return CEGUI::Key::AppMenu;
	case SDLK_POWER:        return CEGUI::Key::Power;
	default:                return scancode;
	}
}

GUI::GUI( Engine& engine ) : Component( engine, "GUI" ),
							 p_system( 0 ),
							 p_renderer( 0 ),
							 p_root( 0 )
{
	try
	{
		// OpenGL-Renderer erzeugen
		const Vec2i resolution( engine.getVideo().getResolution() );
		p_renderer = new CEGUI::OpenGLRenderer( 0, resolution.x, resolution.y );

		// Systemobjekt erzeugen
		p_system = new CEGUI::System( p_renderer );

		// Standardressourcengruppen setzen
        CEGUI::Imageset::setDefaultResourceGroup( "imagesets" );
        CEGUI::Font::setDefaultResourceGroup( "fonts" );
        CEGUI::Scheme::setDefaultResourceGroup( "schemes" );
        CEGUI::WidgetLookManager::setDefaultResourceGroup( "looknfeel" );
        CEGUI::WindowManager::setDefaultResourceGroup( "layouts" );

		// Standardverzeichnisse setzen
		CEGUI::DefaultResourceProvider* p_rp = static_cast<CEGUI::DefaultResourceProvider*>( p_system->getResourceProvider() );
		p_rp->setResourceGroupDirectory( "imagesets", "data\\gui\\imagesets\\" );
		p_rp->setResourceGroupDirectory( "fonts", "data\\gui\\fonts\\" );
		p_rp->setResourceGroupDirectory( "schemes", "data\\gui\\schemes\\" );
		p_rp->setResourceGroupDirectory( "looknfeel", "data\\gui\\looknfeel\\" );
		p_rp->setResourceGroupDirectory( "layouts", "data\\gui\\layouts\\" );

		// Schema laden
		CEGUI::SchemeManager::getSingleton().loadScheme( "RRLook.scheme" );

		// Schriftart laden
		CEGUI::FontManager::getSingleton().createFont( "David-14.font" );
		CEGUI::FontManager::getSingleton().createFont( "David-22.font" );
		CEGUI::FontManager::getSingleton().createFont( "David-32.font" );

		// Standardschriftart und Mauszeiger setzen
		p_system->setDefaultFont( "David-14" );
		p_system->setDefaultMouseCursor( "RRLook", "MouseArrow" );

		// Wurzelfenster erzeugen
		CEGUI::WindowManager& wndMgr = CEGUI::WindowManager::getSingleton();
		p_root = wndMgr.createWindow( "DefaultWindow", "Root" );
		p_system->setGUISheet( p_root );
	}
	catch( CEGUI::Exception& e )
	{
		setError( 1 );
		engine.getLogger().error( std::string( "CEGUI-Exception: " ) + e.getMessage().c_str(), LOGGER_AUTO );
		return;
	}

	engine.connectPostRender( this, &GUI::onPostRender );
	engine.connectSDLEvent( this, &GUI::onSDLEvent );
	engine.connectUpdate( this, &GUI::onUpdate );
}

GUI::~GUI()
{
	// CEGUI herunterfahren
	delete p_system;
	delete p_renderer;
}

void GUI::loadLayout( const std::string& filename,
					  const std::string& parent )
{
	// Layout laden
	CEGUI::Window* p_wnd = getWindowManager().loadWindowLayout( filename );
	p_wnd->hide();

	CEGUI::Window* p_parent = getWindow( parent );
	if( !p_parent )
	{
		// neues Wurzelfenster erzeugen
		CEGUI::WindowManager& wndMgr = CEGUI::WindowManager::getSingleton();
		p_parent = wndMgr.createWindow( "DefaultWindow", parent );
		p_parent->hide();
		p_root->addChildWindow( p_parent );
	}

	p_parent->addChildWindow( p_wnd );
}

CEGUI::WindowManager& GUI::getWindowManager() const
{
	return CEGUI::WindowManager::getSingleton();
}

CEGUI::Window* GUI::getWindow( const std::string& name ) const
{
	CEGUI::WindowManager& wndMgr = CEGUI::WindowManager::getSingleton();
	if( wndMgr.isWindowPresent( name ) ) return wndMgr.getWindow( name );
	else return 0;
}

CEGUI::Window* GUI::getRoot() const
{
	return p_root;
}

void GUI::showAndFocusWindow( const std::string& name )
{
	CEGUI::Window* p_window = getWindow( name );
	if( p_window )
	{
		p_window->show();
		p_window->activate();
	}
}

bool GUI::areWindowsVisible( const CEGUI::Window* p_window ) const
{
	if( !p_window ) p_window = p_root;

	if( !p_window->isVisible() ) return false;
	else if( p_window->getType() != "DefaultWindow" ) return true;

	size_t n = p_window->getChildCount();
	for( size_t i = 0; i < n; i++ )
	{
		if( areWindowsVisible( p_window->getChildAtIdx( i ) ) ) return true;
	}

	return false;
}

void GUI::onPostRender()
{
	getEngine().getVideo().setTexture( 0 );

	// GUI rendern
	p_system->renderGUI();
}

void GUI::onSDLEvent( const SDL_Event& e )
{
	// Eingaben an CEGUI weiterleiten
	switch( e.type )
	{
	case SDL_KEYDOWN:
		p_system->injectKeyDown( convertSDLKeyToCEGUIKey( e.key.keysym.sym, e.key.keysym.scancode ) );
		if( e.key.keysym.unicode != 0 ) p_system->injectChar( e.key.keysym.unicode );
		break;
	case SDL_KEYUP:
		p_system->injectKeyUp( convertSDLKeyToCEGUIKey( e.key.keysym.sym, e.key.keysym.scancode ) );
		break;
	case SDL_MOUSEMOTION:
		p_system->injectMousePosition( static_cast<float>( e.motion.x ), static_cast<float>( e.motion.y ) );
		break;
	case SDL_MOUSEBUTTONDOWN:
		switch( e.button.button )
		{
		case SDL_BUTTON_LEFT:
			p_system->injectMouseButtonDown( CEGUI::LeftButton );
			break;
		case SDL_BUTTON_MIDDLE:
			p_system->injectMouseButtonDown( CEGUI::MiddleButton );
			break;
		case SDL_BUTTON_RIGHT:
			p_system->injectMouseButtonDown( CEGUI::RightButton );
			break;
		case SDL_BUTTON_WHEELDOWN:
			p_system->injectMouseWheelChange( -1 );
			break;
		case SDL_BUTTON_WHEELUP:
			p_system->injectMouseWheelChange( 1 );
			break;
		}
		break;
	case SDL_MOUSEBUTTONUP:
		switch( e.button.button )
		{
		case SDL_BUTTON_LEFT:
			p_system->injectMouseButtonUp( CEGUI::LeftButton );
			break;
		case SDL_BUTTON_MIDDLE:
			p_system->injectMouseButtonUp( CEGUI::MiddleButton );
			break;
		case SDL_BUTTON_RIGHT:
			p_system->injectMouseButtonUp( CEGUI::RightButton );
			break;
		}
		break;
	}
}

void GUI::onUpdate()
{
	// GUI aktualisieren
	p_system->injectTimePulse( getEngine().getTickDurationInSeconds() );

	// Der Mauszeiger soll nur angezeigt werden, wenn Fenster sichtbar sind.
	CEGUI::MouseCursor::getSingleton().setVisible( areWindowsVisible() );
}