#include <PCH.h>
#include <NeuralNet.h>
#include <floatfann.h>

NeuralNet::NeuralNet( ResourceManager& manager,
					  const std::string& filename,
					  bool unique,
					  const ParameterMap& options ) : Resource( manager, filename, unique, options ),
													  p_fann( 0 )
{
	Logger& logger = manager.getEngine().getLogger();

	p_fann = fann_create_from_file( filename.c_str() );
	if( !p_fann )
	{
		setError( 1 );
		std::ostringstream msg;
		msg << filename << ": Laden des Neuronalen Netzes fehlgeschlagen";
		logger.error( msg.str(), LOGGER_AUTO );
		return;
	}

	std::ostringstream msg;
	msg << filename << ": " << getNumInputs() << " -> " << getNumOutputs() << ", Neuronen: " << getNumNeurons() << ", Verbindungen: " << getNumConnections();
	logger.finer( msg.str(), LOGGER_AUTO );
}

NeuralNet::~NeuralNet()
{
	if( p_fann )
	{
		fann_destroy( p_fann );
	}
}

unsigned int NeuralNet::getNumInputs() const
{
	return fann_get_num_input( p_fann );
}

unsigned int NeuralNet::getNumOutputs() const
{
	return fann_get_num_output( p_fann );
}

unsigned int NeuralNet::getNumNeurons() const
{
	return fann_get_total_neurons( p_fann );
}

unsigned int NeuralNet::getNumConnections() const
{
	return fann_get_total_connections( p_fann );
}

float* NeuralNet::run( float* p_input )
{
	return fann_run( p_fann, p_input );
}