#include <PCH.h>
#include <Obj_Accelerator.h>
#include <Obj_Player.h>

Obj_Accelerator::Obj_Accelerator( Level& level,
								  const TiXmlElement* p_root ) : GameObject( level, p_root, false ),
																 active( 0.0f )
{
	setMaterial( "Trigger" );

	setCollisionBits( CB_NONE, CB_PLAYER | CB_ENEMY | CB_OBJECT | CB_PROJECTILE );

	// Radius lesen
	radius = 4.0f;
	p_root->QueryFloatAttribute( "radius", &radius );

	// Zielgeschwindigkeit lesen
	targetSpeed = 50.0f;
	p_root->QueryFloatAttribute( "targetSpeed", &targetSpeed );

	// Strke lesen
	strength = 1.0f;
	p_root->QueryFloatAttribute( "strength", &strength );

	// Aktiviert?
	int activated_ = 1;
	p_root->QueryIntAttribute( "activated", &activated_ );
	activated = activated_ ? true : false;

	addCircle( Vec2f( 0.0f, 0.0f ), radius );
	setBoundingCircle( Vec2f( 0.0f, 0.0f ), radius );

	// Sound laden
	sound = env.engine.getSoundManager().get( "data\\accelerator.ogg" );
	sound->setDefaultSoundGroupName( "SFX" );
	sound->setMaxPlayingInstances( 4 );
}

Obj_Accelerator::~Obj_Accelerator()
{
}

void Obj_Accelerator::update()
{
	GameObject::update();

	lastObjectsOnMe = currentObjectsOnMe;
	currentObjectsOnMe.clear();

	adjustSmooth( active, 0.0f, 10.0f, env.dt );
}

void Obj_Accelerator::render( RenderPass pass )
{
	if( pass == RP_BACKGROUND_1 )
	{
		// Sprite rendern
		glPushMatrix();
		setupGLMatrix();
		if( activated ) glColor4f( 1.0f, 1.0f, 1.0f, 0.35f + active * 0.5f + active * 0.5f * sinf( level.getTime() * 50.0f ) );
		else glColor4f( 0.5f, 0.5f, 0.5f, 0.35f );
		video.drawSprite( Vec2i( 128, 864 ), Vec2i( 128, 128 ), 2.0f * Vec2f( radius ) );
		glPopMatrix();
	}
}

void Obj_Accelerator::onCollision( GameObject* p_other,
								   CollisionInfo& info )
{
	GameObject::onCollision( p_other, info );

	if( !activated ) return;

	if( p_other )
	{
		dBodyID body = p_other->getBody();
		if( body )
		{
			const Vec2f relVelocity( absDirectionToRelDirection( p_other->getVelocity() ) );
			const float mass = p_other->getTotalMass();
			Vec2f relForce( ( Vec2f( 0.0f, -targetSpeed ) - relVelocity ) * mass * strength * 10.0f * active );
			if( relForce.y > 0.0f ) relForce.y = 0.0f;
			const Vec2f absForce( relDirectionToAbsDirection( relForce ) );

			dBodyAddForce( body, absForce.x, absForce.y, 0.0 );

			currentObjectsOnMe.insert( p_other );
			if( lastObjectsOnMe.find( p_other ) == lastObjectsOnMe.end() )
			{
				// Sound abspielen
				SoundInstance* p_soundInst = sound->createInstance();
				if( p_soundInst )
				{
					p_soundInst->attachTo( this );
					p_soundInst->setPitch( randFloat( 0.75f, 0.85f ) );
					p_soundInst->play();
				}
			}

			adjustSmooth( active, 1.0f, 10.0f, env.dt );
		}
	}
}