#include <PCH.h>
#include <Obj_BulletHole.h>

/*
 * Typenbersicht:
 * 1: blauer Tintenfleck
 * 2: grner Tintenfleck
 * 3: Brandfleck
 *
 */

Obj_BulletHole::Obj_BulletHole( Level& level,
							    int type ) : GameObject( level, 0, false ),
											 type( type )
{
	if( type == 1 || type == 2 )
	{
		sizeOnTexture = Vec2i( 64, 64 );
		texCoords = Vec2i( randInt( 0, 3 ) * 64, 128 );
		if( type == 2 ) texCoords.y += 64;

		size = randFloat( 1.5f, 2.5f );
		timeToLive = randFloat( 5.0f, 6.0f );
		setAngle( randFloat( 0.0f, 6.282f ) );
	}
	else if( type == 3 )
	{
		sizeOnTexture = Vec2i( 64, 64 );
		texCoords = Vec2i( randInt( 0, 3 ) * 64, 256 );

		size = randFloat( 2.5f, 3.5f );
		timeToLive = randFloat( 600.0f, 1200.0f );
		setAngle( randFloat( -0.05f, 0.05f ) + randInt( 0, 1 ) ? 3.141f : 0.0f );
	}

	setBoundingCircle( Vec2f( 0.0f, 0.0f ), 2.0f );
}

Obj_BulletHole::~Obj_BulletHole()
{
}

void Obj_BulletHole::update()
{
	GameObject::update();

	timeToLive -= env.dt;

	if( timeToLive <= 0.0f )
	{
		kill();
		return;
	}
}

void Obj_BulletHole::render( RenderPass pass )
{
	if( pass == RP_BACKGROUND_2 )
	{
		// Sprite rendern
		if( type == 3 )
		{
			glPushMatrix();
			setupGLMatrix();
			glScalef( size, size, 1.0f );
			glColor4f( 1.0f, 1.0f, 1.0f, timeToLive );
			video.drawSprite( texCoords, sizeOnTexture );
			glPopMatrix();
		}
	}
	else if( pass == RP_BACKGROUND_3 )
	{
		// Sprite rendern
		if( type == 1 || type == 2 )
		{
			glPushMatrix();
			setupGLMatrix();
			float scaling = size * min( timeToLive + 0.25f, 1.0f );
			glScalef( scaling, scaling, 1.0f );
			glColor4f( 1.0f, 1.0f, 1.0f, timeToLive );
			video.drawSprite( texCoords, sizeOnTexture );
			glPopMatrix();
		}
	}
}