#include <PCH.h>
#include <Obj_Checkpoint.h>
#include <Obj_Player.h>

Obj_Checkpoint::Obj_Checkpoint( Level& level,
								const TiXmlElement* p_root ) : GameObject( level, p_root, false ),
															   p_neonSoundInst( 0 ),
															   flash( 0.0f ),
															   activated( false ),
															   activationTime( 0.0f )
{
	setMaterial( "Trigger" );

	// Der Checkpoint kann nur mit dem Spieler "kollidieren".
	setCollisionBits( CB_NONE, CB_PLAYER );

	addCircle( Vec2f( 0.0f, 0.0f ), 3.0f );
	setBoundingCircle( Vec2f( 0.0f, 0.0f ), 5.0f );

	// Nummer lesen
	int number_ = 0;
	p_root->QueryIntAttribute( "number", &number_ );
	number = static_cast<unsigned int>( number_ );

	// Schilddauer lesen
	shieldDuration = 3.0f;
	p_root->QueryFloatAttribute( "shieldDuration", &shieldDuration );

	// Gesundheit und Treibstoff fr den wiedergeborenen Spieler lesen
	healthAmount = 100.0f;
	p_root->QueryFloatAttribute( "healthAmount", &healthAmount );
	fuelAmount = 100.0f;
	p_root->QueryFloatAttribute( "fuelAmount", &fuelAmount );

	// Sounds laden
	sound = env.engine.getSoundManager().get( "data\\checkpoint.ogg" );
	sound->setDefaultSoundGroupName( "SFX" );
	neonSound = env.engine.getSoundManager().get( "data\\neon2.ogg" );
	neonSound->setDefaultSoundGroupName( "SFX" );
	neonSound->setDefaultDistances( 5.0f, 10000.0f );
}

Obj_Checkpoint::~Obj_Checkpoint()
{
}

void Obj_Checkpoint::update()
{
	GameObject::update();

	if( flash <= 0.0f )
	{
		flash -= env.dt;
		if( flash <= -0.25f && activated )
		{
			flash = 5.0f;
			p_neonSoundInst = neonSound->createInstance();
			if( p_neonSoundInst )
			{
				attachChild( p_neonSoundInst, false );
				p_neonSoundInst->setVolume( randFloat( 0.7f, 0.8f ) );
				p_neonSoundInst->setPitch( randFloat( 1.0f, 1.1f ) );
				p_neonSoundInst->play();
			}
		}
	}
	else
	{
		adjustSmooth( flash, -0.1f, 5.0f, env.dt );
	}
}

void Obj_Checkpoint::render( RenderPass pass )
{
	if( pass == RP_SHADOW || pass == RP_OBJECT_1 )
	{
		// Sprite rendern
		glPushMatrix();
		setupGLMatrix();

		if( pass == RP_OBJECT_1 )
		{
			// Wenn der Checkpoint noch nicht aktiviert wurde, ist er dunkler.
			float brightness = activated ? 1.0f : 0.75f;
			glColor3fv( Vec3f( brightness ) );
		}

		video.drawSprite( Vec2i( 256, 864 ), Vec2i( 192, 160 ), Vec2f( 6.0f, 5.0f ) );

		if( pass == RP_OBJECT_1 && flash > 0.0f )
		{
			// Leuchten rendern
			video.setupAlphaBlending( true, GL_SRC_ALPHA, GL_ONE );
			glColor3f( flash + randFloat( -0.1f, 0.1f ), flash + randFloat( -0.1f, 0.1f ), flash + randFloat( -0.1f, 0.1f ) );
			glTranslatef( -0.6f, -1.7f, 0.0f );
			video.drawSprite( Vec2i( 320, 320 ), Vec2i( 160, 64 ), Vec2f( 5.0f, 2.0f ) );
			video.setupAlphaBlending( true, GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
		}

		glPopMatrix();
	}
}

void Obj_Checkpoint::onCollision( GameObject* p_other,
								  CollisionInfo& info )
{
	GameObject::onCollision( p_other, info );

	if( activated ) return;

	if( p_other && p_other->getType() == "Player" )
	{
		// Der Checkpoint ist jetzt aktiviert!
		activated = true;
		activationTime = level.getTime();

		// Text anzeigen
		level.addText( "OK!", getPosition(), Vec4f( 0.5f, 0.75f, 1.0f, 1.0f ), 3.0f, 2.0f );

		// Sound abspielen
		SoundInstance* p_soundInst = sound->createInstance();
		if( p_soundInst )
		{
			p_soundInst->attachTo( this, false );
			p_soundInst->play();
		}
	}
}