#include <PCH.h>
#include <Obj_Debris.h>

Obj_Debris::Obj_Debris( Level& level,
					    DebrisType type ) : GameObject( level, 0, true ),
											type( type ),
											timeToLive( 0.0f ),
											scaling( 1.0f ),
											p_smokeEmitter( 0 ),
											particleRate( 50.0f ),
											particleRateDecay( 5.0f )
{
	setCollisionBits( CB_NONE, CB_STATIC );

	timeToLive = randFloat( 1.0f, 5.0f );
	texCoords = Vec2i( 256, 128 ) + Vec2i( randInt( 0, 3 ) * 32, randInt( 0, 1 ) * 32 );
	if( type == DT_PLAYER ) texCoords.y += 64;
	else if( type == DT_ENEMY_AASTATION ) texCoords.y += 128;
	else if( type == DT_ENERGYNODE ) texCoords.x += 128;
	sizeOnTexture = Vec2i( 32, 32 );
	scaling = randFloat( 0.25f, 2.0f );

	addCircle( Vec2f( 0.0f, 0.0f ), 0.25f * scaling );
	setTotalMass( 0.05f * scaling );
	setFriction( 0.75f, 0.5f );

	// Emitter erzeugen
	p_smokeEmitter = level.getPS().createSimpleEmitter();
	p_smokeEmitter->addParticleStyle( 1 );
	p_smokeEmitter->setParticleTimeToLive( 0.3f, 0.2f );
	p_smokeEmitter->setParticlePositionRadius( 0.25f );
	p_smokeEmitter->setParticleSpeed( 3.0f, 2.0f );
	p_smokeEmitter->setParticleStartColor( Vec4f( 1.0f, 0.5f, 0.0f, 1.0f ), Vec4f( 0.1f, 0.1f, 0.1f, 0.75f ) );
	p_smokeEmitter->setParticleEndColor( Vec4f( 3.0f, 3.0f, 3.0f, 0.0f ), Vec4f( 0.1f, 0.1f, 0.1f, 0.0f ) );
	p_smokeEmitter->setParticleStartSize( 1.0f, 0.1f );
	p_smokeEmitter->setParticleEndSize( 2.0f, 0.5f );
	p_smokeEmitter->setParticleGravity( 0.0f, 0.1f );
	p_smokeEmitter->setParticleAnimFrame( 0.0f, 10.0f );
	attachChild( p_smokeEmitter, false );

	particleRate = randFloat( 80.0f, 100.0f );
	particleRateDecay = randFloat( 40.0f, 50.0f );

	setBoundingCircle( Vec2f( 0.0f, 0.0f ), 2.0f );
}

Obj_Debris::~Obj_Debris()
{
	if( p_smokeEmitter ) level.getPS().destroyEmitter( p_smokeEmitter );
}

void Obj_Debris::update()
{
	GameObject::update();

	if( p_smokeEmitter )
	{
		p_smokeEmitter->setRate( particleRate );
		particleRate -= env.dt * particleRateDecay;
	}

	if( timeToLive <= 0.0f )
	{
		kill();
		return;
	}

	timeToLive -= env.dt;
}

void Obj_Debris::render( RenderPass pass )
{
	if( pass == RP_SHADOW || pass == RP_OBJECT_2 )
	{
		// Sprite rendern
		if( timeToLive < 1.0f ) glColor4fv( level.getPassColor() * Vec4f( 1.0f, 1.0f, 1.0f, timeToLive ) );

		glPushMatrix();
		setupGLMatrix();
		glScalef( scaling, scaling, 1.0f );
		video.drawSprite( texCoords, sizeOnTexture );
		glPopMatrix();
	}
}