#include <PCH.h>
#include <Obj_Enemy_Eraser.h>
#include <Obj_Projectile.h>

Obj_Enemy_Eraser::Obj_Enemy_Eraser( Level& level,
								    const TiXmlElement* p_root ) : Obj_Enemy( level, p_root, true ),
																   p_engineSoundInst( 0 )
{
	setGravityInfluence( false );
	setMaxHealth( 5.0f );
	setHealth( 5.0f );
	setMaterial( "Rubber" );

	std::vector<Vec2f> vertices;
	vertices.push_back( Vec2f( -2.0f, -0.3f ) );
	vertices.push_back( Vec2f( 2.0f, -0.3f ) );
	vertices.push_back( Vec2f( 2.0f, 0.3f ) );
	vertices.push_back( Vec2f( -2.0f, 0.3f ) );
	addPolyLine( vertices, true, 0.5f );

	setBoundingCircle( Vec2f( 0.0f, 0.0f ), 2.5f );

	setTotalMass( 1.0f );
	setFriction( 0.5f, 0.001f );
	scaling = 6.0f;
	range = 25.0f;

	texCoords = Vec2i( 256, 0 );
	sizeOnTexture = Vec2i( 128, 128 );

	// Parameter konfigurieren
	setAISimpleMode();
	registerParameter( "SteeringX", PT_OUTPUT, -1.0f, 1.0f );
	registerParameter( "SteeringY", PT_OUTPUT, -1.0f, 1.0f );
	registerParameter( "Fire", PT_OUTPUT_USER_ONLY );
	bindKey( KEY_LEFT, "SteeringX", -1.0f );
	bindKey( KEY_RIGHT, "SteeringX", 1.0f );
	bindKey( KEY_UP, "SteeringY", -1.0f );
	bindKey( KEY_DOWN, "SteeringY", 1.0f );
	bindKey( KEY_SPACE, "Fire", 1.0f );

	// Waffen konfigurieren
	setupWeapon( WT_INK_BLASTER_2, Vec2f( 0.0f, 0.0f ) );
	addAmmo( WT_INK_BLASTER_2, ~0 );
	selectWeapon( WT_INK_BLASTER_2 );

	// Sound fr den Antrieb laden
	SoundManager& soundMgr = env.engine.getSoundManager();
	SoundHandle engineSound = soundMgr.get( "data\\engine1.ogg" );
	engineSound->setDefaultSoundGroupName( "SFX" );
	engineSound->setDefaultDistances( 10.0f, 10000.0f );
	p_engineSoundInst = engineSound->createInstance();
	if( p_engineSoundInst )
	{
		p_engineSoundInst->attachTo( this, false );
		p_engineSoundInst->setVolume( 0.0f );
		p_engineSoundInst->play( true );
	}
}

Obj_Enemy_Eraser::~Obj_Enemy_Eraser()
{
	if( p_engineSoundInst ) p_engineSoundInst->stop();
}

void Obj_Enemy_Eraser::update()
{
	Obj_Enemy::update();

	if( getHealth() <= 0.0f )
	{
		level.addPlayerScore( 1000, getPosition() );
		explode( randInt( 15, 20 ), DT_ENEMY_ERASER, 2.0f );
		kill();
		return;
	}

	if( isFrozen() ) return;

	float steeringX = getParameter( "SteeringX" );
	float steeringY = getParameter( "SteeringY" );

	dBodyAddForce( getBody(), steeringX * 7.5f, steeringY * 7.5f, 0.0 );

	float targetAngle = steeringX;
	float angleDiff = targetAngle - getAngle();
	dBodyAddTorque( getBody(), 0.0, 0.0, angleDiff * 10.0f );

	if( getParameter( "Fire" ) > 0.0f ) fireAt( getTarget() );

	if( p_engineSoundInst )
	{
		// Antriebssound verarbeiten
		float thrust = Vec2f( steeringX, steeringY ).length();
		float soundFactor = 0.6f * thrust * ( thrust + 0.02f * getVelocity().length() );
		p_engineSoundInst->setVolumeTarget( soundFactor, 2.5f );
		p_engineSoundInst->setPitchTarget( 0.25f + soundFactor, 2.5f );
	}
}

void Obj_Enemy_Eraser::think()
{
	Obj_Enemy::think();

	const Vec2f myPosition( getPosition() );
	Vec2f steering( movementTargetPosition - myPosition );
	if( fabsf( steering.x ) > fabsf( steering.y ) ) steering /= fabsf( steering.x );
	else steering /= fabsf( steering.y );
	setParameter( "SteeringX", steering.x + randFloat( -0.1f, 0.1f ) );
	setParameter( "SteeringY", steering.y + randFloat( -0.1f, 0.1f ) );
}

void Obj_Enemy_Eraser::onKill()
{
	if( p_engineSoundInst ) p_engineSoundInst->setMuted( true );
}

void Obj_Enemy_Eraser::onRebirth()
{
	if( p_engineSoundInst ) p_engineSoundInst->setMuted( false );
}

void Obj_Enemy_Eraser::addAvoidanceAreas()
{
	level.addAvoidanceCircle( 1, this, getTransform(), Vec2f( 0.0f, 0.0f ), 4.0f );
}