#include <PCH.h>
#include <Obj_LevelTarget.h>
#include <Obj_Player.h>

Obj_LevelTarget::Obj_LevelTarget( Level& level,
								  const TiXmlElement* p_root ) : GameObject( level, p_root, false ),
																 flash( 0.0f ),
																 p_soundInst( 0 )
{
	setMaxHealth( FLT_MAX );
	setHealth( FLT_MAX );
	setMaterial( "Metal" );

	// Das Ziel kann mit Objekten kollidieren, aber nicht mit Wnden.
	setCollisionBits( CB_STATIC | CB_OBJECT, ~CB_STATIC );

	addCircle( Vec2f( 0.5f, 0.0f ), 1.5f );

	setBoundingCircle( Vec2f( 0.0f, 0.0f ), 20.0f );

	// Sound laden, Soundinstanz erzeugen und an das Objekt hngen
	SoundHandle sound = env.engine.getSoundManager().get( "data\\jammer.ogg" );
	sound->setDefaultSoundGroupName( "SFX" );
	sound->setDefaultDistances( 5.0f, 10000.0f );
	p_soundInst = sound->createInstance();
	if( p_soundInst )
	{
		p_soundInst->attachTo( this, false );
		p_soundInst->setVolume( 0.0f );
		p_soundInst->play( true );
	}
}

Obj_LevelTarget::~Obj_LevelTarget()
{
	if( p_soundInst )
	{
		p_soundInst->stop();
	}
}

void Obj_LevelTarget::update()
{
	GameObject::update();

	if( getHealth() <= 0.0f ) destroy();

	if( flash <= 0.0f )
	{
		flash -= env.dt;
		if( flash <= -0.25f )
		{
			flash = 0.0001f;
			if( p_soundInst )
			{
				p_soundInst->setVolume( randFloat( 0.9f, 1.0f ) );
				p_soundInst->setPitch( randFloat( 1.4f, 1.6f ) );
				p_soundInst->setVolumeTarget( 0.25f, 3.0f );
				p_soundInst->setPitchTarget( 1.5f, 3.0f );
			}
		}
	}
	else
	{
		flash += env.dt;
		if( flash >= 1.0f ) flash = 0.0f;
	}
}

void Obj_LevelTarget::render( RenderPass pass )
{
	if( pass == RP_SHADOW || pass == RP_OBJECT_1 )
	{
		// Sprite rendern
		glPushMatrix();
		setupGLMatrix();

		if( pass == RP_OBJECT_1 && flash > 0.0f )
		{
			// Strsignal rendern
			const float t = level.getTime();
			const float alpha = 1.0f - flash;
			const float size = 1.0f + 32.0f * flash * ( 1.0f + 0.05f * sinf( 100.0f * t ) );
			const float angle = t + 180.0f * flash;
			glColor4f( 1.0f, randFloat( 0.25f, 0.75f ), randFloat( 0.0f, 0.25f ), alpha * alpha * alpha * randFloat( 0.5f, 1.5f ) );
			glPushMatrix();
			glRotatef( angle, 0.0f, 0.0f, 1.0f );
			video.drawSprite( Vec2i( 288, 640 ), Vec2i( 224, 224 ), Vec2f( size ) );
			glPopMatrix();
			glColor4fv( level.getPassColor() );
		}

		video.drawSprite( Vec2i( 384, 512 ), Vec2i( 128, 128 ), Vec2f( 4.0f, 4.0f ) );

		glPopMatrix();
	}
}

void Obj_LevelTarget::onCollision( GameObject* p_other,
								   CollisionInfo& info )
{
	GameObject::onCollision( p_other, info );
	if( info.passThrough ) return;

	if( p_other && p_other->getType() == "Player" )
	{
		if( info.frontal && info.p_params )
		{
			// Position der Kollision relativ zum Spieler bestimmen
			const Vec2f relPos( p_other->absPositionToRelPosition( info.collisionPoint ) );

			// Entfernung zur Spitze bestimmen
			const float dist = ( relPos - Vec2f( 0.0f, -1.9f ) ).length();
			if( dist < 1.0f )
			{
				// Der Spieler wird ebenfalls zerstrt.
				Obj_Player* p_player = static_cast<Obj_Player*>( p_other );
				p_player->addHealth( -FLT_MAX );
				p_player->addBonusScore();

				setHealth( 0.0f );
			}
		}
	}
}

void Obj_LevelTarget::destroy()
{
	// Ziel zerstrt!
	SoundHandle sound = env.engine.getSoundManager().get( "data\\explosion2.ogg" );
	SoundInstance* p_soundInst = sound->createInstance( true );
	if( p_soundInst )
	{
		p_soundInst->setPosition( getPosition() );
		p_soundInst->play();
	}

	for( int i = 1; i <= 4; i++ )
	{
		explode( randInt( 8, 16 ), DT_RANDOM,
				 randFloat( 2.0f, 4.0f ), randFloat( 1.0f, 4.0f ), true,
				 0, 0.0f, 0.0f );
	}

	level.addPlayerScore( 5000, getPosition() );

	kill();
}