#include <PCH.h>
#include <Obj_Mine.h>
#include <Damagable.h>

Obj_Mine::Obj_Mine( Level& level,
					const TiXmlElement* p_root ) : GameObject( level, p_root, true )
{
	setMaterial( "Metal" );

	// Dynamisch oder statisch?
	int dynamic_ = 0;
	p_root->QueryIntAttribute( "dynamic", &dynamic_ );
	dynamic = dynamic_ ? true : false;

	// Aktiviert?
	int activated_ = 1;
	p_root->QueryIntAttribute( "activated", &activated_ );
	activated = activated_ ? true : false;

	if( !dynamic ) deleteBody();

	// Die Mine kann mit Objekten kollidieren, aber nicht mit Wnden.
	setCollisionBits( CB_STATIC | CB_OBJECT, ~CB_STATIC );
	addCircle( Vec2f( 0.0f, 0.0f ), 1.0f );
	setBoundingCircle( Vec2f( 0.0f, 0.0f ), 1.5f );

	if( dynamic ) setTotalMass( 0.1f );

	t = randFloat( 0.0f, 6.2832f );
	if( dynamic ) setAngle( t );
}

Obj_Mine::~Obj_Mine()
{
}

void Obj_Mine::update()
{
	GameObject::update();

	if( !dynamic ) setAngle( t + level.getTime() );
}

void Obj_Mine::render( RenderPass pass )
{
	if( pass == RP_SHADOW || pass == RP_OBJECT_1 )
	{
		// Sprite rendern
		glPushMatrix();
		setupGLMatrix();

		if( !activated && pass == RP_OBJECT_1 ) glColor3f( 0.5f, 0.5f, 0.5f );
		video.drawSprite( Vec2i( 448, 864 ), Vec2i( 64, 64 ), Vec2f( 2.0f, 2.0f ) );

		glPopMatrix();
	}
}

void Obj_Mine::onCollision( GameObject* p_other,
							CollisionInfo& info )
{
	GameObject::onCollision( p_other, info );

	if( !activated ) return;

	if( p_other )
	{
		Damagable* p_damagable = dynamic_cast<Damagable*>( p_other );
		if( p_damagable )
		{
			// Das andere Objekt wird zerstrt.
			p_damagable->addHealth( -FLT_MAX );

			detonate();
		}
	}
}

void Obj_Mine::addAvoidanceAreas()
{
	level.addAvoidanceCircle( 1000, this, getTransform(), Vec2f( 0.0f, 0.0f ), 5.0f );
}

void Obj_Mine::detonate()
{
	// Die Mine explodiert.
	explode( 0, DT_NONE, 0.5f, 0.75f, false, 3, randFloat( 0.9f, 0.95f ), randFloat( 0.75f, 0.85f ) );

	kill();
}