#include <PCH.h>
#include <Obj_Player.h>
#include <Obj_Projectile.h>

Obj_Player::Obj_Player( Level& level,
						const TiXmlElement* p_root ) : GameObject( level, p_root, true ),
													   Controllable( level.getEnv() ),
													   Armed( level.getEnv() ),
													   thrust( 0.0f ),
													   fuel( 100.0f ),
													   shield( 0.0f ),
													   mouseAiming( false ),
													   score( 0 ),
													   p_smokeEmitter( 0 ),
													   p_fireEmitter( 0 ),
													   p_engineSoundInst( 0 ),
													   p_shieldSoundInst( 0 )
{
	setTurnToZombie( true );
	setMaxHealth( 100.0f );
	setHealth( 100.0f );
	setMaterial( "Player" );

	setUserMode();

	// Der Spieler kann mit allen Objekten kollidieren.
	setCollisionBits( CB_PLAYER, CB_ALL );

	std::vector<Vec2f> vertices;
	vertices.push_back( Vec2f( 0.0f, -1.5f ) );
	vertices.push_back( Vec2f( 0.5f, 1.2f ) );
	vertices.push_back( Vec2f( 0.85f, 1.5f ) );
	vertices.push_back( Vec2f( -0.85f, 1.5f ) );
	vertices.push_back( Vec2f( -0.5f, 1.2f ) );
	addPolyLine( vertices, true, 0.5f );

	setBoundingCircle( Vec2f( 0.0f, 0.0f ), 2.5f );

	setTotalMass( 1.0f );
	setFriction( 0.75f, 0.001f );

	// Parameter konfigurieren
	registerParameter( "Thrust", PT_OUTPUT );
	registerParameter( "Steering", PT_OUTPUT, -1.0f, 1.0f );
	registerParameter( "Fire", PT_OUTPUT_USER_ONLY );
	bindKey( KEY_UP, "Thrust", 1.0f );
	bindKey( KEY_w, "Thrust", 1.0f );
	bindKey( KEY_LEFT, "Steering", -1.0f );
	bindKey( KEY_a, "Steering", -1.0f );
	bindKey( KEY_RIGHT, "Steering", 1.0f );
	bindKey( KEY_d, "Steering", 1.0f );
	bindKey( KEY_SPACE, "Fire", 1.0f );

	// Waffen konfigurieren
	weaponPlatform.attachTo( this, false );
	weaponPlatform.setPosition( Vec2f( 0.0f, -1.5f ) );
	p_defaultWeaponParent = &weaponPlatform;
	setupWeapon( WT_INK_BLASTER );
	addAmmo( WT_INK_BLASTER, ~0 );
	setupWeapon( WT_MISSILE );
	setupWeapon( WT_CANNON );
	selectWeapon( WT_INK_BLASTER );

	// Emitter fr Rauch erzeugen
	p_smokeEmitter = level.getPS().createSimpleEmitter();
	p_smokeEmitter->setPosition( Vec2f( 0.0f, 1.9f ) );
	p_smokeEmitter->addParticleStyle( 1 );
	p_smokeEmitter->setParticleTimeToLive( 1.0f, 0.25f );
	p_smokeEmitter->setParticlePositionRadius( 0.1f );
	p_smokeEmitter->setParticleDirection( Vec2f( 0.0f, -1.0f ), 0.15f );
	p_smokeEmitter->setParticleSpeed( 5.0f, 0.1f );
	p_smokeEmitter->setParticleStartColor( Vec4f( 1.0f, 0.5f, 0.0f, 1.0f ), Vec4f( 0.1f, 0.1f, 0.1f, 0.5f ) );
	p_smokeEmitter->setParticleEndColor( Vec4f( 2.0f, 2.0f, 2.0f, 0.0f ), Vec4f( 0.1f, 0.1f, 0.1f, 0.0f ) );
	p_smokeEmitter->setParticleStartSize( 0.75f, 0.1f );
	p_smokeEmitter->setParticleEndSize( 3.0f, 0.2f );
	p_smokeEmitter->setParticleGravity( 0.0f, 0.1f );
	p_smokeEmitter->setParticleAnimFrame( 0.0f, 10.0f );
	attachChild( p_smokeEmitter, false );

	// Emitter fr Stichflamme erzeugen
	p_fireEmitter = level.getPS().createSimpleEmitter();
	p_fireEmitter->setPosition( Vec2f( 0.0f, 2.0f ) );
	p_fireEmitter->addParticleStyle( 2 );
	p_fireEmitter->setParticleTimeToLive( 0.05f, 0.01f );
	p_fireEmitter->setParticlePositionRadius( 0.5f );
	p_fireEmitter->setParticleDirection( Vec2f( 0.0f, -1.0f ), 0.25f );
	p_fireEmitter->setParticleSpeed( 5.0f, 0.1f );
	p_fireEmitter->setParticleStartColor( Vec4f( 1.0f, 1.0f, 1.0f, 1.0f ), Vec4f( 0.1f, 0.1f, 0.1f, 0.5f ) );
	p_fireEmitter->setParticleEndColor( Vec4f( 1.0f, 1.0f, 1.0f, 0.0f ), Vec4f( 0.1f, 0.1f, 0.1f, 0.0f ) );
	p_fireEmitter->setParticleStartSize( 4.0f, 0.5f );
	p_fireEmitter->setParticleEndSize( 8.0f, 0.5f );
	p_fireEmitter->setParticleGravity( 0.0f, 0.1f );
	p_fireEmitter->setParticleAnimFrame( 0.0f, 10.0f );
	attachChild( p_fireEmitter, false );

	// Emitter fr Rauch bei greren Schden erzeugen
	p_damageEmitter = level.getPS().createSimpleEmitter();
	p_damageEmitter->addParticleStyle( 1 );
	p_damageEmitter->setParticleTimeToLive( 1.0f, 0.25f );
	p_damageEmitter->setParticlePositionRadius( 1.5f );
	p_damageEmitter->setParticleSpeed( 2.0f, 0.5f );
	p_damageEmitter->setParticleStartColor( Vec4f( 0.5f, 0.5f, 0.5f, 0.25f ), Vec4f( 0.5f, 0.5f, 0.5f, 0.1f ) );
	p_damageEmitter->setParticleEndColor( Vec4f( 2.0f, 2.0f, 2.0f, 0.0f ), Vec4f( 0.1f, 0.1f, 0.1f, 0.0f ) );
	p_damageEmitter->setParticleStartSize( 0.75f, 0.1f );
	p_damageEmitter->setParticleEndSize( 3.0f, 0.2f );
	p_damageEmitter->setParticleGravity( 0.0f, 0.1f );
	p_damageEmitter->setParticleAnimFrame( 0.0f, 10.0f );
	attachChild( p_damageEmitter, false );

	SoundManager& soundMgr = env.engine.getSoundManager();

	// Sound fr den Antrieb laden, Soundinstanz erzeugen und an das Spielerobjekt hngen
	SoundHandle engineSound = soundMgr.get( "data\\engine3.ogg" );
	engineSound->setDefaultSoundGroupName( "SFX" );
	p_engineSoundInst = engineSound->createInstance( true );
	if( p_engineSoundInst )
	{
		p_engineSoundInst->attachTo( this, false );
		p_engineSoundInst->setVolume( 0.0f );
		p_engineSoundInst->play( true );
	}

	// Sound fr das Schutzschild laden
	shieldSound = soundMgr.get( "data\\shield.ogg" );
	shieldSound->setDefaultSoundGroupName( "SFX" );

	crosshairPosition = getPosition();
}

Obj_Player::~Obj_Player()
{
	if( p_engineSoundInst ) p_engineSoundInst->stop();
	if( p_shieldSoundInst ) p_shieldSoundInst->stop();

	level.getPS().destroyEmitter( p_smokeEmitter );
	level.getPS().destroyEmitter( p_fireEmitter );
	level.getPS().destroyEmitter( p_damageEmitter );
}

void Obj_Player::update()
{
	GameObject::update();
	updateControllable();
	updateArmed();

	if( getHealth() <= 0.0f )
	{
		explode( randInt( 15, 20 ), DT_PLAYER, 2.0f );
		kill();

		// letzte Punktzahl merken
		level.setFinalScore( score );
		return;
	}

	if( isFrozen() ) return;

	const Vec2f direction( -getDirection() );

	if( mouseAiming )
	{
		if( env.engine.getInput().isMouseButtonDown( MBTN_RIGHT ) )
		{
			const Vec2f targetVector( crosshairPosition - getPosition() );
			const float targetDist = targetVector.length();
			const Vec2f relTargetDir( absDirectionToRelDirection( targetVector / targetDist ) );
			setParameter( "Steering", relTargetDir.x * 2.0f * ( relTargetDir.y > 0.0f ? 1000.0f : 1.0f ) );
			const float f = targetDist < 7.5f ? targetDist / 7.5f : 1.0f;
			setParameter( "Thrust", -relTargetDir.y * 2.0f * f );
		}
	}

	// Treibstoff-Check
	if( fuel <= 0.0f ) setParameter( "Thrust", 0.0f );

	// Schub verarbeiten
	if( thrust > 0.0f )
	{
		const Vec2f force( direction * thrust * 32.5f );
		dBodyAddForce( getBody(), force.x, force.y, 0.0f );

		fuel -= env.dt * thrust;
		if( fuel < 0.0f ) fuel = 0.0f;
	}

	p_smokeEmitter->setRate( thrust * 50.0f );
	p_fireEmitter->setRate( thrust * 50.0f );

	float damage = 0.0f;
	if( getHealth() < 10.0f ) damage = 2.0f * ( 30.0f - getHealth() );
	p_damageEmitter->setRate( damage );

	// Schubsteuerung
	adjustLinear( thrust, getParameter( "Thrust" ), 5.0f, env.dt );

	float steering = getParameter( "Steering" );
	if( getHealth() < 10.0f )
	{
		// zufllig schlingern
		steering += getVelocity().length() * 0.025f * randFloat( -1.0f, 1.0f );
		steering = clamp( steering, -1.0f, 1.0f );
	}

	// Lenkung
	if( steering != 0.0f )
	{
		float torque = steering * 47.5f;
		dBodyAddTorque( getBody(), 0.0, 0.0, torque );
	}

	if( p_engineSoundInst )
	{
		// Antriebssound verarbeiten
		float soundFactor = thrust * ( thrust + 0.075f * getVelocity().length() );
		p_engineSoundInst->setVolumeTarget( ( soundFactor / 4.0f ), 5.0f );
		p_engineSoundInst->setPitchTarget( 0.35f + soundFactor, 5.0f );
	}

	if( getParameter( "Fire" ) > 0.0f ) fire( Vec2f( 0.0f, -1.0f ) );

	if( mouseAiming && canFireAtCrosshair )
	{
		if( env.engine.getInput().isMouseButtonDown( MBTN_LEFT ) )
		{
			// Feuer!
			fire( firingDirectionAtCrosshair, true );
		}
	}

	// Waffenauswahl
	if( getControlMode() == CM_USER )
	{
		Input& input = env.engine.getInput();
		if( input.isKeyDown( KEY_1 ) ) selectWeapon( WT_INK_BLASTER );
		if( input.isKeyDown( KEY_2 ) ) selectWeapon( WT_MISSILE );
		if( input.isKeyDown( KEY_3 ) ) selectWeapon( WT_CANNON );
	}

	if( !getWeapons()[getCurrentWeapon()].ammo )
	{
		// Munition leer? Zurck zur Standardwaffe!
		selectWeapon( WT_INK_BLASTER );
	}

	// Schutzschild verarbeiten
	if( shield > 0.0f ) setShield( shield - env.dt );

	if( mouseAiming )
	{
		// Position des Zielfadenkreuzes berechnen
		CEGUI::Point pixels = CEGUI::MouseCursor::getSingleton().getPosition();
		double projectionMatrix[16];
		int viewport[4];
		glGetDoublev( GL_PROJECTION_MATRIX, projectionMatrix );
		glGetIntegerv( GL_VIEWPORT, viewport );
		glPushMatrix();
		glLoadMatrixf( video.getCameraMatrix() );
		double cameraMatrix[16];
		glGetDoublev( GL_MODELVIEW_MATRIX, cameraMatrix );
		glPopMatrix();
		double wx, wy, wz;
		gluUnProject( pixels.d_x, static_cast<double>( video.getResolution().y ) - pixels.d_y, 1.0, cameraMatrix, projectionMatrix, viewport, &wx, &wy, &wz );
		crosshairPosition = Vec2d( wx, wy );

		// In welche Richtung msste geschossen werden?
		canFireAtCrosshair = false;
		if( computeHitWithGravity( weaponPlatform.getAbsPosition(), getVelocity(),
								   crosshairPosition, Vec2f( 0.0f, 0.0f ),
								   getWeapons()[getCurrentWeapon()].calcSpeed,
								   level.getGravity(),
								   static_cast<Vec2f*>( 0 ), static_cast<float*>( 0 ), &firingDirectionAtCrosshair ) )
		{
			// Liegt die Richtung im richtigen Winkelbereich?
			firingDirectionAtCrosshair.normalize();
			if( ( firingDirectionAtCrosshair ^ direction ) > 0.5f )
			{
				// Ja!
				canFireAtCrosshair = true;
			}
		}
	}
}

void Obj_Player::render( RenderPass pass )
{
	if( pass == RP_SHADOW || pass == RP_OBJECT_2 )
	{
		// Sprite rendern
		glPushMatrix();
		setupGLMatrix();
		const Vec2f relMovementDir( absDirectionToRelDirection( getVelocity() ).normalized() );
		const float speedSq = getVelocity().lengthSq();
		glScalef( 4.0f + 0.0005f * fabsf( relMovementDir.x ) * speedSq, 4.0f + 0.0005f * fabsf( relMovementDir.y ) * speedSq, 1.0f );
		video.drawSprite( Vec2i( 0, 0 ), Vec2i( 128, 128 ) );

		if( pass == RP_OBJECT_2 && shield > 0.0f )
		{
			float alpha = 0.6f + 0.4f * sinf( level.getTime() * 15.0f );
			glColor4f( 1.0f, 1.0f, 1.0f, min( 1.0f, shield ) * alpha );
			video.drawSprite( Vec2i( 0, 864 ), Vec2i( 128, 160 ), Vec2f( 1.0f, 1.25f ) );
		}

		glPopMatrix();
	}
	else if( pass == RP_FOREGROUND )
	{
		if( mouseAiming )
		{
			// Zielfadenkreuz anzeigen
			glPushMatrix();
			glTranslatef( crosshairPosition.x, crosshairPosition.y, 0.0f );
			if( canFireAtCrosshair ) glColor4f( 0.25f, 1.0f, 0.25f, 0.75f );
			else glColor4f( 1.0f, 0.25f, 0.25f, 0.5f );
			video.drawSprite( Vec2i( 448, 928 ), Vec2i( 64, 64 ), Vec2f( 3.0f, 3.0f ) );
			glPopMatrix();
		}

		const WeaponInfo& weapon = getWeapons()[getCurrentWeapon()];
		if( weapon.showTrajectory )
		{
			// Flugbahn anzeigen
			float t = 0.0f;
			float alpha = 0.9f;
			const float dt = 0.05f;
			Vec2f position = relPositionToAbsPosition( weapon.relPosition );
			Vec2f velocity = getVelocity() + weapon.speed * -getDirection();

			std::vector<Vec2f> positions;
			std::vector<Vec4f> colors;

			while( alpha > 0.0f )
			{
				positions.push_back( position );
				colors.push_back( Vec4f( 1.0f, 1.0f, 1.0f, t == 0.0f ? 0.0f : alpha ) );

				velocity *= powf( weapon.trajectoryFriction, dt );
				velocity += dt * level.getGravity();
				position += dt * velocity;
				t += dt;
				alpha -= 1.25f * dt;
			}

			CartoonRenderer& cr = level.getCR();
			cr.setStyle( "hud" );
			cr.drawPolyLine( positions, false, 0.5f, Vec4f( 1.0f ), &colors );
		}
	}
}

void Obj_Player::onCollision( GameObject* p_other,
							  CollisionInfo& info )
{
	GameObject::onCollision( p_other, info );
	if( info.passThrough ) return;

	// Power-Ups machen keinen Schaden
	if( p_other && p_other->getType() == "PowerUp" ) return;

	if( p_other && info.frontal && info.p_params )
	{
		float massScale = min( 1.0f, p_other->getTotalMass() / getTotalMass() );
		addHealth( -0.01f * info.speed * info.speed * massScale * p_other->getPlayerCollisionDamageScale() );
	}
}

void Obj_Player::addAvoidanceAreas()
{
	level.addAvoidanceCircle( 100, this, getTransform(), Vec2f( 0.0f, 0.0f ), 5.0f );
}

void Obj_Player::onKill()
{
	p_smokeEmitter->setRate( 0.0f );
	p_fireEmitter->setRate( 0.0f );
	p_damageEmitter->setRate( 0.0f );
	if( p_engineSoundInst ) p_engineSoundInst->setMuted( true );
	if( p_shieldSoundInst ) p_shieldSoundInst->setMuted( true );
}

void Obj_Player::onRebirth()
{
	if( p_engineSoundInst ) p_engineSoundInst->setMuted( false );
	if( p_shieldSoundInst ) p_shieldSoundInst->setMuted( false );
}

void Obj_Player::addFuel( float value )
{
	fuel += value;
	fuel = clamp( fuel, 0.0f, 100.0f );
}

void Obj_Player::setFuel( float value )
{
	fuel = clamp( value, 0.0f, 100.0f );
}

float Obj_Player::getFuel() const
{
	return fuel;
}

void Obj_Player::addScore( int value )
{
	int newScore = score + value;
	score = newScore >= 0 ? newScore : 0;
}

void Obj_Player::setScore( int value )
{
	score = max( 0, value );
}

unsigned int Obj_Player::getScore() const
{
	return score;
}

void Obj_Player::addBonusScore()
{
	level.addPlayerScore( static_cast<int>( fuel * 20.0f + getHealth() * 40.0f ), getPosition() );
	fuel = 0.0f;
	setHealth( 0.0f );
}

void Obj_Player::addShield( float value )
{
	setShield( shield + value );
}

void Obj_Player::setShield( float value )
{
	float oldShield = shield;
	shield = max( 0.0f, value );

	setInvincible( shield > 0.0f );

	if( shield > 0.0f && oldShield == 0.0f )
	{
		// Sound starten
		p_shieldSoundInst = shieldSound->createInstance( true );
		if( p_shieldSoundInst )
		{
			p_shieldSoundInst->attachTo( this );
			p_shieldSoundInst->setVolume( 0.0f );
			p_shieldSoundInst->setPitch( 0.1f );
			p_shieldSoundInst->setVolumeTarget( 1.0f, 5.0f );
			p_shieldSoundInst->setPitchTarget( 1.0f, 5.0f );
			p_shieldSoundInst->play( true );
		}
	}
	else if( shield == 0.0f && oldShield > 0.0f )
	{
		// Sound stoppen
		if( p_shieldSoundInst )
		{
			p_shieldSoundInst->setVolumeTarget( 0.0f, 5.0f );
			p_shieldSoundInst->setPitchTarget( 0.0f, 5.0f );
			p_shieldSoundInst->setStopCountDown( -1.0f );
			p_shieldSoundInst = 0;
		}
	}
}

float Obj_Player::getShield() const
{
	return shield;
}

void Obj_Player::setMouseAiming( bool enable )
{
	mouseAiming = enable;
}

bool Obj_Player::getMouseAiming() const
{
	return mouseAiming;
}