#include <PCH.h>
#include <Obj_PlayerGhost.h>
#include <HighscoresDB.h>

Obj_PlayerGhost::Obj_PlayerGhost( Level& level,
								  const std::string& filename,
							  	  bool record) : GameObject( level, 0, false ),
												 filename( filename ),
												 record( record ),
												 time( 0 ),
												 timeNeeded( ~0 )
{
	if( !record )
	{
		FILE* p_file = fopen( filename.c_str(), "rb" );
		if( !p_file ) return;

		// Level-Hash-Code lesen und berprfen
		unsigned long long levelHash;
		fread( &levelHash, sizeof( levelHash ), 1, p_file );
		if( levelHash != HighscoresDB::inst().getHashCode( "data\\levels\\" + level.getFilename() ) )
		{
			// Dieser Ghost wurde mit einem vernderten Level aufgenommen!
			fclose( p_file );
			return;
		}

		// Zeit lesen
		fread( &timeNeeded, sizeof( timeNeeded ), 1, p_file );

		// Frames lesen
		size_t n = 0;
		fread( &n, sizeof( n ), 1, p_file );
		for( size_t i = 0; i < n; i++ )
		{
			Frame frame;
			if( fread( &frame, sizeof( frame ), 1, p_file ) ) frames.push_back( frame );
		}

		fclose( p_file );
	}

	setBoundingCircle( Vec2f( 0.0f, 0.0f ), 2.5f );
	if( record ) setVisible( false );
}

Obj_PlayerGhost::~Obj_PlayerGhost()
{
}

void Obj_PlayerGhost::update()
{
	GameObject::update();

	if( isFrozen() ) return;

	if( record )
	{
		GameObject* p_player = level.getObjectByID( "player" );
		if( p_player )
		{
			// aktuelle Position und aktuellen Winkel des Spielers speichern
			Frame frame;
			frame.position = p_player->getPosition();
			frame.angle = p_player->getAngle();
			frames.push_back( frame );
		}
	}
	else
	{
		if( time < frames.size() )
		{
			// Position und Winkel lesen
			const Frame& frame = frames[time];
			setPosition( frame.position );
			setAngle( frame.angle );
			time++;
		}
	}
}

void Obj_PlayerGhost::render( RenderPass pass )
{
	if( record ) return;
	if( time >= frames.size() ) return;

	if( pass == RP_BACKGROUND_2 )
	{
		// Sprite rendern
		glPushMatrix();
		setupGLMatrix();
		glScalef( 4.0f, 4.0f, 1.0f );
		Vec4f color( level.getPassColor() );
		color.a *= 0.25f;
		glColor4fv( color );
		video.drawSprite( Vec2i( 0, 0 ), Vec2i( 128, 128 ) );
		glPopMatrix();
	}
}

void Obj_PlayerGhost::save()
{
	if( !record ) return;

	timeNeeded = ~0 - level.getFinalScore();

	FILE* p_file = fopen( filename.c_str(), "wb" );

	// Hash-Code des Levels speichern
	unsigned long long levelHash = HighscoresDB::inst().getHashCode( "data\\levels\\" + level.getFilename() );
	fwrite( &levelHash, sizeof( levelHash ), 1, p_file );

	// Zeit speichern
	fwrite( &timeNeeded, sizeof( timeNeeded ), 1, p_file );

	// Frames speichern
	size_t n = frames.size();
	fwrite( &n, sizeof( n ), 1, p_file );
	for( size_t i = 0; i < n; i++ ) fwrite( &frames[i], sizeof( frames[i] ), 1, p_file );

	fclose( p_file );
}

bool Obj_PlayerGhost::isEmpty() const
{
	return frames.empty();
}

unsigned int Obj_PlayerGhost::getTimeNeeded() const
{
	return timeNeeded;
}