#include <PCH.h>
#include <Obj_PowerUp.h>
#include <Obj_Player.h>

/*
 * Typenbersicht:
 * 1:  Streichholzraketen
 * 2:  Erbsenkanone
 * 10: Schutzschild
 *
 */

Obj_PowerUp::Obj_PowerUp( Level& level,
						  const TiXmlElement* p_root ) : GameObject( level, p_root, true ),
														 type( 0 ),
														 points( 0 ),
														 shieldDuration( 0.0f )
{
	setMaterial( "PowerUp" );

	// Dynamisch oder statisch?
	int dynamic_ = 0;
	p_root->QueryIntAttribute( "dynamic", &dynamic_ );
	dynamic = dynamic_ ? true : false;

	if( !dynamic ) deleteBody();

	// Das Power-Up kann mit Objekten kollidieren, aber nicht mit Wnden.
	setCollisionBits( CB_STATIC | CB_OBJECT, ~CB_STATIC );
	addCircle( Vec2f( 0.0f, 0.0f ), 1.5f );
	setBoundingCircle( Vec2f( 0.0f, 0.0f ), 2.0f );

	if( dynamic ) setTotalMass( 0.1f );

	// Typ lesen
	p_root->QueryIntAttribute( "subType", &type );

	SoundManager& soundMgr = env.engine.getSoundManager();

	switch( type )
	{
	case 1:
		points = 500;
		texCoords = Vec2i( 0, 736 );
		sound = soundMgr.get( "data\\powerup1.ogg" );
		break;
	case 2:
		points = 500;
		texCoords = Vec2i( 128, 736 );
		sound = soundMgr.get( "data\\powerup1.ogg" );
		break;
	case 10:
		points = 500;
		texCoords = Vec2i( 64, 736 );
		sound = soundMgr.get( "data\\powerup1.ogg" );
		shieldDuration = 10.0f;
		p_root->QueryFloatAttribute( "shieldDuration", &shieldDuration );
		break;
	case 11:
		points = 500;
		texCoords = Vec2i( 192, 736 );
		sound = soundMgr.get( "data\\powerup1.ogg" );
		break;
	}

	t = randFloat( 0.0f, 6.2832f );
	if( dynamic ) setAngle( t );
}

Obj_PowerUp::~Obj_PowerUp()
{
}

void Obj_PowerUp::update()
{
	GameObject::update();

	if( !dynamic ) setAngle( t + level.getTime() );
}

void Obj_PowerUp::render( RenderPass pass )
{
	if( pass == RP_SHADOW || pass == RP_OBJECT_1 )
	{
		// Sprite rendern
		glPushMatrix();
		setupGLMatrix();

		if( pass == RP_OBJECT_1 )
		{
			const float size = 6.5f + 1.5f * sinf( t + level.getTime() * 10.0f );
			const float alpha = 0.1f + 0.05f * cosf( t + level.getTime() * 10.0f );
			glPushMatrix();
			glRotatef( t - level.getTime() * 90.0f, 0.0f, 0.0f, 1.0f );
			glColor4f( 1.0f, 0.875f, 0.75f, alpha );
			video.setupAlphaBlending( true, GL_SRC_ALPHA, GL_ONE );
			video.drawSprite( Vec2i( 0, 800 ), Vec2i( 64, 64 ), Vec2f( size ) );
			video.setupAlphaBlending( true, GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
			glColor4f( 1.0f, 1.0f, 1.0f, 1.0f );
			glPopMatrix();
		}

		video.drawSprite( texCoords, Vec2i( 64, 64 ), Vec2f( 3.0f, 3.0f ) );

		glPopMatrix();
	}
}

void Obj_PowerUp::onCollision( GameObject* p_other,
							   CollisionInfo& info )
{
	GameObject::onCollision( p_other, info );

	if( p_other && p_other->getType() == "Player" )
	{
		Obj_Player* p_player = dynamic_cast<Obj_Player*>( p_other );

		switch( type )
		{
		case 1:
			// 10 Raketen eingesammelt
			p_player->addAmmo( WT_MISSILE, 10 );
			break;
		case 2:
			// 30 Schuss fr die Erbsenkanone eingesammelt
			p_player->addAmmo( WT_CANNON, 30 );
			break;
		case 10:
			// Schutzschild aktivieren
			p_player->addShield( shieldDuration );
			break;
		case 11:
			// Mauszielhilfe
			p_player->setMouseAiming( true );
			break;
		}

		// Punkte hinzufgen
		level.addPlayerScore( points, getPosition() );

		if( sound )
		{
			// Sound abspielen
			SoundInstance* p_soundInst = sound->createInstance();
			if( p_soundInst )
			{
				attachChild( p_soundInst, false );
				p_soundInst->setVolume( 0.5f );
				p_soundInst->setPitch( randFloat( 0.9f, 1.1f ) );
				p_soundInst->play();
			}
		}

		// Funken
		SimpleEmitter* p_emitter = level.getPS().createSimpleEmitter();
		attachChild( p_emitter, false );
		p_emitter->setRate( 2500.0f );
		p_emitter->setTimeToLive( 0.1f );
		p_emitter->addParticleStyle( 4 );
		p_emitter->addParticleStyle( 8 );
		p_emitter->setParticleTimeToLive( 0.4f, 0.2f );
		p_emitter->setParticlePositionRadius( 2.0f );
		p_emitter->setParticleSpeed( 8.0f, 2.0f );
		p_emitter->setParticleStartColor( Vec4f( 1.0f, 0.75f, 0.0f, 0.75f ), Vec4f( 0.4f, 0.4f, 0.4f, 0.25f ) );
		p_emitter->setParticleEndColor( Vec4f( 1.0f, 0.0f, 0.0f, 0.0f ), Vec4f( 0.4f, 0.0f, 0.0f, 0.0f ) );
		p_emitter->setParticleStartSize( 2.0f, 0.5f );
		p_emitter->setParticleEndSize( 5.0f, 1.0f );
		p_emitter->setParticleGravity( 4.0f, 1.0f );
		p_emitter->setParticleFriction( 0.25f, 0.1f );

		kill();
	}
}