#include <PCH.h>
#include <Obj_Projectile.h>
#include <Obj_BulletHole.h>
#include <Obj_Enemy.h>
#include <Damagable.h>

/*
 * Typenbersicht:
 * 1: blauer Tintentropfen
 * 2: grner Tintentropfen
 * 3: Flugabwehrgeschoss
 * 4: zielsuchende Rakete
 * 5: Kanonengeschoss
 *
 */

Obj_Projectile::Obj_Projectile( Level& level,
								GameObject* p_owner,
							    int type ) : GameObject( level, 0, true ),
											 p_owner( p_owner ),
											 p_fireEmitter( 0 ),
											 p_smokeEmitter( 0 ),
											 type( type ),
											 age( 0.0f ),
											 damage( 0.0f ),
											 damageVar( 0.0f ),
											 timeToLive( 30.0f ),
											 oldVelocity( 0.0f, 0.0f ),
											 bounce( false )
{
	setMaterial( "Projectile" );

	// Das Projektil kann mit allen Objekten kollidieren, aber am Anfang noch nicht mit dem Schtzen.
	setCollisionBits( CB_PROJECTILE, CB_ALL );
	setCollisionEnabled( p_owner, false );

	setBoundingCircle( Vec2f( 0.0f, 0.0f ), 2.0f );

	SoundManager& soundMgr = env.engine.getSoundManager();

	if( type == 1 || type == 2 || type == 3 )
	{
		damage = 2.0f;
		damageVar = 0.5f;
		bounce = true;

		addCircle( Vec2f( 0.0f, 0.0f ), type == 3 ? 0.4f : 0.25f );
		setTotalMass( type == 3 ? 0.2f : 0.025f );
		setFriction( 0.99f, 0.0f );

		sizeOnTexture = Vec2i( 32, 32 );
		texCoords = Vec2i( 128, 0 ) + Vec2i( randInt( 0, 1 ) * 32, randInt( 0, 1 ) * 32 );
		if( type == 2 || type == 3 ) texCoords.y += 64;

		// Sound laden
		impactSound = soundMgr.get( "data\\impact1.ogg" );
		impactSound->setDefaultSoundGroupName( "SFX" );
		level.cacheSound( impactSound );
	}
	else if( type == 4 )
	{
		damage = 7.5f;
		damageVar = 1.0f;

		addSegment( Vec2f( 0.0f, -0.6f ), Vec2f( 0.0f, 0.6f ), 0.1f );
		setTotalMass( 0.2f );
		setFriction( 0.5f, 0.001f );

		sizeOnTexture = Vec2i( 32, 64 );
		texCoords = Vec2i( 192, 0 );

		// Emitter fr Rauch erzeugen
		p_smokeEmitter = level.getPS().createSimpleEmitter();
		p_smokeEmitter->setPosition( Vec2f( 0.0f, 0.3f ) );
		p_smokeEmitter->addParticleStyle( 1 );
		p_smokeEmitter->setParticleTimeToLive( 0.4f, 0.1f );
		p_smokeEmitter->setParticlePositionRadius( 0.05f );
		p_smokeEmitter->setParticleDirection( Vec2f( 0.0f, -1.0f ), 0.15f );
		p_smokeEmitter->setParticleSpeed( 3.0f, 0.1f );
		p_smokeEmitter->setParticleStartColor( Vec4f( 1.0f, 0.5f, 0.0f, 1.0f ), Vec4f( 0.1f, 0.1f, 0.1f, 0.5f ) );
		p_smokeEmitter->setParticleEndColor( Vec4f( 2.0f, 2.0, 4.0f, 0.0f ), Vec4f( 0.1f, 0.1f, 0.1f, 0.0f ) );
		p_smokeEmitter->setParticleStartSize( 0.25f, 0.1f );
		p_smokeEmitter->setParticleEndSize( 2.0f, 0.2f );
		p_smokeEmitter->setParticleGravity( 0.0f, 0.1f );
		p_smokeEmitter->setParticleAnimFrame( 0.0f, 10.0f );
		attachChild( p_smokeEmitter, false );

		// Emitter fr Stichflamme erzeugen
		p_fireEmitter = level.getPS().createSimpleEmitter();
		p_fireEmitter->setPosition( Vec2f( 0.0f, 0.4f ) );
		p_fireEmitter->addParticleStyle( 2 );
		p_fireEmitter->setParticleTimeToLive( 0.05f, 0.01f );
		p_fireEmitter->setParticlePositionRadius( 0.05f );
		p_fireEmitter->setParticleDirection( Vec2f( 0.0f, -1.0f ), 0.5f );
		p_fireEmitter->setParticleSpeed( 5.0f, 0.1f );
		p_fireEmitter->setParticleStartColor( Vec4f( 1.0f, 1.0f, 1.0f, 1.0f ), Vec4f( 0.1f, 0.1f, 0.1f, 0.5f ) );
		p_fireEmitter->setParticleEndColor( Vec4f( 1.0f, 1.0f, 1.0f, 0.0f ), Vec4f( 0.1f, 0.1f, 0.1f, 0.0f ) );
		p_fireEmitter->setParticleStartSize( 2.0f, 0.5f );
		p_fireEmitter->setParticleEndSize( 4.0f, 0.5f );
		p_fireEmitter->setParticleGravity( 0.0f, 0.1f );
		p_fireEmitter->setParticleAnimFrame( 0.0f, 10.0f );
		attachChild( p_fireEmitter, false );
	}
	else if( type == 5 )
	{
		damage = 9.0f;
		damageVar = 1.0f;
		bounce = true;

		addCircle( Vec2f( 0.0f, 0.0f ), 0.3f );
		setTotalMass( 0.25f );
		setFriction( 0.7f, 0.0f );

		sizeOnTexture = Vec2i( 32, 32 );
		texCoords = Vec2i( 192, 64 + randInt( 0, 1 ) * 32 );

		// Sound laden
		impactSound = soundMgr.get( "data\\impact2.ogg" );
		impactSound->setDefaultSoundGroupName( "SFX" );
		level.cacheSound( impactSound );
	}
}

Obj_Projectile::~Obj_Projectile()
{
	if( p_fireEmitter ) level.getPS().destroyEmitter( p_fireEmitter );
	if( p_smokeEmitter ) level.getPS().destroyEmitter( p_smokeEmitter );
}

void Obj_Projectile::update()
{
	GameObject::update();

	age += env.dt;
	if( age > 1.0f )
	{
		// Das Projektil kann jetzt auch den Schtzen treffen.
		excludeCollisions.erase( p_owner );
	}

	timeToLive -= env.dt;
	if( timeToLive <= 0.0f )
	{
		if( type == 3 ) burst( randInt( 8, 12 ), 2 );
		else if( type == 4 ) explode( 0, DT_NONE, 0.25f, 0.5f, false, 3, randFloat( 0.8f, 0.9f ), randFloat( 0.9f, 1.1f ) );
		kill();
		return;
	}

	if( type == 1 || type == 2 || type == 3 || type == 5 )
	{
		setDirection( -getVelocity() );
	}
	else if( type == 4 )
	{
		GameObject* p_target = getTarget();

		float thrust = 1.0f;

		if( age > 0.4f )
		{
			if( age < 3.4f )
			{
				float steering = 0.4f * sinf( 20.0f * age );
				bool targetLost = true;

				if( p_target )
				{
					if( canSee( p_target, 25.0f ) )
					{
						// zum Ziel fliegen
						Vec2f hitPoint, shootVector;
						float hitTime;
						if( computeHitNoGravity( getPosition(), Vec2f( 0.0f, 0.0f ),
												 p_target->getPosition(), p_target->getVelocity(),
												 min( 50.0f, getVelocity().length() + 7.5f ),
												 &hitPoint, &hitTime, &shootVector ) )
						{
							shootVector -= 0.85f * getVelocity();
							shootVector.normalize();
							const Vec2f relDir( absDirectionToRelDirection( shootVector ) );
							if( relDir.y < 0.5f )
							{
								steering = relDir.x;
								thrust = max( 0.35f, -shootVector ^ getDirection() );
								targetLost = false;
							}
						}
					}
				}

				if( targetLost && getTargetID() != "player" )
				{
					setTargetID( "" );
					p_target = 0;
				}

				// Lenkung
				float torque = steering * 1.5f;
				dBodyAddTorque( getBody(), 0.0, 0.0, torque );

				// Schub geben
				const Vec2f direction( -getDirection() );
				const Vec2f force( thrust * direction * 10.0f );
				dBodyAddForce( getBody(), force.x, force.y, 0.0f );
			}
			else
			{
				// Die Rakete ist ausgebrannt!
				thrust = 0.0f;
			}
		}
		else
		{
			thrust = 0.0f;
		}

		p_smokeEmitter->setRate( thrust * 120.0f );
		p_fireEmitter->setRate( thrust * 50.0f );

		if( !p_target )
		{
			float bestScore = 0.0f;

			// neues Ziel suchen
			const std::map<std::string, GameObject*>& objects = level.getObjects();
			for( std::map<std::string, GameObject*>::const_iterator it = objects.begin();
				 it != objects.end();
				 it++ )
			{
				if( dynamic_cast<Obj_Enemy*>( it->second ) )
				{
					if( canSee( it->second, 25.0f ) )
					{
						const Vec2f targetPos( it->second->getPosition() );
						const Vec2f dirToTarget( ( targetPos - getPosition() ).normalized() );
						const float dot = getDirection() ^ dirToTarget;

						if( dot < -0.25f )
						{
							// potenzielles Ziel gefunden
							float score = -dot;
							if( score > bestScore )
							{
								setTargetID( it->first );
								bestScore = score;
							}
						}
					}
				}
			}
		}
	}
}

void Obj_Projectile::render( RenderPass pass )
{
	if( pass == RP_SHADOW || pass == RP_OBJECT_1 )
	{
		// Sprite rendern
		if( type == 1 || type == 2 || type == 3 || type == 5 )
		{
			glPushMatrix();
			setupGLMatrix();
			const float s = 0.5f + 0.05f * getVelocity().length();
			if( type == 1 || type == 2 || type == 5 ) glScalef( 0.5f, s, 1.0f );
			else if( type == 3 || type == 5 ) glScalef( 1.0f, 2.0f * s, 1.0f );

			if( timeToLive < 1.0f && ( type == 1 || type == 2 ) )
			{
				Vec4f color( level.getPassColor() );
				color.a *= timeToLive;
				glColor4fv( color );
			}

			video.drawSprite( texCoords, sizeOnTexture );
			glPopMatrix();
		}
		else if( type == 4 )
		{
			glPushMatrix();
			setupGLMatrix();
			video.drawSprite( texCoords, sizeOnTexture, Vec2f( 1.0f, 2.0f ) );
			glPopMatrix();
		}
	}
}

void Obj_Projectile::onCollision( GameObject* p_other,
								  CollisionInfo& info )
{
	GameObject::onCollision( p_other, info );
	if( info.passThrough ) return;

	if( !info.frontal && bounce )
	{
		// Das Projektil prallt ab.
		return;
	}

	// Kann das andere Objekt Schaden nehmen?
	Damagable* p_damagable = dynamic_cast<Damagable*>( p_other );
	if( p_damagable )
	{
		// Gesundheitspunkte abziehen
		p_damagable->addHealth( -damage + randFloat( -damageVar, damageVar ) );
	}

	timeToLive = 0.0f;

	if( type == 1 || type == 2 )
	{
		// einen Emitter erstellen, der Tintenspritzer erzeugt
		SimpleEmitter* p_emitter = level.getPS().createSimpleEmitter();
		p_emitter->setTimeToLive( 0.1f );
		p_emitter->setRate( 50.0f );
		p_emitter->setVisibilityBehavior( false, 2.0f );
		p_emitter->setPosition( info.collisionPoint );
		p_emitter->addParticleStyle( 6 );
		p_emitter->setParticleTimeToLive( 0.3f, 0.2f );
		p_emitter->setParticlePositionRadius( 0.25f );
		p_emitter->setParticleSpeed( 10.0f, 4.0f );

		if( type == 1 )
		{
			p_emitter->setParticleStartColor( Vec4f( 0.1f, 0.1f, 0.75f, 0.5f ), Vec4f( 0.1f, 0.1f, 0.2f, 0.25f ) );
			p_emitter->setParticleEndColor( Vec4f( 0.1f, 0.1f, 0.75f, 0.5f ), Vec4f( 0.1f, 0.1f, 0.2f, 0.25f ) );
		}
		else if( type == 2 )
		{
			p_emitter->setParticleStartColor( Vec4f( 0.1f, 0.75f, 0.1f, 0.5f ), Vec4f( 0.1f, 0.2f, 0.1f, 0.25f ) );
			p_emitter->setParticleEndColor( Vec4f( 0.1f, 0.75f, 0.1f, 0.5f ), Vec4f( 0.1f, 0.2f, 0.1f, 0.25f ) );
		}

		p_emitter->setParticleStartSize( 2.0f, 0.5f );
		p_emitter->setParticleEndSize( 0.0f, 0.0f );
		p_emitter->setParticleGravity( 8.0f, 1.0f );
		p_emitter->setParticleFriction( 0.25f, 0.1f );

		// "Einschussloch" erstellen
		Obj_BulletHole* p_bulletHole = new Obj_BulletHole( level, type );
		p_bulletHole->setPosition( getPosition() );

		// Sound abspielen
		SoundInstance* p_soundInst = impactSound->createInstance();
		if( p_soundInst )
		{
			p_soundInst->setPosition( info.collisionPoint );
			p_soundInst->setVolume( randFloat( 0.4f, 0.5f ) );
			p_soundInst->setPitch( type == 1 ? randFloat( 0.75f, 1.25f ) : randFloat( 0.5f, 1.0f ) );
			p_soundInst->play();
		}
	}
	else if( type == 3 )
	{
		if( oldVelocity.lengthSq() == 0.0f ) oldVelocity = getVelocity();
	}
	else if( type == 5 )
	{
		// einen Emitter erstellen, der "Erbsenspritzer" erzeugt
		SimpleEmitter* p_emitter = level.getPS().createSimpleEmitter();
		p_emitter->setTimeToLive( 0.1f );
		p_emitter->setRate( 150.0f );
		p_emitter->setVisibilityBehavior( false, 2.0f );
		p_emitter->setPosition( info.collisionPoint );
		p_emitter->addParticleStyle( 9 );
		p_emitter->setParticleTimeToLive( 0.3f, 0.2f );
		p_emitter->setParticlePositionRadius( 0.25f );
		p_emitter->setParticleSpeed( 10.0f, 4.0f );
		p_emitter->setParticleStartColor( Vec4f( 0.2f, 0.5f, 0.0f, 0.5f ), Vec4f( 0.1f, 0.1f, 0.1f, 0.25f ) );
		p_emitter->setParticleEndColor( Vec4f( 0.2f, 0.5f, 0.0f, 0.5f ), Vec4f( 0.1f, 0.1f, 0.1f, 0.25f ) );
		p_emitter->setParticleStartSize( 2.0f, 0.5f );
		p_emitter->setParticleEndSize( 0.0f, 0.0f );
		p_emitter->setParticleGravity( 8.0f, 1.0f );
		p_emitter->setParticleFriction( 0.25f, 0.1f );

		// Sound abspielen
		SoundInstance* p_soundInst = impactSound->createInstance();
		if( p_soundInst )
		{
			p_soundInst->setPosition( info.collisionPoint );
			p_soundInst->setVolume( randFloat( 0.8f, 0.9f ) );
			p_soundInst->setPitch( randFloat( 0.75f, 1.25f ) );
			p_soundInst->play();
		}
	}
}

void Obj_Projectile::setTimeToLive( float timeToLive )
{
	this->timeToLive = timeToLive;
}

float Obj_Projectile::getTimeToLive() const
{
	return timeToLive;
}

int Obj_Projectile::getProjectileType() const
{
	return type;
}

void Obj_Projectile::burst( unsigned int numProjectiles,
						    int projectileType )
{
	explode( 0, DT_NONE, 1.0f, 0.5f, false, 3, 0.8f, randFloat( 0.75f, 1.25f ) );

	std::vector<Obj_Projectile*> projectiles;

	// Splitterprojektile erzeugen
	for( unsigned int i = 0; i < numProjectiles; i++ )
	{
		Obj_Projectile* p_projectile = new Obj_Projectile( level, p_owner, projectileType );
		p_projectile->setVelocity( randVec2fDir() * randFloat( 30.0f, 40.0f ) );
		if( oldVelocity.lengthSq() != 0.0f ) p_projectile->setPosition( getPosition() - 1.5f * oldVelocity.normalized() );
		else p_projectile->setPosition( getPosition() );
		p_projectile->setTimeToLive( randFloat( 0.5f, 0.75f ) );
		projectiles.push_back( p_projectile );
	}

	// Die Splitterprojektile sollen untereinander nicht kollidieren.
	for( size_t i = 0; i < projectiles.size(); i++ )
	{
		for( size_t j = i + 1; j < projectiles.size(); j++ )
		{
			projectiles[i]->excludeCollisions.insert( projectiles[j] );
		}
	}

	// Sound abspielen
	SoundInstance* p_soundInst = impactSound->createInstance();
	if( p_soundInst )
	{
		p_soundInst->setPosition( getPosition() );
		p_soundInst->setVolume( randFloat( 0.4f, 0.5f ) );
		p_soundInst->setPitch( randFloat( 0.5f, 1.0f ) );
		p_soundInst->play();
	}
}