#include <PCH.h>
#include <Obj_TestTarget.h>

Obj_TestTarget::Obj_TestTarget( Level& level,
							    const std::string& id ) : GameObject( level, 0, true, id ),
														  Controllable( level.getEnv() ),
														  doReset( false ),
														  hitCount( 0 )
{
	setMaterial( "Rubber" );

	// Das Testziel kann mit allen Objekten kollidieren.
	setCollisionBits( CB_OBJECT, CB_ALL );

	addCircle( Vec2f( 0.0f, 0.0f ), 2.0f );

	setBoundingCircle( Vec2f( 0.0f, 0.0f ), 2.5f );

	setTotalMass( 0.01f );
	setFriction( 0.5f, 0.5f );
	setGravityInfluence( false );

	reset();
}

Obj_TestTarget::~Obj_TestTarget()
{
}

void Obj_TestTarget::update()
{
	GameObject::update();

	if( isFrozen() ) return;

	dBodyAddForce( getBody(), randFloat( -100.0f, 100.0f ), randFloat( -100.0f, 100.0f ), 0.0 );

	if( doReset )
	{
		reset();
		doReset = false;
	}
}

void Obj_TestTarget::render( RenderPass pass )
{
	if( pass == RP_SHADOW || pass == RP_OBJECT_2 )
	{
		// Sprite rendern
		glPushMatrix();
		setupGLMatrix();
		glScalef( 4.0f, 4.0f, 1.0f );
		video.drawSprite( Vec2i( 384, 384 ), Vec2i( 128, 128 ) );
		glPopMatrix();
	}
}

void Obj_TestTarget::onCollision( GameObject* p_other,
								  CollisionInfo& info )
{
	GameObject::onCollision( p_other, info );
	if( info.passThrough ) return;

	if( !p_other ) return;

	if( p_other->getType() != "StaticWall" )
	{
		// Objekt neu positionieren
		doReset = true;
	}

	if( p_other->getType() == "Projectile" )
	{
		hitCount++;
	}
}

void Obj_TestTarget::reset()
{
	setPosition( Vec2f( randFloat( 5.0f, 35.0f ), randFloat( 5.0f, 25.0f ) ) );
	setVelocity( Vec2f( randFloat( -10.0f, 10.0f ), randFloat( -10.0f, 10.0f ) ) );
}

int Obj_TestTarget::getHitCount() const
{
	return hitCount;
}