#include <PCH.h>
#include <ScriptObj_Counter.h>

ScriptObj_Counter::ScriptObj_Counter( Level& level,
									  int displayMode,
									  int direction,
									  float duration,
									  luabind::object onElapsed ) : ScriptObject( level ),
																	displayMode( displayMode ),
																	direction( direction ),
																	duration( duration ),
																	onElapsed( onElapsed ),
																	time( direction > 0 ? 0.0f : duration ),
																	paused( false ),
																	color( 1.0f ),
																	numBeeps( 0 ),
																	oldNumSecondsLeft( -1 )
{
}

ScriptObj_Counter::~ScriptObj_Counter()
{
}

void ScriptObj_Counter::update()
{
	int numSecondsLeft = oldNumSecondsLeft;

	if( !paused )
	{
		if( direction > 0 && time < duration )
		{
			time += env.dt;
			if( time >= duration )
			{
				Scripting::inst().call( onElapsed, this );
				time = duration;
			}

			numSecondsLeft = static_cast<int>( duration - time );
		}
		else if( direction < 0 && time > 0.0f )
		{
			time -= env.dt;
			if( time <= 0.0f )
			{
				Scripting::inst().call( onElapsed, this );
				time = 0.0f;
			}

			numSecondsLeft = static_cast<int>( time );
		}
	}

	if( oldNumSecondsLeft != -1 && oldNumSecondsLeft != numSecondsLeft && numSecondsLeft < numBeeps )
	{
		// Piep!
		env.engine.getAudio().playSound( "data\\beep.ogg", "SFX", "Sample", "2D", 0.8f, 1.0f );
	}

	oldNumSecondsLeft = numSecondsLeft;
}

void ScriptObj_Counter::render()
{
	if( !displayMode ) return;

	const unsigned int msecs = static_cast<unsigned int>( time * 1000.0f );
	const unsigned int secs = msecs / 1000;
	const unsigned int mins = secs / 60;

	char text[16] = "";

	if( displayMode == 1 )
	{
		sprintf( text, "%02d:%02d", mins, secs % 60 );
	}
	else if( displayMode == 2 )
	{
		sprintf( text, "%02d:%02d.%02d", mins, secs % 60, ( msecs / 10 ) % 100 );
	}
	else return;

	glPushMatrix();
	glLoadIdentity();
	glTranslatef( -12.5f, 16.0f, 0.0f );

	env.scoreFont->print( text, 1.5f, color );

	glPopMatrix();
}