#include <PCH.h>
#include <Scripting.h>
#include <Obj_Player.h>
#include <Obj_Enemy.h>
#include <Obj_Laser.h>
#include <Obj_EnergyNode.h>
#include <Obj_LevelTarget.h>
#include <Obj_PowerUp.h>
#include <Obj_Checkpoint.h>
#include <Obj_Projectile.h>
#include <Obj_Garage.h>
#include <Obj_Mine.h>
#include <Obj_Accelerator.h>
#include <ScriptObj_Counter.h>
#include <Console.h>
#include <luabind/operator.hpp>
#include <luabind/adopt_policy.hpp>
#include <luabind/iterator_policy.hpp>
using namespace luabind;

template<typename T> void bindVecDetailToLua( class_<Vec<2, T> >& classDef )
{
	classDef
		.def( constructor<T, T>() )
		.def_readwrite( "x", &Vec<2, T>::x )
		.def_readwrite( "y", &Vec<2, T>::y )
		.property( "perp", &Vec<2, T>::perp );
}

template<typename T> void bindVecDetailToLua( class_<Vec<3, T> >& classDef )
{
	classDef
		.def( constructor<T, T, T>() )
		.def_readwrite( "x", &Vec<3, T>::x )
		.def_readwrite( "y", &Vec<3, T>::y )
		.def_readwrite( "z", &Vec<3, T>::z )
		.def_readwrite( "r", &Vec<3, T>::x )
		.def_readwrite( "g", &Vec<3, T>::y )
		.def_readwrite( "b", &Vec<3, T>::z )
		.def( "cross", &Vec<3, T>::operator % );
}

template<typename T> void bindVecDetailToLua( class_<Vec<4, T> >& classDef )
{
	classDef
		.def( constructor<T, T, T, T>() )
		.def_readwrite( "x", &Vec<4, T>::x )
		.def_readwrite( "y", &Vec<4, T>::y )
		.def_readwrite( "z", &Vec<4, T>::z )
		.def_readwrite( "w", &Vec<4, T>::w )
		.def_readwrite( "r", &Vec<4, T>::x )
		.def_readwrite( "g", &Vec<4, T>::y )
		.def_readwrite( "b", &Vec<4, T>::z )
		.def_readwrite( "a", &Vec<4, T>::w );
}

template<unsigned int DIM, typename T> luabind::scope bindVecToLua( const char* p_name )
{
	class_<Vec<DIM, T> > classDef( p_name );

	classDef
		.def( constructor<const Vec<DIM, T>&>() )
		.def( constructor<T>() )
		.property( "isZero", &Vec<DIM, T>::isZero )
		.property( "length", &Vec<DIM, T>::length )
		.property( "lengthSq", &Vec<DIM, T>::lengthSq )
		.property( "normalized", &Vec<DIM, T>::normalized )
		.def( const_self + other<const Vec<DIM, T>&>() )
		.def( const_self - other<const Vec<DIM, T>&>() )
		.def( -const_self )
		.def( const_self * T() )
		.def( T() * const_self )
		.def( const_self / T() )
		.def( const_self == other<const Vec<DIM, T>&>() )
		.def( "normalize", &Vec<DIM, T>::normalize )
		.def( "dot", &Vec<DIM, T>::operator ^ );

	bindVecDetailToLua( classDef );

	return classDef;
}

int runThread( lua_State* p_lua )
{
	Level* p_level = Scripting::inst().getLevel( p_lua );
	if( !p_level )
	{
		Scripting::inst().getConsole()->print( "SCRIPT ERROR: runThread kann nur innerhalb eines Levels aufgerufen werden!\n", Vec4f( 1.0f, 0.5f, 0.5f, 1.0f ) );
		return 0;
	}

	lua_State* p_root = Scripting::inst().getCurrentLuaState();
	lua_State* p_thread = lua_newthread( p_root );
	int ref = luaL_ref( p_root, LUA_GLOBALSINDEX );
	Scripting::inst().reference[p_thread] = ref;
	Scripting::inst().parent[p_thread] = p_root;

	// Wie viele Parameter?
	int n = lua_gettop( p_lua );

	if( !n )
	{
		// Einen Parameter brauchen wir auf jeden Fall!
		Scripting::inst().getConsole()->print( "SCRIPT ERROR: runThread erwartet mindestens einen Parameter!\n", Vec4f( 1.0f, 0.5f, 0.5f, 1.0f ) );
		return 0;
	}

	// Funktion und Parameter auf den Thread-Stack packen, damit sie dort aufgerufen werden kann
	lua_xmove( p_lua, p_thread, n );

	int ret = lua_resume( p_thread, n - 1 );
	if( ret != LUA_YIELD )
	{
		// Der Thread hat sich beendet.
		Scripting::inst().finalizeThread( p_thread );
	}

	return 0;
}

void sleep( lua_State* p_lua,
		    float numSeconds )
{
	Level* p_level = Scripting::inst().getLevel( p_lua );
	if( !p_level )
	{
		Scripting::inst().getConsole()->print( "SCRIPT ERROR: sleep kann nur innerhalb eines Levels aufgerufen werden!\n", Vec4f( 1.0f, 0.5f, 0.5f, 1.0f ) );
		return;
	}

	lua_yield( p_lua, 0 );
	p_level->addResumeCountDown( p_lua, numSeconds );
}

bool areEqual1( const GameObject* p_obj1,
				const GameObject* p_obj2 )
{
	return p_obj1 == p_obj2;
}

bool areEqual2( const ScriptObject* p_obj1,
				const ScriptObject* p_obj2 )
{
	return p_obj1 == p_obj2;
}

double randDouble_noparams()
{
	return randDouble();
}

template<typename A, typename B> B* staticCast( A* p_object )
{
	return static_cast<B*>( p_object );
}

template<typename A, typename B> B* dynamicCast( A* p_object )
{
	return dynamic_cast<B*>( p_object );
}

class CustomException : public luabind::error
{
public:
	CustomException( const std::string& msg ) : luabind::error( 0 ),
												msg( msg )
	{
		Scripting::inst().getConsole()->print( std::string( "SCRIPT ERROR: " ) + msg + "\n", Vec4f( 1.0f, 0.5f, 0.5f, 1.0f ) );
	}

	const char* what() const
	{
		return msg.c_str();
	}

private:
	const std::string msg;
};

struct SoundInstanceWrapper
{
	Vec2f getPosition() const							{ return p_soundInstance->getPosition(); }
	void setPosition( const Vec2f& position )			{ p_soundInstance->setPosition( position ); }
	Vec2f getAbsPosition() const						{ return p_soundInstance->getAbsPosition(); }
	void setAbsPosition( const Vec2f& position )		{ p_soundInstance->setAbsPosition( position ); }
	float getVolume() const								{ return p_soundInstance->getVolume(); }
	void setVolume( float volume )						{ p_soundInstance->setVolume( volume ); }
	float getPitch() const								{ return p_soundInstance->getPitch(); }
	void setPitch( float pitch )						{ p_soundInstance->setPitch( pitch ); }
	bool isLooping() const								{ return p_soundInstance->isLooping(); }
	void setLooping( bool looping )						{ p_soundInstance->setLooping( looping ); }
	bool isMuted() const								{ return p_soundInstance->isMuted(); }
	void setMuted( bool muted )							{ p_soundInstance->setMuted( muted ); }
	bool isPaused() const								{ return p_soundInstance->isPaused(); }
	void setPaused( bool paused )						{ p_soundInstance->setPaused( paused ); }
	float getPlayCursor() const							{ return p_soundInstance->getPlayCursor(); }
	void setPlayCursor( float seconds )					{ p_soundInstance->setPlayCursor( seconds ); }
	float getMinDistance() const						{ float dist; p_soundInstance->getDistances( &dist, 0 ); return dist; }
	void setMinDistance( float dist )					{ p_soundInstance->setDistances( dist, getMaxDistance() ); }
	float getMaxDistance() const						{ float dist; p_soundInstance->getDistances( 0, &dist ); return dist; }
	void setMaxDistance( float dist )					{ p_soundInstance->setDistances( getMinDistance(), dist ); }
	bool getCalculateVelocity() const					{ return p_soundInstance->getCalculateVelocity(); }
	void setCalculateVelocity( bool enable )			{ p_soundInstance->setCalculateVelocity( enable ); }
	void play()											{ p_soundInstance->play(); }
	void stop()											{ p_soundInstance->stop(); p_soundInstance = 0; }
	void attachTo( GameObject* p_obj )					{ p_soundInstance->attachTo( p_obj, false ); }
	void attachTo2( GameObject* p_obj, bool mntn )		{ p_soundInstance->attachTo( p_obj, mntn ); }
	void detachFromParent()								{ p_soundInstance->detachFromParent( false ); }
	void detachFromParent2( bool mntn )					{ p_soundInstance->detachFromParent( mntn ); }
	void setStopCountDown( float time )					{ p_soundInstance->setStopCountDown( time ); }
	void setPauseCountDown( float time )				{ p_soundInstance->setPauseCountDown( time ); }
	void setVolumeTarget( float volume, float speed )	{ p_soundInstance->setVolumeTarget( volume, speed ); }
	void setPitchTarget( float pitch, float speed )		{ p_soundInstance->setPitchTarget( pitch, speed ); }

	SoundHandle sound;
	SoundInstance* p_soundInstance;
};

struct SoundWrapper
{
	SoundInstanceWrapper* createInstance()
	{
		SoundInstance* p_inst = sound->createInstance();
		if( !p_inst ) return 0;

		SoundInstanceWrapper* w = new SoundInstanceWrapper;
		w->sound = sound;
		w->p_soundInstance = p_inst;

		return w;
	}

	SoundInstanceWrapper* createInstance2( bool forceCreation )
	{
		SoundInstance* p_inst = sound->createInstance( true );
		if( !p_inst ) return 0;

		SoundInstanceWrapper* w = new SoundInstanceWrapper;
		w->sound = sound;
		w->p_soundInstance = p_inst;

		return w;
	}

	float getMinDistance() const					{ float dist; sound->getDefaultDistances( &dist, 0 ); return dist; }
	void setMinDistance( float dist )				{ sound->setDefaultDistances( dist, getMaxDistance() ); }
	float getMaxDistance() const					{ float dist; sound->getDefaultDistances( 0, &dist ); return dist; }
	void setMaxDistance( float dist )				{ sound->setDefaultDistances( getMinDistance(), dist ); }
	float getInstanceCreationInterval() const		{ return sound->getInstanceCreationInterval(); }
	void setInstanceCreationInterval( float v )		{ sound->setInstanceCreationInterval( v ); }
	unsigned int getMaxPlayingInstances() const		{ return sound->getMaxPlayingInstances(); }
	void setMaxPlayingInstances( unsigned int v )	{ sound->setMaxPlayingInstances( v ); }

	SoundHandle sound;
};

SoundWrapper* loadSound( const std::string& filename,
						 const std::string& soundGroup,
						 const std::string& type,
						 const std::string& mode )
{
	SoundWrapper* w = new SoundWrapper;
	w->sound = 0;

	SoundManager& soundMgr = Scripting::inst().getConsole()->getEngine().getSoundManager();
	ParameterMap options;
	options.set<std::string>( "Type", type );
	options.set<std::string>( "Mode", mode );
	SoundHandle sound = soundMgr.get( filename, false, options );
	if( !sound ) return 0;

	sound->setDefaultSoundGroupName( soundGroup );
	w->sound = sound;

	return w;
}

SoundInstanceWrapper* playSound( const std::string& filename,
								 float volume = 1.0f,
								 float pitch = 1.0f )
{
	SoundManager& soundMgr = Scripting::inst().getConsole()->getEngine().getSoundManager();
	ParameterMap options;
	options.set<std::string>( "Type", "Sample" );
	options.set<std::string>( "Mode", "2D" );
	SoundHandle sound = soundMgr.get( filename, false, options );
	if( !sound ) return 0;

	SoundInstance* p_inst = sound->createInstance();
	if( !p_inst ) return 0;

	SoundInstanceWrapper* w = new SoundInstanceWrapper;
	w->p_soundInstance = p_inst;
	w->sound = sound;

	p_inst->play();

	return w;
}

SoundInstanceWrapper* playSound2( const std::string& filename )
{
	return playSound( filename );
}

SoundInstanceWrapper* playMusic2( const std::string& filename,
								  bool restartIfAlreadyPlaying,
								  float fadeInSpeed,
								  float fadeOutSpeed )
{
	Audio& audio = Scripting::inst().getConsole()->getEngine().getAudio();

	SoundInstance* p_inst = audio.playMusic( filename, restartIfAlreadyPlaying, fadeInSpeed, fadeOutSpeed );
	if( !p_inst ) return 0;

	SoundInstanceWrapper* w = new SoundInstanceWrapper;
	w->p_soundInstance = p_inst;
	w->sound = p_inst->getSound();

	return w;
}

SoundInstanceWrapper* playMusic( const std::string& filename )
{
	return playMusic2( filename, false, 1.0f, 1.0f );
}

class Dummy1 {};
class Dummy2 {};

int luaPrint (lua_State *L) {
  Console* p_console = Scripting::inst().getConsole();
  int n = lua_gettop(L);  /* number of arguments */
  int i;
  lua_getglobal(L, "tostring");
  for (i=1; i<=n; i++) {
    const char *s;
    lua_pushvalue(L, -1);  /* function to be called */
    lua_pushvalue(L, i);   /* value to print */
    lua_call(L, 1, 1);
    s = lua_tostring(L, -1);  /* get result */
    if (s == NULL)
      return luaL_error(L, LUA_QL("tostring") " must return a string to "
                           LUA_QL("print"));
    if (i>1) p_console->print( "\t" );
    p_console->print( s );
    lua_pop(L, 1);  /* pop result */
  }
  p_console->print( "\n" );
  return 0;
}

int luaError( lua_State* p_lua )
{
	int n = lua_gettop( p_lua );

	std::ostringstream text;
	for( int i = 1; i <= n; i++ )
	{
		const char* p_str = static_cast<const char*>( lua_tostring( p_lua, i ) );
		if( p_str )
		{
			text << p_str;
			if( i != n ) text << "\t";
		}
	}

	lua_settop( p_lua, 0 );

	const std::string msg( text.str() );
	Console* p_console = Scripting::inst().getConsole();
	if( p_console ) p_console->print( std::string( "LUA ERROR: " ) + msg + "\n", Vec4f( 1.0f, 0.5f, 0.5f, 1.0f ) );

	return 0;
}

Scripting::Scripting() : p_rootLua( 0 )
{
	p_rootLua = createInterpreter();
}

Scripting::Scripting( const Scripting& copy )
{
}

Scripting::~Scripting()
{
	exit();
}

Scripting& Scripting::inst()
{
	static Scripting inst;
	return inst;
}

void Scripting::init( Console* p_console )
{
	this->p_console = p_console;
}

void Scripting::exit()
{
	if( p_rootLua )
	{
		lua_close( p_rootLua );
		p_rootLua = 0;
	}
}

lua_State* Scripting::createInterpreter()
{
	// lua-Umgebung erzeugen
	lua_State* p_lua = lua_open();
	luaL_openlibs( p_lua );
	lua_register( p_lua, "runThread", runThread );
	lua_register( p_lua, "print", luaPrint );
	lua_register( p_lua, "error", luaError );
	lua_register( p_lua, "_ALERT", luaError );
	lua_register( p_lua, "_ERRORMESSAGE", luaError );
	bindToLua( p_lua );
	return p_lua;
}

void Scripting::executeScript( lua_State* p_lua,
							   const std::string& code )
{
	if( code.empty() ) return;

	if( !p_lua ) p_lua = luaStates.empty() ? p_rootLua : luaStates.top();

	try
	{
		if( luaL_dostring( p_lua, code.c_str() ) ) luaError( p_lua );
	}
	catch( luabind::error& err )
	{
		printException( err );
	}
}

void Scripting::call( luabind::object function )
{
	if( !function || luabind::type( function ) != LUA_TFUNCTION ) return;

	try
	{
		luabind::call_function<void>( function );
	}
	catch( luabind::error& err )
	{
		printException( err );
	}
}

void Scripting::print( const std::string& text )
{
	lua_getglobal( p_rootLua, "print" );
	lua_pushstring( p_rootLua, text.c_str() );
	lua_pcall( p_rootLua, 1, 0, 0 );
}

void Scripting::luaInfo()
{
	lua_State* p_lua = luaStates.empty() ? p_rootLua : luaStates.top();

	int n = lua_gettop( p_lua );
	std::ostringstream msg;
	msg << "Stack: " << n;

	print( msg.str() );
}

Level* Scripting::getLevel( lua_State* p_lua )
{
	try
	{
		Level* p_level = luabind::object_cast<Level*>( luabind::globals( p_lua )["level"] );
		return p_level;
	}
	catch( luabind::error& )
	{
		return 0;
	}
}

void Scripting::resumeThread( lua_State* p_lua )
{
	int ret = lua_resume( p_lua, 0 );
	if( ret != LUA_YIELD )
	{
		// Der Thread hat sich beendet.
		finalizeThread( p_lua );
	}
}

void Scripting::finalizeThread( lua_State* p_lua )
{
	lua_State* p_parent = parent[p_lua];
	luaL_unref( p_parent, LUA_GLOBALSINDEX, reference[p_lua] );
}

void Scripting::pushCurrentLuaState( lua_State* p_currentLuaState )
{
	luaStates.push( p_currentLuaState );
	p_console->print( "Kontext gewechselt\n", Vec4f( 1.0f, 0.5f, 0.25, 1.0f ) );
}

void Scripting::popCurrentLuaState()
{
	if( !luaStates.empty() )
	{
		luaStates.pop();
		p_console->print( "Kontext gewechselt\n", Vec4f( 1.0f, 0.5f, 0.25, 1.0f ) );
	}
}

lua_State* Scripting::getLua() const
{
	return p_rootLua;
}

lua_State* Scripting::getCurrentLuaState() const
{
	if( luaStates.empty() ) return p_rootLua;
	else return luaStates.top();
}

Console* Scripting::getConsole() const
{
	return p_console;
}

void Scripting::printException( const luabind::error& err )
{
	if( p_console )
	{
		std::ostringstream msg;
		msg << "LUABIND EXCEPTION: " << err.what() << std::endl;
		p_console->print( msg.str(), Vec4f( 1.0f, 0.5f, 0.5f, 1.0f ) );
	}
}

void Scripting::bindToLua( lua_State* p_lua )
{
	open( p_lua );

	module( p_lua )
	[
		def( "sleep", &sleep, raw( _1 ) ),

		def( "areEqual", &areEqual1 ),
		def( "areEqual", &areEqual2 ),
		def( "randInt", static_cast<int ( * )( int, int )>( &randInt ) ),
		def( "randInt", static_cast<unsigned int ( * )()>( &randInt ) ),
		def( "randReal", &randDouble_noparams ),
		def( "randReal", &randDouble ),
		def( "randDir", &randVec2fDir ),
		def( "beginColor", static_cast<std::string ( * )( const Vec4f& )>( &beginColor ) ),
		def( "beginColor", static_cast<std::string ( * )( float, float, float, float )>( &beginColor ) ),
		def( "endColor", &endColor ),

		def( "toDamagable", &dynamicCast<GameObject, Damagable> ),
		def( "toArmed", &dynamicCast<GameObject, Armed> ),

		def( "toPlayer", &dynamicCast<GameObject, Obj_Player> ),
		def( "toEnemy", &dynamicCast<GameObject, Obj_Enemy> ),
		def( "toLaser", &dynamicCast<GameObject, Obj_Laser> ),
		def( "toEnergyNode", &dynamicCast<GameObject, Obj_EnergyNode> ),
		def( "toLevelTarget", &dynamicCast<GameObject, Obj_LevelTarget> ),
		def( "toPowerUp", &dynamicCast<GameObject, Obj_PowerUp> ),
		def( "toCheckpoint", &dynamicCast<GameObject, Obj_Checkpoint> ),
		def( "toProjectile", &dynamicCast<GameObject, Obj_Projectile> ),
		def( "toGarage", &dynamicCast<GameObject, Obj_Garage> ),
		def( "toMine", &dynamicCast<GameObject, Obj_Mine> ),
		def( "toAccelerator", &dynamicCast<GameObject, Obj_Accelerator> ),

		def( "toCounter", &dynamicCast<ScriptObject, ScriptObj_Counter> ),

		def( "loadSound", &loadSound, adopt( result ) ),
		def( "playSound", &playSound, adopt( result ) ),
		def( "playSound", &playSound2, adopt( result ) ),
		def( "playMusic", &playMusic, adopt( result ) ),
		def( "playMusic", &playMusic2, adopt( result ) ),
		def( "getPathFilename", &getPathFilename ),
		def( "getPathDirectory", &getPathDirectory ),

		bindVecToLua<2, float>( "Vec2r" ),
		bindVecToLua<3, float>( "Vec3r" ),
		bindVecToLua<4, float>( "Vec4r" ),

		class_<Dummy1>( "CollisionBits" )
			.enum_( "constants" )
			[
				value( "None", CB_NONE ),
				value( "Static", CB_STATIC ),
				value( "Player", CB_PLAYER ),
				value( "Enemy", CB_ENEMY ),
				value( "Projectile", CB_PROJECTILE ),
				value( "Laser", CB_LASER ),
				value( "Object", CB_OBJECT ),
				value( "BlockView", CB_BLOCK_VIEW ),
				value( "All", CB_ALL )
			],

		class_<Dummy2>( "Weapon" )
			.enum_( "constants" )
			[
				value( "BlasterBlue", WT_INK_BLASTER ),
				value( "Missile", WT_MISSILE ),
				value( "Cannon", WT_CANNON ),
				value( "AutoGun", WT_AUTO_GUN ),
				value( "BlasterGreen", WT_INK_BLASTER_2 ),
				value( "AA", WT_AA )
			],

		class_<SoundInstanceWrapper>( "SoundInstance" )
			.property( "position", &SoundInstanceWrapper::getPosition, &SoundInstanceWrapper::setPosition )
			.property( "absPosition", &SoundInstanceWrapper::getAbsPosition, &SoundInstanceWrapper::setAbsPosition )
			.property( "volume", &SoundInstanceWrapper::getVolume, &SoundInstanceWrapper::setVolume )
			.property( "pitch", &SoundInstanceWrapper::getPitch, &SoundInstanceWrapper::setPitch )
			.property( "looping", &SoundInstanceWrapper::isLooping, &SoundInstanceWrapper::setLooping )
			.property( "muted", &SoundInstanceWrapper::isMuted, &SoundInstanceWrapper::setMuted )
			.property( "paused", &SoundInstanceWrapper::isPaused, &SoundInstanceWrapper::setPaused )
			.property( "playCursor", &SoundInstanceWrapper::getPlayCursor, &SoundInstanceWrapper::setPlayCursor )
			.property( "minDistance", &SoundInstanceWrapper::getMinDistance, &SoundInstanceWrapper::setMinDistance )
			.property( "maxDistance", &SoundInstanceWrapper::getMaxDistance, &SoundInstanceWrapper::setMaxDistance )
			.property( "calculateVelocity", &SoundInstanceWrapper::getCalculateVelocity, &SoundInstanceWrapper::setCalculateVelocity )
			.def( "play", &SoundInstanceWrapper::play )
			.def( "stop", &SoundInstanceWrapper::stop )
			.def( "attachTo", &SoundInstanceWrapper::attachTo )
			.def( "attachTo", &SoundInstanceWrapper::attachTo2 )
			.def( "detachFromParent", &SoundInstanceWrapper::detachFromParent )
			.def( "detachFromParent", &SoundInstanceWrapper::detachFromParent2 )
			.def( "setStopCountDown", &SoundInstanceWrapper::setStopCountDown )
			.def( "setPauseCountDown", &SoundInstanceWrapper::setPauseCountDown )
			.def( "setVolumeTarget", &SoundInstanceWrapper::setVolumeTarget )
			.def( "setPitchTarget", &SoundInstanceWrapper::setPitchTarget ),

		class_<SoundWrapper>( "Sound" )
			.property( "defaultMinDistance", &SoundWrapper::getMinDistance, &SoundWrapper::setMinDistance )
			.property( "defaultMaxDistance", &SoundWrapper::getMaxDistance, &SoundWrapper::setMaxDistance )
			.property( "instanceCreationInterval", &SoundWrapper::getInstanceCreationInterval, &SoundWrapper::setInstanceCreationInterval )
			.property( "maxPlayingInstances", &SoundWrapper::getMaxPlayingInstances, &SoundWrapper::setMaxPlayingInstances )
			.def( "createInstance", &SoundWrapper::createInstance, adopt( result ) )
			.def( "createInstance", &SoundWrapper::createInstance2, adopt( result ) ),

		class_<GameObject>( "Object" )
			.property( "id", &GameObject::getID )
			.property( "type", &GameObject::getType )
			.property( "zombie", &GameObject::isZombie )
			.property( "turnToZombie", &GameObject::getTurnToZombie, &GameObject::setTurnToZombie )
			.property( "position", &GameObject::getPosition, &GameObject::setPosition )
			.property( "angle", &GameObject::getAngle, &GameObject::setAngle )
			.property( "direction", &GameObject::getDirection, &GameObject::setDirection )
			.property( "velocity", &GameObject::getVelocity, &GameObject::setVelocity )
			.property( "angularVelocity", &GameObject::getAngularVelocity, &GameObject::setAngularVelocity )
			.property( "mass", &GameObject::getTotalMass, &GameObject::setTotalMass )
			.property( "gravityInfluence", &GameObject::getGravityInfluence, &GameObject::setGravityInfluence )
			.property( "movementFriction", &GameObject::getMovementFriction, &GameObject::setMovementFriction )
			.property( "rotationFriction", &GameObject::getRotationFriction, &GameObject::setRotationFriction )
			.property( "material", &GameObject::getMaterial, &GameObject::setMaterial )
			.property( "visible", &GameObject::isVisible, &GameObject::setVisible )
			.property( "frozen", &GameObject::isFrozen, &GameObject::setFrozen )
			.property( "passThrough", &GameObject::getPassThrough, &GameObject::setPassThrough )
			.property( "target", &GameObject::getTarget, &GameObject::setTarget )
			.property( "playerCollisionDamageScale", &GameObject::getPlayerCollisionDamageScale, &GameObject::setPlayerCollisionDamageScale )
			.property( "categoryBits", &GameObject::getCategoryBits, &GameObject::setCategoryBits )
			.property( "collideBits", &GameObject::getCollideBits, &GameObject::setCollideBits )
			.property( "onCollision", &GameObject::getCollisionCallback, &GameObject::setCollisionCallback )
			.property( "onKilled", &GameObject::getOnKilled, &GameObject::setOnKilled )
			.property( "onReanimated", &GameObject::getOnReanimated, &GameObject::setOnReanimated )
			.def( "setCollisionEnabled", &GameObject::setCollisionEnabled )
			.def( "isCollisionEnabled", &GameObject::isCollisionEnabled )
			.def( "relToAbsPos", &GameObject::relPositionToAbsPosition )
			.def( "absToRelPos", &GameObject::absPositionToRelPosition )
			.def( "relToAbsAngle", &GameObject::relAngleToAbsAngle )
			.def( "absToRelAngle", &GameObject::absAngleToRelAngle )
			.def( "relToAbsDir", &GameObject::relDirectionToAbsDirection )
			.def( "absToRelDir", &GameObject::absDirectionToRelDirection )
			.def( "kill", &GameObject::kill )
			.def( "reanimate", &GameObject::rebirth )
			.def( "explode", &GameObject::explode_lua )
			.def( "canSee", &GameObject::canSee )
			.def( "canSee", &GameObject::canSee_lua ),

		class_<Damagable>( "Damagable" )
			.property( "health", &Damagable::getHealth, &Damagable::setHealth )
			.property( "maxHealth", &Damagable::getMaxHealth, &Damagable::setMaxHealth )
			.property( "invincible", &Damagable::isInvincible, &Damagable::setInvincible ),

		class_<Armed>( "Armed" )
			.property( "weapon", &Armed::getCurrentWeapon, &Armed::selectWeapon )
			.def( "setupWeapon", &Armed::setupWeapon_lua )
			.def( "setAmmo", &Armed::setAmmo )
			.def( "getAmmo", &Armed::getAmmo )
			.def( "fire", &Armed::fire )
			.def( "fireAt", &Armed::fireAt ),

		class_<Obj_Player, bases<GameObject, Damagable, Armed> >( "Player" )
			.property( "score", &Obj_Player::getScore, &Obj_Player::setScore )
			.property( "fuel", &Obj_Player::getFuel, &Obj_Player::setFuel )
			.property( "shield", &Obj_Player::getShield, &Obj_Player::setShield )
			.property( "mouseAiming", &Obj_Player::getMouseAiming, &Obj_Player::setMouseAiming ),

		class_<Obj_Enemy, bases<GameObject, Damagable, Armed> >( "Enemy" ),

		class_<Obj_Laser, bases<GameObject> >( "Laser" )
			.property( "active", &Obj_Laser::isActive, &Obj_Laser::setActive )
			.property( "powered", &Obj_Laser::isPowered ),

		class_<Obj_EnergyNode, bases<GameObject, Damagable> >( "EnergyNode" ),

		class_<Obj_LevelTarget, bases<GameObject, Damagable> >( "LevelTarget" ),

		class_<Obj_PowerUp, bases<GameObject> >( "PowerUp" )
			.property( "subType", &Obj_PowerUp::getSubType )
			.property( "reward", &Obj_PowerUp::getReward, &Obj_PowerUp::setReward )
			.property( "shieldDuration", &Obj_PowerUp::getShieldDuration, &Obj_PowerUp::setShieldDuration ),

		class_<Obj_Checkpoint, bases<GameObject> >( "Checkpoint" )
			.property( "activated", &Obj_Checkpoint::isActivated )
			.property( "activationTime", &Obj_Checkpoint::getActivationTime )
			.property( "number", &Obj_Checkpoint::getNumber, &Obj_Checkpoint::setNumber )
			.property( "shieldDuration", &Obj_Checkpoint::getShieldDuration, &Obj_Checkpoint::setShieldDuration )
			.property( "healthAmount", &Obj_Checkpoint::getHealthAmount, &Obj_Checkpoint::setHealthAmount )
			.property( "fuelAmount", &Obj_Checkpoint::getFuelAmount, &Obj_Checkpoint::setFuelAmount ),

		class_<Obj_Projectile, bases<GameObject> >( "Projectile" )
			.property( "subType", &Obj_Projectile::getProjectileType )
			.property( "owner", &Obj_Projectile::getOwner )
			.property( "timeToLive", &Obj_Projectile::getTimeToLive, &Obj_Projectile::setTimeToLive ),

		class_<Obj_Garage, bases<GameObject> >( "Garage" )
			.property( "repairPrice", &Obj_Garage::getRepairPrice, &Obj_Garage::setRepairPrice )
			.property( "refuelPrice", &Obj_Garage::getRefuelPrice, &Obj_Garage::setRefuelPrice )
			.property( "repairCapacity", &Obj_Garage::getRepairCapacity, &Obj_Garage::setRepairCapacity )
			.property( "refuelCapacity", &Obj_Garage::getRefuelCapacity, &Obj_Garage::setRefuelCapacity ),

		class_<Obj_Mine, bases<GameObject> >( "Mine" )
			.def( "detonate", &Obj_Mine::detonate )
			.property( "activated", &Obj_Mine::isActivated, &Obj_Mine::setActivated ),

		class_<Obj_Accelerator, bases<GameObject> >( "Accelerator" )
			.property( "radius", &Obj_Accelerator::getRadius )
			.property( "targetSpeed", &Obj_Accelerator::getTargetSpeed, &Obj_Accelerator::setTargetSpeed )
			.property( "strength", &Obj_Accelerator::getStrength, &Obj_Accelerator::setStrength )
			.property( "activated", &Obj_Accelerator::isActivated, &Obj_Accelerator::setActivated ),

		class_<ScriptObject>( "ScriptObject" )
			.property( "type", &ScriptObject::getType )
			.def( "kill", &ScriptObject::kill ),

		class_<ScriptObj_Counter, bases<ScriptObject> >( "Counter" )
			.property( "displayMode", &ScriptObj_Counter::getDisplayMode, &ScriptObj_Counter::setDisplayMode )
			.property( "direction", &ScriptObj_Counter::getDirection, &ScriptObj_Counter::setDirection )
			.property( "duration", &ScriptObj_Counter::getDuration, &ScriptObj_Counter::setDuration )
			.property( "onElapsed", &ScriptObj_Counter::getOnElapsed, &ScriptObj_Counter::setOnElapsed )
			.property( "time", &ScriptObj_Counter::getTime, &ScriptObj_Counter::setTime )
			.property( "paused", &ScriptObj_Counter::isPaused, &ScriptObj_Counter::setPaused )
			.property( "color", &ScriptObj_Counter::getColor, &ScriptObj_Counter::setColor )
			.property( "numBeeps", &ScriptObj_Counter::getNumBeeps, &ScriptObj_Counter::setNumBeeps ),

		class_<Level>( "Level" )
			.property( "filename", &Level::getFilename )
			.property( "name", &Level::getName )
			.property( "type", &Level::getType )
			.property( "description", &Level::getDescription )
			.property( "gravity", &Level::getGravity, &Level::setGravity )
			.property( "time", &Level::getTime )
			.property( "raceTime", &Level::getRaceTime )
			.property( "cameraTracking", &Level::isCameraTrackingEnabled, &Level::setCameraTrackingEnabled )
			.property( "cameraTarget", &Level::getCameraTarget, &Level::setCameraTarget_lua )
			.property( "cameraPosition", &Level::getCamera, &Level::setCamera )
			.property( "cameraZoom", &Level::getCameraZoom, &Level::setCameraZoom )
			.property( "cameraAngle", &Level::getCameraAngle, &Level::setCameraAngle )
			.property( "objects", &Level::getObjectsSet, return_stl_iterator )
			.property( "visibleObjects", &Level::getVisibleObjects, return_stl_iterator )
			.def( "getObject", &Level::getObjectByID )
			.def( "addPlayerScore", &Level::addPlayerScore_lua )
			.def( "addPlayerScore", &Level::addPlayerScore_lua2 )
			.def( "addText", &Level::addText )
			.def( "addSMS", &Level::addSMS )
			.def( "addTimerCallback", &Level::addTimerCallback )
			.def( "removeTimerCallback", &Level::removeTimerCallback )
			.def( "createCounter", &Level::createCounter )
	];
}